{------------------------------------------------------------------------------}
{ Copyright  2000-2003, Simon Armstrong.  All Rights Reserved.                }
{                                                                              }
{ Copyright:                                                                   }
{ All SadMan Software source code is copyrighted by Simon Armstrong (hereafter }
{ "author"), and shall remain the exclusive property of the author. This       }
{ software and any accompanying documentation are protected by United Kingdom  }
{ copyright law and also by International Treaty provisions. Any use of this   }
{ software in violation of copyright law or the terms of this agreement will   }
{ be prosecuted to the best of the author's ability.                           }
{                                                                              }
{ Under no conditions may you remove the copyright notices made part of the    }
{ software or documentation.                                                   }
{                                                                              }
{ Distribution Rights:                                                         }
{ You are granted a non-exlusive, royalty-free right to produce and distribute }
{ compiled binary files (executables, DLLs, etc.) that are built with any of   }
{ the source code unless specifically stated otherwise.                        }
{ You are further granted permission to redistribute any of the source code in }
{ source code form, provided that the original archive as found on the         }
{ SadMan Software web site (http://www.simes.clara.co.uk) is distributed       }
{ unmodified.                                                                  }
{                                                                              }
{ Restrictions:                                                                }
{ Without the express written consent of the author, you may not:              }
{   * Distribute modified versions of any SadMan Software source code by       }
{     itself. You must include the original archive as you found it at the web }
{     site.                                                                    }
{   * Sell or lease any portion of SadMan Software source code. You are, of    }
{     course, free to sell any of your own original code that works with,      }
{     enhances, etc. SadMan Software source code.                              }
{   * Distribute SadMan Software source code for profit.                       }
{                                                                              }
{ Warranty:                                                                    }
{ There is absolutely no warranty of any kind whatsoever with any of this      }
{ source code (hereafter "software"). The software is provided to you "AS-IS", }
{ and all risks and losses associated with it's use are assumed by you. In no  }
{ event shall the author of the softare, Simon Armstrong, be held accountable  }
{ for any damages or losses that may occur from use or misuse of the software. }
{                                                                              }
{ Support:                                                                     }
{ Support is provided via email to Support@simes.clara.co.uk. However, I can   }
{ not guarantee any support whatsoever. While I do try to answer all questions }
{ that I receive and address all problems that are reported to me, you must    }
{ understand that I simply can not guarantee that this will always be so.      }
{------------------------------------------------------------------------------}
unit SsExpEval;

{$D-} { debug information             }
{$I+} { I/O checking                  }
{$L-} { local symbol information      }
{$Q+} { arithmetic overflow checking  }
{$R+} { range checking                }
{$S+} { stack overflow checking       }
{$Y-} { symbol reference information  }

interface

uses
  {$IFNDEF VER80}
  Math,
  {$ENDIF}
  Classes, SysUtils;

type
  EBadName = class(Exception);
  ENoSuchName = class(Exception);
  ENoSuchIndex = class(Exception);

  TFunctionCode = (fnUnknown,
    {$IFNDEF VER80}
    fnArcCos,
    fnArcCosD,
    fnArcCosh,
    fnArcSin,
    fnArcSinD,
    fnArcSinh,
    fnArcTanh,
    fnCosh,
    fnCotan,
    fnLog10,
    fnLog2,
    fnLn,
    fnSinh,
    fnGradToRad,
    fnRadToGrad,
    fnCeil,
    fnFloor,
    {$ENDIF}
    fnArcTan,
    fnArcTanD,
    fnCos,
    fnCosD,
    fnExp,
    fnRound,
    fnSin,
    fnSinD,
    fnSqr,
    fnSqrt,
    fnTan,
    fnTanD,
    fnTanh,
    fnTrunc,
    fnDegToRad,
    fnRadToDeg);

  TGetValueEvent = procedure(Sender: TObject; Identifier: string; var Value: extended; var Undefined: boolean) of object;
  EExpressionError = class(Exception);

  TNamedValue = class(TObject)
  private
    FName: string;
    FValue: extended;
  public
    constructor Create(AName: string; AValue: extended);
    property Name: string read FName;
    property Value: extended read FValue write FValue;
  end;

  TNamedValues = class(TPersistent)
  private
    FCaseSensitive: boolean;
    FOnChange: TNotifyEvent;
    FUpdateCount: integer;
    FValueList: TList;
    function GetCount: integer;
    function GetValueByIndex(const index: integer): extended;
    function GetValueByName(const aName: string): extended;
    function GetValues(const index: integer): TNamedValue;
    procedure ReadData(Reader: TReader);
    procedure WriteData(Writer: TWriter);
  protected
    procedure Changed;
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(const aName: string; const aValue: extended);
    procedure Assign(Source: TPersistent); override;
    procedure BeginUpdate;
    procedure Clear;
    procedure Delete(const aName: string);
    procedure EndUpdate;
    class function IsValidName(const aName: string): boolean;
    function IndexOf(const aName: string): integer;
    property Count: integer read GetCount;
    property ValueByIndex[const index: integer]: extended read GetValueByIndex;
    property ValueByName[const aName: string]: extended read GetValueByName;
    property Values[const index: integer]: TNamedValue read GetValues;
  published
    property CaseSensitive: boolean read FCaseSensitive write FCaseSensitive default false;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TExpressionEvaluator = class(TComponent)
  private
    FCaseSensitive: boolean;
    FExpression: string;
    FLook: char;
    FNamedValues: TNamedValues;
    FOnChange: TNotifyEvent;
    FOnGetValue: TGetValueEvent;
    FPtr: integer;
    FTempVariables: TNamedValues; { in case a user-defined variable is used more than once in an expression }
    procedure Error(const s: string);
    function EvalExpression: extended;
    function EvalFunction(const fn: TFunctionCode; const arg: extended): extended;
    procedure Expected(const s: string);
    function Factor: extended;
    procedure GetChar;
    function GetFunctionCode(const str: string): TFunctionCode;
    function GetName: string;
    function GetNumber: extended;
    function GetValue: extended;
    function IsAlpha(const c: char): boolean;
    function IsAlphaNumeric(const c: char): boolean;
    function IsDigit(const c: char): boolean;
    procedure Match(const x: char);
    procedure SetCaseSensitive(const v: boolean);
    procedure SetExpression(const e: string);
    procedure SetNamedValues(const nv: TNamedValues);
    procedure SkipWhitespace;
    function Term: extended;
  protected
    procedure Changed;
    procedure Loaded; override;
  public
    function IsBuiltIn(const FuncName: string): boolean;
    property Value: extended read GetValue;
  published
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property CaseSensitive: boolean read FCaseSensitive write SetCaseSensitive default false;
    property Expression: string read FExpression write SetExpression;
    property NamedValues: TNamedValues read FNamedValues write SetNamedValues;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnGetValue: TGetValueEvent read FOnGetValue write FOnGetValue;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SadMan', [TExpressionEvaluator]);
end;

{$R *.r32}

{******************************************************************************}
{ TNamedValue }

constructor TNamedValue.Create(AName: string; AValue: extended);
begin
  inherited Create;
  FName := AName;
  FValue := AValue;
end;

{******************************************************************************}
{ TNamedValues }

constructor TNamedValues.Create;
begin
  inherited Create;
  FValueList := TList.Create;
end;

destructor TNamedValues.Destroy;
begin
  Clear;
  FValueList.Free;
  inherited Destroy;
end;

procedure TNamedValues.Assign(Source: TPersistent);
var
  i: integer;
  nv: TNamedValue;
begin
  if Source = Self then exit;
  if Source is TNamedValues then begin
    BeginUpdate;
    try
      FCaseSensitive := TNamedValues(Source).CaseSensitive;
      Clear;
      for i := 0 to TNamedValues(Source).Count - 1 do begin
        nv := TNamedValues(Source).Values[i];
        Add(nv.Name, nv.Value);
      end;
    finally
      EndUpdate;
    end;
  end else
    inherited Assign(Source);
end;

procedure TNamedValues.Add(const aName: string; const aValue: extended);
var
  i: integer;
begin
  if not IsValidName(aName) then
    raise EBadName.Create(aName + ' is not a valid value name');

  BeginUpdate;
  try
    i := IndexOf(aName);
    if i <> -1 then Delete(aName);
    FValueList.Add(TNamedValue.Create(aName, aValue));
  finally
    EndUpdate;
  end;
end;

procedure TNamedValues.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TNamedValues.Changed;
begin
  if FUpdateCount <> 0 then exit;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TNamedValues.Clear;
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    TNamedValue(FValueList[i]).Free;
  FValueList.Clear;
  Changed;
end;

procedure TNamedValues.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Values', ReadData, WriteData, Count > 0);
end;

procedure TNamedValues.Delete(const aName: string);
var
  i: integer;
begin
  i := IndexOf(aName);
  if i = -1 then
    raise ENoSuchName.Create('No value with name "' + aName + '"');
  TNamedValue(FValueList.Items[i]).Free;
  FValueList.Delete(i);
  FValueList.Pack;
  Changed;
end;

procedure TNamedValues.EndUpdate;
begin
  if FUpdateCount > 0 then Dec(FUpdateCount);
  if FUpdateCount = 0 then Changed;
end;

function TNamedValues.GetCount: integer;
begin
  result := FValueList.Count;
end;

function TNamedValues.GetValueByIndex(const index: integer): extended;
begin
  result := Values[index].Value;
end;

function TNamedValues.GetValueByName(const aName: string): extended;
var
  i: integer;
begin
  i := IndexOf(aName);
  if i = -1 then
    raise ENoSuchName.Create('No value with name "' + aName + '"');
  result := ValueByIndex[i];
end;

function TNamedValues.GetValues(const index: integer): TNamedValue;
begin
  if (0 <= index) and (index < Count) then
    result := TNamedValue(FValueList.Items[index])
  else
    raise ENoSuchIndex.Create('Invalid index ' + IntToStr(index));
end;

function TNamedValues.IndexOf(const aName: string): integer;
begin
  for result := 0 to Count - 1 do
    with TNamedValue(FValueList.Items[result]) do
      if (Name = aName) or
        (not FCaseSensitive and (CompareText(Name, aName) = 0)) then
        exit;
  result := -1;
end;

class function TNamedValues.IsValidName(const aName: string): boolean;
var
  i: integer;
begin
  result := false;
  for i := 1 to Length(aName) do
    if not (aName[i] in ['A'..'Z', 'a'..'z', '0'..'9', '_']) then
      exit;
  result := true;
end;

procedure TNamedValues.ReadData(Reader: TReader);
var
  n: string;
  v: extended;
begin
  Clear;
  Reader.ReadListBegin;
  while not Reader.EndOfList do begin
    n := Reader.ReadString;
    v := Reader.ReadFloat;
    Add(n, v);
  end;
  Reader.ReadListEnd;
end;

procedure TNamedValues.WriteData(Writer: TWriter);
var
  i: Integer;
  nv: TNamedValue;
begin
  Writer.WriteListBegin;
  for i := 0 to Count - 1 do begin
    nv := Values[i];
    Writer.WriteString(nv.Name);
    Writer.WriteFloat(nv.Value);
  end;
  Writer.WriteListEnd;
end;

{******************************************************************************}
const
  Functions: array[TFunctionCode] of string = ('',
    {$IFNDEF VER80}
    'ArcCos',
    'ArcCosD',
    'ArcCosH',
    'ArcSin',
    'ArcSinD',
    'ArcSinH',
    'ArcTanH',
    'CosH',
    'Cotan',
    'Log10',
    'Log2',
    'Ln',
    'SinH',
    'GradToRad',
    'RadToGrad',
    'Ceil',
    'Floor',
    {$ENDIF}
    'ArcTan',
    'ArcTanD',
    'Cos',
    'CosD',
    'Exp',
    'Round',
    'Sin',
    'SinD',
    'Sqr',
    'Sqrt',
    'Tan',
    'TanD',
    'TanH',
    'Trunc',
    'DegToRad',
    'RadToDeg');

{$IFDEF VER80}
const
  PI = 3.14159265358979323846;

function DegToRad(const r: extended): extended;
begin
  result := r * 180.0 / PI;
end;

function Power(const x, y: extended): extended;
var
  i: integer;
begin
  result := 1;
  for i := 1 to trunc(y) do
    result := result * x;
  result := result + result * x * (y - trunc(y));
end;

function RadToDeg(const r: extended): extended;
begin
  result := r * PI / 180.0;
end;
{$ENDIF}

{ TExpressionEvaluator }

constructor TExpressionEvaluator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FNamedValues := TNamedValues.Create;
  if not (csDesigning in ComponentState) then begin
    FTempVariables := TNamedValues.Create;
    FTempVariables.CaseSensitive := FCaseSensitive;
  end;
end;

destructor TExpressionEvaluator.Destroy;
begin
  FNamedValues.Free;
  FTempVariables.Free;
  inherited Destroy;
end;

procedure TExpressionEvaluator.Changed;
begin
  if csLoading in ComponentState then exit;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TExpressionEvaluator.Error(const s: string);
begin
  raise EExpressionError.Create(s);
end;

function TExpressionEvaluator.EvalExpression: extended;
  function IsAddOp(const c: char): boolean;
  begin
    result := c in ['+', '-'];
  end;
begin
  result := 0.0;
  try
    if not IsAddop(FLook) then
      result := Term;
    while IsAddop(FLook) do begin
      case FLook of
        '+': begin
            Match('+');
            result := result + Term;
          end;
        '-': begin
            Match('-');
            result := result - Term;
          end;
      end;
    end;
  except
    on E: Exception do
      Error(e.message);
  end;
end;

function TExpressionEvaluator.EvalFunction(const fn: TFunctionCode; const arg: extended): extended;
begin
  result := arg;
  try
    case fn of
      {$IFNDEF VER80}
      fnArcCos: result := ArcCos(arg);
      fnArcCosD: result := RadToDeg(ArcCos(arg));
      fnArcCosh: result := ArcCosh(arg);
      fnArcSin: result := ArcSin(arg);
      fnArcSinD: result := RadToDeg(ArcSin(arg));
      fnArcSinh: result := ArcSinh(arg);
      fnArcTanh: result := ArcTanh(arg);
      fnCosh: result := Cosh(arg);
      fnCotan: result := Cotan(arg);
      fnLog10: result := Log10(arg);
      fnLog2: result := Log2(arg);
      fnLn: result := Ln(arg);
      fnSinh: result := Sinh(arg);
      fnGradToRad: result := GradToRad(arg);
      fnRadToGrad: result := RadToGrad(arg);
      fnTan: result := Tan(arg);
      fnTanD: result := Tan(DegToRad(arg));
      fnTanh: result := Tanh(arg);
      fnCeil: result := Ceil(arg);
      fnFloor: result := Floor(arg);
      {$ELSE}
      fnTan: result := Sin(arg) / Cos(arg);
      fnTanD: result := Sin(DegToRad(arg)) / Cos(DegToRad(arg));
      {$ENDIF}
      fnArcTan: result := ArcTan(arg);
      fnArcTanD: result := RadToDeg(ArcTan(arg));
      fnCos: result := Cos(arg);
      fnCosD: result := Cos(DegToRad(arg));
      fnExp: result := Exp(arg);
      fnRound: result := Round(arg);
      fnSin: result := Sin(arg);
      fnSinD: result := Sin(DegToRad(arg));
      fnSqr: result := Sqr(arg);
      fnSqrt: result := Sqrt(arg);
      fnTrunc: result := Trunc(arg);
      fnDegToRad: result := DegToRad(arg);
      fnRadToDeg: result := RadToDeg(arg);
    else
      Error('Unknown function');
    end;
  except
    on E: Exception do
      Error(e.message);
  end;
end;

procedure TExpressionEvaluator.Expected(const s: string);
begin
  Error(s + ' expected');
end;

function TExpressionEvaluator.Factor: extended;
var
  undefined: boolean;
  ident: string;
  fn: TFunctionCode;
  i: integer;
begin
  try
    if FLook = '(' then begin
      Match('(');
      result := EvalExpression;
      Match(')');
    end else if IsAlpha(FLook) then begin
      result := 0;
      undefined := true;
      ident := GetName;
      { is the identifier a named value? }
      if undefined and Assigned(FNamedValues) then begin
        i := FNamedValues.IndexOf(ident);
        if i <> -1 then begin
          result := FNamedValues.ValueByIndex[i];
          undefined := false;
        end;
      end;
      if undefined and Assigned(FOnGetValue) then begin
        { has the caller defined it? }
        i := FTempVariables.IndexOf(ident);
        if i <> -1 then begin
          result := FTempVariables.ValueByIndex[i];
          undefined := false;
        end else begin
          { allow the user to define it now }
          FOnGetValue(Self, ident, Result, undefined);
          if not undefined then
            FTempVariables.Add(ident, Result);
        end;
      end;
      if undefined then begin
        fn := GetFunctionCode(ident);
        if fn <> fnUnknown then begin
          Match('(');
          result := EvalFunction(fn, EvalExpression);
          Match(')');
          undefined := false;
        end;
      end;
      if undefined then
        Error('Undefined identifier "' + ident + '"');
    end else
      result := GetNumber;
  except
    on E: Exception do
      Error(e.message);
  end;
end;

procedure TExpressionEvaluator.GetChar;
begin
  if FPtr < Length(FExpression) then begin
    inc(FPtr);
    FLook := FExpression[FPtr];
  end else
    FLook := #0;
end;

function TExpressionEvaluator.GetFunctionCode(const str: string): TFunctionCode;
begin
  for result := low(TFunctionCode) to high(TFunctionCode) do begin
    if CompareText(Functions[result], str) = 0 then
      exit;
  end;
  result := fnUnknown;
end;

function TExpressionEvaluator.GetName: string;
begin
  result := '';
  if not IsAlpha(FLook) then Expected('Name');
  while IsAlphaNumeric(FLook) do begin
    result := result + FLook;
    GetChar;
  end;
  SkipWhiteSpace;
end;

function TExpressionEvaluator.GetNumber: extended;
var
  f: extended;
begin
  result := 0.0;
  try
    if not IsDigit(FLook) then Expected('Value');
    while IsDigit(FLook) do begin
      result := 10 * result + Ord(FLook) - Ord('0');
      GetChar;
    end;
    if FLook = '.' then begin
      Match('.');
      f := 10;
      while IsDigit(FLook) do begin
        result := result + (Ord(FLook) - Ord('0')) / f;
        f := f * 10;
        GetChar;
      end;
    end;
    SkipWhiteSpace;
  except
    on E: Exception do
      Error(e.message);
  end;
end;

function TExpressionEvaluator.GetValue: extended;
begin
  try
    FPtr := 0;
    GetChar;
    result := EvalExpression;
    if FLook <> #0 then
      Error('incomplete expression');
  finally
    FTempVariables.Clear;
  end;
end;

function TExpressionEvaluator.IsAlpha(const c: char): boolean;
begin
  result := upcase(c) in ['A'..'Z'];
end;

function TExpressionEvaluator.IsAlphaNumeric(const c: char): boolean;
begin
  result := IsAlpha(c) or IsDigit(c);
end;

function TExpressionEvaluator.IsDigit(const c: char): boolean;
begin
  result := c in ['0'..'9'];
end;

procedure TExpressionEvaluator.Loaded;
begin
  inherited Loaded;
  if FExpression <> '' then
    Changed;
end;

procedure TExpressionEvaluator.Match(const x: char);
begin
  if FLook = x then begin
    GetChar;
    SkipWhiteSpace;
  end else
    Expected('''' + x + '''');
end;

procedure TExpressionEvaluator.SetCaseSensitive(const v: boolean);
begin
  if FCaseSensitive = v then exit;
  FCaseSensitive := v;
  if Assigned(FTempVariables) then
    FTempVariables.CaseSensitive := FCaseSensitive;
  if Assigned(FNamedValues) then
    FNamedValues.CaseSensitive := FCaseSensitive;
  Changed;
end;

procedure TExpressionEvaluator.SetExpression(const e: string);
begin
  if FExpression <> e then begin
    FExpression := e;
    Changed;
  end;
end;

procedure TExpressionEvaluator.SetNamedValues(const nv: TNamedValues);
begin
  FNamedValues.Assign(nv);
  CaseSensitive := FNamedValues.CaseSensitive;
end;

procedure TExpressionEvaluator.SkipWhitespace;
begin
  while FLook in [' ', #9] do
    GetChar;
end;

function TExpressionEvaluator.Term: extended;
begin
  result := Factor;
  try
    while FLook in ['^', '*', '/'] do begin
      case FLook of
        '^': begin
            Match('^');
            result := Power(result, Factor);
          end;
        '*': begin
            Match('*');
            result := result * Factor;
          end;
        '/': begin
            Match('/');
            result := result / Factor;
          end;
      end;
    end;
  except
    on E: Exception do
      Error(e.message);
  end;
end;

function TExpressionEvaluator.IsBuiltIn(const FuncName: string): boolean;
begin
  result := GetFunctionCode(FuncName) <> fnUnknown;
end;

end.

