unit ComplexSupport;

// A complex value is defined as a variant array that contains exactly two
//  doubles.
// The [0]th element is the real part and...
//  the [1]st element is the imaginary part.

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, Math,Variants;

//==============================================================================
const
  cRealPart = 0;
  cImaginaryPart = 1;

//==============================================================================
function VarComplexAdd(aValue1, aValue2: variant): variant;
function VarComplexSubtract(aValue1, aValue2: variant): variant;
function VarComplexMultiply(aValue1, aValue2: variant): variant;
function VarComplexDivide(aValue1, aValue2: variant): variant;

//==============================================================================
function VarComplexGreater(aValue1, aValue2: variant): boolean;
function VarComplexGreaterOrEqual(aValue1, aValue2: variant): boolean;
function VarComplexEqual(aValue1, aValue2: variant): boolean;
function VarComplexNotEqual(aValue1, aValue2: variant): boolean;
function VarComplexLessOrEqual(aValue1, aValue2: variant): boolean;
function VarComplexLess(aValue1, aValue2: variant): boolean;

//==============================================================================
function VarComplexNegation(aValue: variant): variant;
function VarComplexConjugate(aValue: variant): variant;
function VarComplexInverse(aValue: variant): variant;
function VarComplexTimesPosI(aValue: variant): variant;
function VarComplexTimesNegI(aValue: variant): variant;

//==============================================================================
function VarComplexNorm(aValue: variant): double;
function VarComplexAngle(aValue: variant): double;

//==============================================================================
function VarComplexExp(aValue: variant): variant;
function VarComplexLn(aValue: variant): variant;
function VarComplexSquareRoot(aValue: variant): variant;

//==============================================================================
function VarComplexCos(aValue: variant): variant;
function VarComplexSin(aValue: variant): variant;
function VarComplexTan(aValue: variant): variant;
function VarComplexArcCos(aValue: variant): variant;
function VarComplexArcSin(aValue: variant): variant;
function VarComplexArcTan(aValue: variant): variant;
function VarComplexCosH(aValue: variant): variant;
function VarComplexSinH(aValue: variant): variant;
function VarComplexTanH(aValue: variant): variant;
function VarComplexArcCosH(aValue: variant): variant;
function VarComplexArcSinH(aValue: variant): variant;
function VarComplexArcTanH(aValue: variant): variant;

//==============================================================================
function VarComplexFor(const aReal, aImaginary: variant): variant;
procedure ParseRealAndImaginaryPartsOfVarComplex(const aValue: variant; var aReal, aImaginary: double);
function RealPartOfVarComplex(const aValue: variant): double;
function ImaginaryPartOfVarComplex(const aValue: variant): double;
function VarIsComplex(const aValue: variant): boolean;
function VarAsComplex(const aValue: variant): variant;
function VarComplexAsType(aValue: variant; aType: integer): variant;
function VarSimplify(const aValue: variant): variant;

//==============================================================================
var
  vVarComplexAsTypeFormat: string = '%.4f + %.4fi';

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport;

//==============================================================================
function VarComplexAdd(aValue1, aValue2: variant): variant;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := VarComplexFor(aValue1[cRealPart] + aValue2[cRealPart],
                          aValue1[cImaginaryPart] + aValue2[cImaginaryPart]);
end;
function VarComplexSubtract(aValue1, aValue2: variant): variant;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := VarComplexFor(aValue1[cRealPart] - aValue2[cRealPart],
                          aValue1[cImaginaryPart] - aValue2[cImaginaryPart]);
end;
function VarComplexMultiply(aValue1, aValue2: variant): variant;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := VarComplexFor((aValue1[cRealPart] * aValue2[cRealPart]) -
                          (aValue1[cImaginaryPart] * aValue2[cImaginaryPart]),
                          (aValue1[cRealPart] * aValue2[cImaginaryPart]) +
                          (aValue1[cImaginaryPart] * aValue2[cRealPart]));
end;
function VarComplexDivide(aValue1, aValue2: variant): variant;
var
  vDenominator: double;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  vDenominator := aValue2[cRealPart] * aValue2[cRealPart] +
                  aValue2[cImaginaryPart] * aValue2[cImaginaryPart];
  if vDenominator = 0.0 then begin
    SetResult(STATUS_FLOAT_DIVIDE_BY_ZERO);
    Result := Unassigned;
  end else
    Result := VarComplexFor(((aValue1[cRealPart] * aValue1[cRealPart]) +
                             (aValue1[cImaginaryPart] * aValue1[cImaginaryPart])) / vDenominator,
                            ((aValue1[cImaginaryPart] * aValue1[cRealPart]) -
                             (aValue1[cRealPart] * aValue1[cImaginaryPart])) / vDenominator);
end;

//==============================================================================
function VarComplexGreater(aValue1, aValue2: variant): boolean;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := (aValue1[cRealPart] > aValue2[cRealPart]) and
            (aValue1[cImaginaryPart] > aValue2[cImaginaryPart]);
end;
function VarComplexGreaterOrEqual(aValue1, aValue2: variant): boolean;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := (aValue1[cRealPart] >= aValue2[cRealPart]) and
            (aValue1[cImaginaryPart] >= aValue2[cImaginaryPart]);
end;
function VarComplexEqual(aValue1, aValue2: variant): boolean;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := (aValue1[cRealPart] = aValue2[cRealPart]) and
            (aValue1[cImaginaryPart] = aValue2[cImaginaryPart]);
end;
function VarComplexNotEqual(aValue1, aValue2: variant): boolean;
begin
  Result := not VarComplexEqual(aValue1, aValue2);
end;
function VarComplexLessOrEqual(aValue1, aValue2: variant): boolean;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := (aValue1[cRealPart] <= aValue2[cRealPart]) and
            (aValue1[cImaginaryPart] <= aValue2[cImaginaryPart]);
end;
function VarComplexLess(aValue1, aValue2: variant): boolean;
begin
  aValue1 := VarAsComplex(aValue1);
  aValue2 := VarAsComplex(aValue2);
  Result := (aValue1[cRealPart] < aValue2[cRealPart]) and
            (aValue1[cImaginaryPart] < aValue2[cImaginaryPart]);
end;

//==============================================================================
function VarComplexNegation(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(-aValue[cRealPart], -aValue[cImaginaryPart]);
end;
function VarComplexConjugate(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(aValue[cRealPart], -aValue[cImaginaryPart]);
end;
function VarComplexInverse(aValue: variant): variant;
var
  vDenominator: double;
begin
  aValue := VarAsComplex(aValue);
  vDenominator := aValue[cRealPart] * aValue[cRealPart] +
                  aValue[cImaginaryPart] * aValue[cImaginaryPart];
  if vDenominator = 0.0 then begin
    SetResult(STATUS_FLOAT_DIVIDE_BY_ZERO);
    Result := Unassigned;
  end else
    Result := VarComplexFor(aValue[cRealPart] / vDenominator,
                            -(aValue[cImaginaryPart] / vDenominator));
end;
function VarComplexTimesPosI(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(-aValue[cImaginaryPart], aValue[cRealPart]);
end;
function VarComplexTimesNegI(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(aValue[cImaginaryPart], -aValue[cRealPart]);
end;

//==============================================================================
function VarComplexNorm(aValue: variant): double;
begin
  aValue := VarAsComplex(aValue);
  Result := Sqrt((aValue[cRealPart] * aValue[cRealPart]) +
                 (aValue[cImaginaryPart] * aValue[cImaginaryPart]));
end;
function VarComplexAngle(aValue: variant): double;
begin
  aValue := VarAsComplex(aValue);
  Result := ArcTan2(aValue[cRealPart], aValue[cImaginaryPart]);
end;

//==============================================================================
function VarComplexExp(aValue: variant): variant;
var
  vExp: double;
begin
  aValue := VarAsComplex(aValue);
  vExp := Exp(aValue[cRealPart]);
  Result := VarComplexFor(vExp * Cos(aValue[cImaginaryPart]),
                          vExp * Sin(aValue[cImaginaryPart]));
end;
function VarComplexLn(aValue: variant): variant;
var
  vModule: double;
begin
  aValue := VarAsComplex(aValue);
  vModule := aValue[cRealPart] * aValue[cRealPart] +
             aValue[cImaginaryPart] * aValue[cImaginaryPart];
  if vModule = 0.0 then begin
    SetResult(STATUS_FLOAT_DIVIDE_BY_ZERO);
    Result := Unassigned;
  end else
    Result := VarComplexFor(Ln(vModule),
                            ArcTan2(aValue[cRealPart], aValue[cImaginaryPart]));
end;
function VarComplexSquareRoot(aValue: variant): variant;
var
  vValue1, vValue2: variant;
begin
  aValue := VarAsComplex(aValue);
  if (aValue[cRealPart] <> 0.0) or
     (aValue[cImaginaryPart] <> 0.0) then begin
    vValue1 := Sqrt(0.5 * (Abs(aValue[cRealPart]) + VarComplexNorm(aValue)));
    vValue2 := aValue[cImaginaryPart] / (2.0 * vValue1);
    if aValue[cRealPart] >= 0.0 then
      Result := VarComplexFor(vValue1, vValue2)
    else if aValue[cImaginaryPart] < 0.0 then
      Result := VarComplexFor(-vValue2, -vValue1)
    else
      Result := VarComplexFor(vValue2, vValue1);
  end else
    Result := aValue;
end;

//==============================================================================
function VarComplexCos(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(Cos(aValue[cRealPart]) * CosH(aValue[cImaginaryPart]),
                          -Sin(aValue[cRealPart]) * SinH(aValue[cImaginaryPart]));
end;
function VarComplexSin(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(Sin(aValue[cRealPart]) * CosH(aValue[cImaginaryPart]),
                          Cos(aValue[cRealPart]) * SinH(aValue[cImaginaryPart]));
end;
function VarComplexTan(aValue: variant): variant;
var
  vCos: variant;
begin
  vCos := VarComplexCos(aValue);
  if (vCos[cRealPart] = 0.0) and
     (vCos[cImaginaryPart] = 0.0) then begin
    SetResult(STATUS_FLOAT_DIVIDE_BY_ZERO);
    Result := Unassigned;
  end else
    Result := VarComplexDivide(VarComplexSin(aValue), vCos);
end;
function VarComplexArcCos(aValue: variant): variant;
begin
  Result := VarComplexTimesNegI(VarComplexArcCosH(aValue));
end;
function VarComplexArcSin(aValue: variant): variant;
begin
  Result := VarComplexTimesNegI(VarComplexArcSinH(VarComplexTimesPosI(aValue)));
end;
function VarComplexArcTan(aValue: variant): variant;
begin
  Result := VarComplexTimesNegI(VarComplexArcTanH(VarComplexTimesPosI(aValue)));
end;
function VarComplexCosH(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(CosH(aValue[cRealPart]) * Cos(aValue[cImaginaryPart]),
                          SinH(aValue[cRealPart]) * Sin(aValue[cImaginaryPart]));
end;
function VarComplexSinH(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(SinH(aValue[cRealPart]) * Cos(aValue[cImaginaryPart]),
                          CosH(aValue[cRealPart]) * Sin(aValue[cImaginaryPart]));
end;
function VarComplexTanH(aValue: variant): variant;
var
  vCosH: variant;
begin
  vCosH := VarComplexCosH(aValue);
  if (vCosH[cRealPart] = 0.0) and
     (vCosH[cImaginaryPart] = 0.0) then begin
    SetResult(STATUS_FLOAT_DIVIDE_BY_ZERO);
    Result := Unassigned;
  end else
    Result := VarComplexDivide(VarComplexSinH(aValue), vCosH);
end;
function VarComplexArcCosH(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(1 - aValue[cRealPart] * aValue[cRealPart] +
                              aValue[cImaginaryPart] * aValue[cImaginaryPart],
                          -2 * aValue[cRealPart] * aValue[cImaginaryPart]);
  Result := VarComplexNegation(VarComplexLN(VarComplexAdd(
            VarComplexTimesPosI(VarComplexSquareRoot(Result)), aValue)));
end;
function VarComplexArcSinH(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  Result := VarComplexFor(1 + aValue[cRealPart] * aValue[cRealPart] -
                              aValue[cImaginaryPart] * aValue[cImaginaryPart],
                          2 * aValue[cRealPart] * aValue[cImaginaryPart]);
  Result := VarComplexLN(VarComplexAdd(VarComplexSquareRoot(Result), aValue));
end;
function VarComplexArcTanH(aValue: variant): variant;
begin
  aValue := VarAsComplex(aValue);
  aValue := VarComplexDivide(VarComplexFor(1 + aValue[cRealPart],
                                           aValue[cImaginaryPart]),
                             VarComplexFor(1 - aValue[cRealPart],
                                           -aValue[cImaginaryPart]));
  Result := VarComplexFor(0.5 * aValue[cRealPart],
                          0.5 * aValue[cImaginaryPart]);
end;

//==============================================================================
function VarComplexFor(const aReal, aImaginary: variant): variant;
begin
  // create the array
  Result := VarArrayCreate([cRealPart, cImaginaryPart], varDouble);
  Result[cRealPart] := VarComplexAsType(aReal, varDouble);
  Result[cImaginaryPart] := VarComplexAsType(aImaginary, varDouble);
end;
procedure ParseRealAndImaginaryPartsOfVarComplex(const aValue: variant; var aReal, aImaginary: double);
begin
  if VarIsComplex(aValue) then begin
    aReal := aValue[cRealPart];
    aImaginary := aValue[cImaginaryPart];
  end else begin
    aReal := VarAsType(aValue, varDouble);
    aImaginary := 0;
  end;
end;
function RealPartOfVarComplex(const aValue: variant): double;
var
  vJunk: double;
begin
  ParseRealAndImaginaryPartsOfVarComplex(aValue, Result, vJunk);
end;
function ImaginaryPartOfVarComplex(const aValue: variant): double;
var
  vJunk: double;
begin
  ParseRealAndImaginaryPartsOfVarComplex(aValue, vJunk, Result);
end;
function VarIsComplex(const aValue: variant): boolean;
begin
  Result := (VarArrayDimCount(aValue) = 1) and
            (VarArrayLowBound(aValue, 1) = cRealPart) and
            (VarArrayHighBound(aValue, 1) = cImaginaryPart) and
            (VarType(aValue[cRealPart]) = varDouble) and
            (VarType(aValue[cImaginaryPart]) = varDouble);
end;
function VarAsComplex(const aValue: variant): variant;
begin
  if VarIsComplex(aValue) then
    Result := aValue
  else
    Result := VarComplexFor(aValue, 0);
end;
function VarComplexAsType(aValue: variant; aType: integer): variant;
var
  vReal, vImaginary: double;
begin
  aValue := VarSimplify(aValue);
  if VarIsComplex(aValue) then begin
    case aType of
    varOleStr, varString: begin
      vReal := aValue[cRealPart];
      vImaginary := aValue[cImaginaryPart];
      Result := Format(vVarComplexAsTypeFormat, [vReal, vImaginary]);
    end else
      Result := VarAsType(aValue[0], aType);
    end;
  end else
    Result := VarAsType(aValue, aType);
end;
function VarSimplify(const aValue: variant): variant;
begin
  if VarIsComplex(aValue) and
     (aValue[cImaginaryPart] = 0) then
    Result := aValue[cRealPart]
  else
    Result := aValue;
end;

end.
