unit DateTimeSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes;

//==============================================================================
function DateOf(const aValue: TDateTime): TDateTime;
function TimeOf(const aValue: TDateTime): TDateTime;

//==============================================================================
function IsInLeapYear(const aValue: TDateTime): boolean;
function DaysInYear(const aValue: TDateTime): integer;
function DaysInAYear(const aYear: integer): integer;
function DaysInMonth(const aValue: TDateTime): integer;
function DaysInAMonth(const aYear, aMonth: integer): integer;

//==============================================================================
function YearOf(const aValue: TDateTime): integer;
function MonthOf(const aValue: TDateTime): integer;
function WeekOf(const aValue: TDateTime): integer;
function DayOf(const aValue: TDateTime): integer;
function HourOf(const aValue: TDateTime): integer;
function MinuteOf(const aValue: TDateTime): integer;
function SecondOf(const aValue: TDateTime): integer;
function MilliSecondOf(const aValue: TDateTime): integer;

//==============================================================================
function StartOfYear(const aValue: TDateTime): TDateTime;
function EndOfYear(const aValue: TDateTime): TDateTime;
function StartOfMonth(const aValue: TDateTime): TDateTime;
function EndOfMonth(const aValue: TDateTime): TDateTime;
function StartOfWeek(const aValue: TDateTime): TDateTime;
function EndOfWeek(const aValue: TDateTime): TDateTime;
function StartOfDay(const aValue: TDateTime): TDateTime;
function EndOfDay(const aValue: TDateTime): TDateTime;

//==============================================================================
function MonthOfYear(const aValue: TDateTime): integer;
function WeekOfYear(const aValue: TDateTime): integer;
function DayOfYear(const aValue: TDateTime): integer;
function HourOfYear(const aValue: TDateTime): comp;
function MinuteOfYear(const aValue: TDateTime): comp;
function SecondOfYear(const aValue: TDateTime): comp;
function MilliSecondOfYear(const aValue: TDateTime): comp;

//==============================================================================
function WeekOfMonth(const aValue: TDateTime): integer;
function DayOfMonth(const aValue: TDateTime): integer;
function HourOfMonth(const aValue: TDateTime): comp;
function MinuteOfMonth(const aValue: TDateTime): comp;
function SecondOfMonth(const aValue: TDateTime): comp;
function MilliSecondOfMonth(const aValue: TDateTime): comp;

//==============================================================================
{function DayOfWeek(const aValue: TDateTime): integer;} // provided by Delphi
function HourOfWeek(const aValue: TDateTime): comp;
function MinuteOfWeek(const aValue: TDateTime): comp;
function SecondOfWeek(const aValue: TDateTime): comp;
function MilliSecondOfWeek(const aValue: TDateTime): comp;

//==============================================================================
function HourOfDay(const aValue: TDateTime): comp;
function MinuteOfDay(const aValue: TDateTime): comp;
function SecondOfDay(const aValue: TDateTime): comp;
function MilliSecondOfDay(const aValue: TDateTime): comp;

//==============================================================================
function MinuteOfHour(const aValue: TDateTime): comp;
function SecondOfHour(const aValue: TDateTime): comp;
function MilliSecondOfHour(const aValue: TDateTime): comp;

//==============================================================================
function SecondOfMinute(const aValue: TDateTime): comp;
function MilliSecondOfMinute(const aValue: TDateTime): comp;

//==============================================================================
function MilliSecondOfSecond(const aValue: TDateTime): comp;

//==============================================================================
function WithinPastYears(aNow, aThen: TDateTime; aYears: integer): boolean;
function WithinPastMonths(aNow, aThen: TDateTime; aMonths: integer): boolean;
function WithinPastWeeks(aNow, aThen: TDateTime; aWeeks: integer): boolean;
function WithinPastDays(aNow, aThen: TDateTime; aDays: integer): boolean;
function WithinPastHours(aNow, aThen: TDateTime; aHours: comp): boolean;
function WithinPastMinutes(aNow, aThen: TDateTime; aMinutes: comp): boolean;
function WithinPastSeconds(aNow, aThen: TDateTime; aSeconds: comp): boolean;
function WithinPastMilliSeconds(aNow, aThen: TDateTime; aMilliSeconds: comp): boolean;

//==============================================================================
function YearsBetween(aNow, aThen: TDateTime): integer;
function MonthsBetween(aNow, aThen: TDateTime): integer;
function WeeksBetween(aNow, aThen: TDateTime): integer;
function DaysBetween(aNow, aThen: TDateTime): integer;
function HoursBetween(aNow, aThen: TDateTime): comp;
function MinutesBetween(aNow, aThen: TDateTime): comp;
function SecondsBetween(aNow, aThen: TDateTime): comp;
function MilliSecondsBetween(aNow, aThen: TDateTime): comp;

//==============================================================================
function YearSpan(aNow, aThen: TDateTime): double;
//function OldYearSpan(aNow, aThen: TDateTime): double;
function MonthSpan(aNow, aThen: TDateTime): double;
//function OldMonthSpan(aNow, aThen: TDateTime): double;
function WeekSpan(aNow, aThen: TDateTime): double;
//function OldWeekSpan(aNow, aThen: TDateTime): double;
function DaySpan(aNow, aThen: TDateTime): double;
function HourSpan(aNow, aThen: TDateTime): double;
function MinuteSpan(aNow, aThen: TDateTime): double;
function SecondSpan(aNow, aThen: TDateTime): double;
function MilliSecondSpan(aNow, aThen: TDateTime): double;

//==============================================================================
function EncodeDateTime(aYear, aMonth, aDay, aHour, aMinute, aSecond, aMilliSecond: integer): TDateTime;
function RecodeDateYear(const aValue: TDateTime; aYear: integer): TDateTime;
function RecodeDateMonth(const aValue: TDateTime; aMonth: integer): TDateTime;
function RecodeDateDay(const aValue: TDateTime; aDay: integer): TDateTime;
function RecodeDateHour(const aValue: TDateTime; aHour: integer): TDateTime;
function RecodeDateMinute(const aValue: TDateTime; aMinute: integer): TDateTime;
function RecodeDateSecond(const aValue: TDateTime; aSecond: integer): TDateTime;
function RecodeDateMilliSecond(const aValue: TDateTime; aMilliSecond: integer): TDateTime;

//==============================================================================
function SpanOfNowAndThen(var aNow, aThen: TDateTime): TDateTime;
function IsNullDateTime(const aTime: TDateTime): boolean;
function NullDateTime: TDateTime;

//==============================================================================
const
  cDaysInAMonth: array[1..12] of integer = (31, 28, 31, 30,
                                            31, 30, 31, 31,
                                            30, 31, 30, 31);
  cApproxDaysInAYear  = 365.25;
  cApproxDaysInAMonth = 30.4375;
  cDaysInAWeek        = 7;
  cHoursInADay        = 24;
  cMinutesInADay      = cHoursInADay * 60;
  cSecondsInADay      = cMinutesInADay * 60;
  cMilliSecondsInADay = cSecondsInADay * 1000;

//==============================================================================
implementation

//==============================================================================
uses
  Dialogs, ResultSupport, MiscSupport;

//==============================================================================
function DateOf(const aValue: TDateTime): TDateTime;
var
  vYear, vMonth, vDay: word;
begin
  DecodeDate(aValue, vYear, vMonth, vDay);
  Result := EncodeDate(vYear, vMonth, vDay);
end;
function TimeOf(const aValue: TDateTime): TDateTime;
var
  vHour, vMinute, vSecond, vMilliSecond: word;
begin
  DecodeTime(aValue, vHour, vMinute, vSecond, vMilliSecond);
  Result := EncodeTime(vHour, vMinute, vSecond, vMilliSecond);
end;

//==============================================================================
function IsInLeapYear(const aValue: TDateTime): boolean;
begin
  Result := IsLeapYear(YearOf(aValue));
end;
function DaysInYear(const aValue: TDateTime): integer;
begin
  Result := DaysInAYear(YearOf(aValue));
end;
function DaysInAYear(const aYear: integer): integer;
begin
  Result := IfThenInt(IsLeapYear(aYear), 366, 365);
end;
function DaysInMonth(const aValue: TDateTime): integer;
begin
  Result := DaysInAMonth(YearOf(aValue), MonthOf(aValue));
end;
function DaysInAMonth(const aYear, aMonth: integer): integer;
begin
  Assert((aMonth >= 1) and (aMonth <= 12), ResultTextFor(DISP_E_BADINDEX));
  Result := cDaysInAMonth[aMonth];
  if (aMonth = 2) and
     IsLeapYear(aYear) then
    Inc(Result);
end;

//==============================================================================
function YearOf(const aValue: TDateTime): integer;
var
  vYear, vMonth, vDay: word;
begin
  DecodeDate(aValue, vYear, vMonth, vDay);
  Result := vYear;
end;
function MonthOf(const aValue: TDateTime): integer;
var
  vYear, vMonth, vDay: word;
begin
  DecodeDate(aValue, vYear, vMonth, vDay);
  Result := vMonth;
end;
function WeekOf(const aValue: TDateTime): integer;
begin
  Result := (DayOfYear(aValue) + DayOfWeek(StartOfYear(aValue)) - 1) div cDaysInAWeek + 1;
end;
function DayOf(const aValue: TDateTime): integer;
var
  vYear, vMonth, vDay: word;
begin
  DecodeDate(aValue, vYear, vMonth, vDay);
  Result := vDay;
end;
function HourOf(const aValue: TDateTime): integer;
var
  vHour, vMinute, vSecond, vMilliSecond: word;
begin
  DecodeTime(aValue, vHour, vMinute, vSecond, vMilliSecond);
  Result := vHour;
end;
function MinuteOf(const aValue: TDateTime): integer;
var
  vHour, vMinute, vSecond, vMilliSecond: word;
begin
  DecodeTime(aValue, vHour, vMinute, vSecond, vMilliSecond);
  Result := vMinute;
end;
function SecondOf(const aValue: TDateTime): integer;
var
  vHour, vMinute, vSecond, vMilliSecond: word;
begin
  DecodeTime(aValue, vHour, vMinute, vSecond, vMilliSecond);
  Result := vSecond;
end;
function MilliSecondOf(const aValue: TDateTime): integer;
var
  vHour, vMinute, vSecond, vMilliSecond: word;
begin
  DecodeTime(aValue, vHour, vMinute, vSecond, vMilliSecond);
  Result := vMilliSecond;
end;

//==============================================================================
function StartOfYear(const aValue: TDateTime): TDateTime;
begin
  Result := StartOfDay(EncodeDate(YearOf(aValue), 1, 1));
end;
function EndOfYear(const aValue: TDateTime): TDateTime;
begin
  Result := EndOfDay(EncodeDate(YearOf(aValue), 12, 31));
end;
function StartOfMonth(const aValue: TDateTime): TDateTime;
begin
  Result := StartOfDay(EncodeDate(YearOf(aValue), MonthOf(aValue), 1));
end;
function EndOfMonth(const aValue: TDateTime): TDateTime;
begin
  Result := EndOfDay(EncodeDate(YearOf(aValue), MonthOf(aValue), DaysInMonth(aValue)));
end;
function StartOfWeek(const aValue: TDateTime): TDateTime;
begin
  Result := StartOfDay(aValue - (DayOfWeek(aValue) - 1));
end;
function EndOfWeek(const aValue: TDateTime): TDateTime;
begin
  Result := EndOfDay(StartOfWeek(aValue) + 6);
end;
function StartOfDay(const aValue: TDateTime): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(0, 0, 0, 0);
end;
function EndOfDay(const aValue: TDateTime): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(11, 59, 59, 999);
end;

//==============================================================================
function MonthOfYear(const aValue: TDateTime): integer;
begin
  Result := MonthOf(aValue);
end;
function WeekOfYear(const aValue: TDateTime): integer;
begin
  Result := WeekOf(aValue);
end;
function DayOfYear(const aValue: TDateTime): integer;
begin
  Result := Trunc(aValue - StartOfYear(aValue)) + 1;
end;
function HourOfYear(const aValue: TDateTime): comp;
begin
  Result := HourOf(aValue) + (DayOfYear(aValue) - 1) * 24;
end;
function MinuteOfYear(const aValue: TDateTime): comp;
begin
  Result := MinuteOf(aValue) + HourOfYear(aValue) * 60;
end;
function SecondOfYear(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue) + MinuteOfYear(aValue) * 60;
end;
function MilliSecondOfYear(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfYear(aValue) + 1000;
end;

//==============================================================================
function WeekOfMonth(const aValue: TDateTime): integer;
begin
  Result := WeekOf(aValue) - WeekOf(StartOfMonth(aValue)) + 1;
end;
function DayOfMonth(const aValue: TDateTime): integer;
begin
  Result := DayOf(aValue);
end;
function HourOfMonth(const aValue: TDateTime): comp;
begin
  Result := HourOf(aValue) + (DayOfMonth(aValue) - 1) * 24;
end;
function MinuteOfMonth(const aValue: TDateTime): comp;
begin
  Result := MinuteOf(aValue) + HourOfMonth(aValue) * 60;
end;
function SecondOfMonth(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue) + MinuteOfMonth(aValue) * 60;
end;
function MilliSecondOfMonth(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfMonth(aValue) + 1000;
end;

//==============================================================================
{function DayOfWeek(const aValue: TDateTime): integer;} // Provided by Delphi
function HourOfWeek(const aValue: TDateTime): comp;
begin
  Result := HourOf(aValue) + (DayOfWeek(aValue) - 1) * 24;
end;
function MinuteOfWeek(const aValue: TDateTime): comp;
begin
  Result := MinuteOf(aValue) + HourOfWeek(aValue) * 60;
end;
function SecondOfWeek(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue) + MinuteOfWeek(aValue) * 60;
end;
function MilliSecondOfWeek(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfWeek(aValue) + 1000;
end;

//==============================================================================
function HourOfDay(const aValue: TDateTime): comp;
begin
  Result := HourOf(aValue);
end;
function MinuteOfDay(const aValue: TDateTime): comp;
begin
  Result := MinuteOf(aValue) + HourOfDay(aValue) * 60;
end;
function SecondOfDay(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue) + MinuteOfDay(aValue) * 60;
end;
function MilliSecondOfDay(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfDay(aValue) + 1000;
end;

//==============================================================================
function MinuteOfHour(const aValue: TDateTime): comp;
begin
  Result := MinuteOf(aValue);
end;
function SecondOfHour(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue) + MinuteOfHour(aValue) * 60;
end;
function MilliSecondOfHour(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfHour(aValue) + 1000;
end;

//==============================================================================
function SecondOfMinute(const aValue: TDateTime): comp;
begin
  Result := SecondOf(aValue);
end;
function MilliSecondOfMinute(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue) + SecondOfMinute(aValue) + 1000;
end;

//==============================================================================
function MilliSecondOfSecond(const aValue: TDateTime): comp;
begin
  Result := MilliSecondOf(aValue);
end;

//==============================================================================
function WithinPastYears(aNow, aThen: TDateTime; aYears: integer): boolean;
begin
  Result := YearsBetween(aNow, aThen) <= aYears;
end;
function WithinPastMonths(aNow, aThen: TDateTime; aMonths: integer): boolean;
begin
  Result := MonthsBetween(aNow, aThen) <= aMonths;
end;
function WithinPastWeeks(aNow, aThen: TDateTime; aWeeks: integer): boolean;
begin
  Result := WeeksBetween(aNow, aThen) <= aWeeks;
end;
function WithinPastDays(aNow, aThen: TDateTime; aDays: integer): boolean;
begin
  Result := DaysBetween(aNow, aThen) <= aDays;
end;
function WithinPastHours(aNow, aThen: TDateTime; aHours: comp): boolean;
begin
  Result := HoursBetween(aNow, aThen) <= aHours;
end;
function WithinPastMinutes(aNow, aThen: TDateTime; aMinutes: comp): boolean;
begin
  Result := MinutesBetween(aNow, aThen) <= aMinutes;
end;
function WithinPastSeconds(aNow, aThen: TDateTime; aSeconds: comp): boolean;
begin
  Result := SecondsBetween(aNow, aThen) <= aSeconds;
end;
function WithinPastMilliSeconds(aNow, aThen: TDateTime; aMilliSeconds: comp): boolean;
begin
  Result := MilliSecondsBetween(aNow, aThen) <= aMilliSeconds;
end;

//==============================================================================
function YearsBetween(aNow, aThen: TDateTime): integer;
begin
  Result := Trunc(YearSpan(aNow, aThen));
end;
function MonthsBetween(aNow, aThen: TDateTime): integer;
begin
  Result := Trunc(MonthSpan(aNow, aThen));
end;
function WeeksBetween(aNow, aThen: TDateTime): integer;
begin
  Result := Trunc(WeekSpan(aNow, aThen));
end;
function DaysBetween(aNow, aThen: TDateTime): integer;
begin
  Result := Trunc(DaySpan(aNow, aThen));
end;
function HoursBetween(aNow, aThen: TDateTime): comp;
begin
  Result := Trunc(HourSpan(aNow, aThen));
end;
function MinutesBetween(aNow, aThen: TDateTime): comp;
begin
  Result := Trunc(MinuteSpan(aNow, aThen));
end;
function SecondsBetween(aNow, aThen: TDateTime): comp;
begin
  Result := Trunc(SecondSpan(aNow, aThen));
end;
function MilliSecondsBetween(aNow, aThen: TDateTime): comp;
begin
  Result := Trunc(MilliSecondSpan(aNow, aThen));
end;

//==============================================================================
// YearSpan and MonthSpan are now approximates, not exact but pretty close
// I will work on something more exact latter but for now this will have to do
function YearSpan(aNow, aThen: TDateTime): double;
begin
  Result := DaySpan(aNow, aThen) / cApproxDaysInAYear;
end;
//function OldYearSpan(aNow, aThen: TDateTime): double;
//begin
//  SpanOfNowAndThen(aNow, aThen);
//  Result := YearOf(aNow) - YearOf(aThen);
//  if DayOfYear(aNow) < DayOfYear(aThen) then
//    Result := Result - 1;
//end;
function MonthSpan(aNow, aThen: TDateTime): double;
begin
  Result := DaySpan(aNow, aThen) / cApproxDaysInAMonth;
end;
//function OldMonthSpan(aNow, aThen: TDateTime): double;
//begin
//  SpanOfNowAndThen(aNow, aThen);
//  Result := (YearOf(aNow) * 12 + MonthOf(aNow)) -
//            (YearOf(aThen) * 12 + MonthOf(aThen));
//  if DayOf(aNow) < DayOf(aThen) then
//    Result := Result - 1;
//end;
function WeekSpan(aNow, aThen: TDateTime): double;
begin
  Result := DaySpan(aNow, aThen) / cDaysInAWeek;
end;
//function OldWeekSpan(aNow, aThen: TDateTime): double;
//begin
//  SpanOfNowAndThen(aNow, aThen);
//  Result := DaysBetween(aNow - DayOfWeek(aNow) + 1,
//                        aThen - DayOfWeek(aThen) + 1) div cDaysInAWeek;
//  if DayOfWeek(aNow) < DayOfWeek(aThen) then
//    Dec(Result);
//end;
function DaySpan(aNow, aThen: TDateTime): double;
begin
  Result := SpanOfNowAndThen(aNow, aThen);
end;
function HourSpan(aNow, aThen: TDateTime): double;
begin
  Result := cHoursInADay * SpanOfNowAndThen(aNow, aThen);
end;
function MinuteSpan(aNow, aThen: TDateTime): double;
begin
  Result := cMinutesInADay * SpanOfNowAndThen(aNow, aThen);
end;
function SecondSpan(aNow, aThen: TDateTime): double;
begin
  Result := cSecondsInADay * SpanOfNowAndThen(aNow, aThen);
end;
function MilliSecondSpan(aNow, aThen: TDateTime): double;
begin
  Result := cMilliSecondsInADay * SpanOfNowAndThen(aNow, aThen);
end;

//==============================================================================
function EncodeDateTime(aYear, aMonth, aDay, aHour, aMinute, aSecond, aMilliSecond: integer): TDateTime;
begin
  Result := EncodeDate(aYear, aMonth, aDay) +
            EncodeTime(aHour, aMinute, aSecond, aMilliSecond);
end;
function RecodeDateYear(const aValue: TDateTime; aYear: integer): TDateTime;
begin
  Result := EncodeDate(aYear, MonthOf(aValue), DayOf(aValue)) + TimeOf(aValue);
end;
function RecodeDateMonth(const aValue: TDateTime; aMonth: integer): TDateTime;
begin
  Result := EncodeDate(YearOf(aValue), aMonth, DayOf(aValue)) + TimeOf(aValue);
end;
function RecodeDateDay(const aValue: TDateTime; aDay: integer): TDateTime;
begin
  Result := EncodeDate(YearOf(aValue), MonthOf(aValue), aDay) + TimeOf(aValue);
end;
function RecodeDateHour(const aValue: TDateTime; aHour: integer): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(aHour, MinuteOf(aValue), SecondOf(aValue), MilliSecondOf(aValue));
end;
function RecodeDateMinute(const aValue: TDateTime; aMinute: integer): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(HourOf(aValue), aMinute, SecondOf(aValue), MilliSecondOf(aValue));
end;
function RecodeDateSecond(const aValue: TDateTime; aSecond: integer): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(HourOf(aValue), MinuteOf(aValue), aSecond, MilliSecondOf(aValue));
end;
function RecodeDateMilliSecond(const aValue: TDateTime; aMilliSecond: integer): TDateTime;
begin
  Result := DateOf(aValue) + EncodeTime(HourOf(aValue), MinuteOf(aValue), SecondOf(aValue), aMilliSecond);
end;

//==============================================================================
function SpanOfNowAndThen(var aNow, aThen: TDateTime): TDateTime;
var
  vTemp: TDateTime;
begin
  if aNow < aThen then begin
    vTemp := aThen;
    aThen := aNow;
    aNow := vTemp;
  end;
  Result := aNow - aThen;
end;
function IsNullDateTime(const aTime: TDateTime): boolean;
begin
  Result := aTime = 0;
end;
function NullDateTime: TDateTime;
begin
  Result := 0;
end;

end.
