unit ExpressIt;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExpressionParser,Variants;

//==============================================================================
type
  TExpressIt = class(TCustomExpressIt)
  public
    // alpha, no omega in this case
    constructor Create(AOwner: TComponent); override;
  published
    // expose ourselves
    property AutoExecute;
    property Expression;
    property ResultText;
    property OnLateValue;
    property OnLateObject;
    property OnCheckToken;
  end;

//==============================================================================
procedure Register;

//==============================================================================
function GlobalExpressIt: TExpressIt;
function EvaluateIt(const aExpression: string): variant;

//==============================================================================
implementation

//==============================================================================
uses
  MiscSupport, ParserSupport, Expression,
  ExpressionOpersStandard, ExpressionFuncsStandard,
  ExpressionOpersStrings, ExpressionFuncsStrings, ExpressionFuncsStringsExtended,
  ExpressionFuncsReals, ExpressionFuncsRealsExtended,
  ExpressionFuncsPathFile, ExpressionFuncsPathFileExtended,
  ExpressionFuncsDateTime, ExpressionFuncsDateTimeExtended, ExpressionFuncsDateTimeSpan,
  ExpressionFuncsDialogs;

//==============================================================================
procedure Register;
begin
  RegisterComponents('Additional', [TExpressIt]);
end;

//==============================================================================
// TExpressIt = class(TCustomExpressIt)
//==============================================================================
constructor TExpressIt.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // add the lexicons
  ExpressionOpersStandard.RegisterLexicon(Self);
  ExpressionFuncsStandard.RegisterLexicon(Self);

  ExpressionOpersStrings.RegisterLexicon(Self);
  ExpressionFuncsStrings.RegisterLexicon(Self);
  ExpressionFuncsStringsExtended.RegisterLexicon(Self);

  ExpressionFuncsReals.RegisterLexicon(Self);
  ExpressionFuncsRealsExtended.RegisterLexicon(Self);

  ExpressionFuncsPathFile.RegisterLexicon(Self);
  ExpressionFuncsPathFileExtended.RegisterLexicon(Self);

  ExpressionFuncsDateTime.RegisterLexicon(Self);
  ExpressionFuncsDateTimeExtended.RegisterLexicon(Self);
  ExpressionFuncsDateTimeSpan.RegisterLexicon(Self);

  ExpressionFuncsDialogs.RegisterLexicon(Self);
end;

//==============================================================================
var
  vExpressIt: TExpressIt = nil;
  vUnitOk: boolean = True;
function GlobalExpressIt: TExpressIt;
begin
  // have we made one yet?
  if vUnitOk and
     not Assigned(vExpressIt) then
    vExpressIt := TExpressIt.Create(nil);

  // return it!
  Result := vExpressIt;
end;
function EvaluateIt(const aExpression: string): variant;
begin
  // all is swell?
  if GlobalExpressIt <> nil then
    Result := GlobalExpressIt.Evaluate(aExpression)
  else
    Result := Unassigned;
end;

//==============================================================================
initialization
  // nothing to do here!
finalization
  vUnitOk := False;
  Dump(vExpressIt);
end.
