unit ExpressionDatasetSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, MiscSupport, DB, DBTables,
  ExpressionParser,Variants;

//==============================================================================
function ExprDataSetCount(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere: string): integer;

//==============================================================================
function ExprDataSetTotal(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere, aField: string): variant;

//==============================================================================
function ExprDataSetLocate(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere: string; aTestCurrentRecord: boolean): boolean;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport;

//==============================================================================
//==============================================================================
function ExprDataSetCount(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere: string): integer;
var
  vFirstRecord: boolean;
begin
  // assume nothing
  Result := 0;

  // to the beginning
  aDataset.First;
  vFirstRecord := True;

  // keep going until we run out of things, we will end up on the last valid record
  while ExprDataSetLocate(aExpressIt, aDataset, aWhere, vFirstRecord) do begin
    Inc(Result);

    // no longer on the first record
    vFirstRecord := False;
  end;
end;

//==============================================================================
//==============================================================================
function ExprDataSetTotal(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere, aField: string): variant;
var
  vField: TField;
  vValue: variant;
  vFirstRecord: boolean;
begin
  // assume nothing
  VarClear(Result);

  // does the field exist?
  vField := aDataset.FindField(aField);
  if vField <> nil then begin

    // to the beginning
    aDataset.First;
    vFirstRecord := True;

    // keep going until we run out of things, we will end up on the last valid record
    while ExprDataSetLocate(aExpressIt, aDataset, aWhere, vFirstRecord) do begin

      // add things together
      vValue := vField.AsVariant;
      Result := VarAsType(Result, VarType(vValue)) + vValue;

      // no longer on the first record
      vFirstRecord := False;
    end;
  end;
end;

//==============================================================================
//==============================================================================
function ExprDataSetLocate(aExpressIt: TCustomExpressIt; aDataset: TDataset;
  const aWhere: string; aTestCurrentRecord: boolean): boolean;
var
  vBookmark: TBookmark;

  // what did he say?
  procedure ForgetBookmark;
  begin
    if vBookmark <> nil then begin
      aDataset.FreeBookmark(vBookmark);
      vBookmark := nil;
    end;
  end;

  // just in case we need to back up
  procedure JumpToBookmark;
  begin
    if vBookmark <> nil then
      aDataset.GotoBookmark(vBookmark);
    ForgetBookmark;
  end;
begin
  // assume we don't move
  Result := False;
  vBookmark := nil;

  // clear any errors
  ClearResult;

  // something to do?
  if not aDataSet.EOF then begin

    // where do we go back to on failure?
    vBookmark := aDataset.GetBookmark;

    // move to the next record, we desired
    if not aTestCurrentRecord then
      aDataSet.Next;

    // ok lets do it then
    while not aDataSet.EOF do begin

      // does this one match?
      try
        if aExpressIt.Evaluate(aWhere) then begin
          Result := True;
          break;

        // if not then move on
        end else if not AnyErrors then
          aDataset.Next;

      // oops something bad happened
      except
        JumpToBookmark;
        raise;
      end;
    end;
  end;

  // did we move? and shouldn't of?
  if not Result then
    JumpToBookmark
  else
    ForgetBookmark;
end;

end.
