unit ExpressionFuncsDialogs;

// ExpressionFuncsDialogs
// boolean  = AskQuestion(caption, body)         // MessageDlg: Ok
// boolean  = AskConfirmation(caption, body)     // MessageDlg: Ok &Cancel
// string   = AskForText(caption, body, string)  // InputBox
// string   = AskForFile(caption, string)        // TOpenDialog
// string   = AskForPath(string)                 // SelectDirectory

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs, FileCtrl,
  ExpressionStack, ExpressionLexicon, ExpressionParser;

//==============================================================================
type
  //============================================================================
  TExpressionFuncAskQuestion = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAskConfirmation = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAskForText = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAskForFile = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAskForPath = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  MiscSupport, ResultSupport, FileInfoSupport, FileSupport;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncAskQuestion);
    Add(TExpressionFuncAskConfirmation);
    Add(TExpressionFuncAskForText);
    Add(TExpressionFuncAskForFile);
    Add(TExpressionFuncAskForPath);
  end;
end;

//==============================================================================
// TExpressionFuncAskQuestion = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAskQuestion.Symbol: string;
begin
  Result := 'AskQuestion';
end;
procedure TExpressionFuncAskQuestion.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(MessageDlg(Pop, mtInformation, [mbOk], 0) = mrOk);
end;

//==============================================================================
// TExpressionFuncAskConfirmation = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAskConfirmation.Symbol: string;
begin
  Result := 'AskConfirmation';
end;
procedure TExpressionFuncAskConfirmation.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(MessageDlg(Pop, mtInformation, [mbYes, mbNo], 0) = mrYes);
end;

//==============================================================================
// TExpressionFuncAskForText = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncAskForText.Symbol: string;
begin
  Result := 'AskForText';
end;
procedure TExpressionFuncAskForText.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPrompt, vDefault: string;
begin
  with AStack do begin
    vDefault := Pop;
    vPrompt := Pop;
    Push(InputBox(Pop, vPrompt, vDefault));
  end;
end;

//==============================================================================
// TExpressionFuncAskForFile = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncAskForFile.Symbol: string;
begin
  Result := 'AskForFile';
end;
procedure TExpressionFuncAskForFile.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPath, vBody: string;
begin
  with AStack do begin
    vBody := Pop;
    vPath := Pop;
    with TOpenDialog.Create(Application) do try
      Title := vBody;
      InitialDir := ExtractFilePath(vPath);
      Filename := vPath;
      if Execute then
        Push(Filename)
      else
        Push('');
    finally
      Free;
    end;
  end;
end;

//==============================================================================
// TExpressionFuncAskForPath = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncAskForPath.Symbol: string;
begin
  Result := 'AskForPath';
end;
procedure TExpressionFuncAskForPath.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPath: string;
begin
  with AStack do begin
    vPath := Pop;
    if SelectDirectory(vPath, [], 0) then
      Push(vPath)
    else
      Push('');
  end;
end;

end.
