unit ExpressionFuncsPathFile;

// ExpressionFuncsPathFile
// filepath    = Exists, DriveOf, PathOf, NameOf, ShortNameOf, FullNameOf
//               ExtensionOf, SizeOf, LastChangeOf, AttributesOf, PathAndFile,
//               WithExtension

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser;

//==============================================================================
type
  //============================================================================
  // filepath
  TExpressionFuncExists = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncPathOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncNameOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncShortnameOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncFullnameOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncDriveOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncExtensionOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncWithExtension = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncLastChangeOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSizeOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAttributesOf = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncPathAndFile = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  MiscSupport, ResultSupport, FileInfoSupport, FileSupport;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncExists);
    Add(TExpressionFuncPathOf);
    Add(TExpressionFuncNameOf);
    Add(TExpressionFuncShortnameOf);
    Add(TExpressionFuncFullnameOf);
    Add(TExpressionFuncDriveOf);
    Add(TExpressionFuncExtensionOf);
    Add(TExpressionFuncWithExtension);
    Add(TExpressionFuncLastChangeOf);
    Add(TExpressionFuncSizeOf);
    Add(TExpressionFuncAttributesOf);
    Add(TExpressionFuncPathAndFile);
  end;
end;

//==============================================================================
// TExpressionFuncExists = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncExists.Symbol: string;
begin
  Result := 'Exists';
end;
procedure TExpressionFuncExists.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Valid);
end;

//==============================================================================
// TExpressionFuncPathOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncPathOf.Symbol: string;
begin
  Result := 'PathOf';
end;
procedure TExpressionFuncPathOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Path);
end;

//==============================================================================
// TExpressionFuncNameOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncNameOf.Symbol: string;
begin
  Result := 'NameOf';
end;
procedure TExpressionFuncNameOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Name);
end;

//==============================================================================
// TExpressionFuncShortnameOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncShortnameOf.Symbol: string;
begin
  Result := 'ShortnameOf';
end;
procedure TExpressionFuncShortnameOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Shortname);
end;

//==============================================================================
// TExpressionFuncFullnameOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncFullnameOf.Symbol: string;
begin
  Result := 'FullnameOf';
end;
procedure TExpressionFuncFullnameOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Full);
end;

//==============================================================================
// TExpressionFuncDriveOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncDriveOf.Symbol: string;
begin
  Result := 'DriveOf';
end;
procedure TExpressionFuncDriveOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Drive);
end;

//==============================================================================
// TExpressionFuncExtensionOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncExtensionOf.Symbol: string;
begin
  Result := 'ExtensionOf';
end;
procedure TExpressionFuncExtensionOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Extension);
end;

//==============================================================================
// TExpressionFuncWithExtension = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncWithExtension.Symbol: string;
begin
  Result := 'WithExtension';
end;
procedure TExpressionFuncWithExtension.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vExtension: string;
begin
  with AStack do begin
    vExtension := Pop;
    Push(ChangeFileExt(Pop, '.' + vExtension));
  end;
end;

//==============================================================================
// TExpressionFuncLastChangeOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncLastChangeOf.Symbol: string;
begin
  Result := 'LastChangeOf';
end;
procedure TExpressionFuncLastChangeOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).LastChangeAsDateTime);
end;

//==============================================================================
// TExpressionFuncSizeOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncSizeOf.Symbol: string;
begin
  Result := 'SizeOf';
end;
procedure TExpressionFuncSizeOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).Size);
end;

//==============================================================================
// TExpressionFuncAttributesOf = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAttributesOf.Symbol: string;
begin
  Result := 'AttributesOf';
end;
procedure TExpressionFuncAttributesOf.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(FileInfoFor(Pop).AttributesAsText);
end;

//==============================================================================
// TExpressionFuncPathAndFile = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncPathAndFile.Symbol: string;
begin
  Result := 'PathAndFile';
end;
procedure TExpressionFuncPathAndFile.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vFile: string;
begin
  with AStack do begin
    vFile := Pop;
    Push(PathAndFile(Pop, vFile));
  end;
end;

end.
