unit ExpressionFuncsRealsExtended;

// ExpressionFuncsRealsExtended
// double      = GradToDeg, GradToCycle, DegToGrad,
//               DegToCycle, CycleToGrad, CycleToDeg,
//               CoTan, ArcTan2, Hypot, LnXP1,
//               RandG, LdExp,
//               Mean, Sum, SumOfSquares, StdDev, PopNStdDev,
//               Variance, PopNVariance, TotalVariance, Norm

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser;

//==============================================================================
type
  //============================================================================
  // values
  TExpressionFuncGradToDeg = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncGradToCycle = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncDegToGrad = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncDegToCycle = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncCycleToGrad = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncCycleToDeg = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncCoTan = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncLnXP1 = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncRandG = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncArcTan2 = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncHypot = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; aStack: TExpressionStack); override;
  end;
  TExpressionFuncMean = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSum = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSumOfSquares = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncStdDev = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncPopNStdDev = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncVariance = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncPopNVariance = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncTotalVariance = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncNorm = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, Math;

//==============================================================================
const
  cMaxCount = 16;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncGradToDeg);
    Add(TExpressionFuncGradToCycle);
    Add(TExpressionFuncDegToGrad);
    Add(TExpressionFuncDegToCycle);
    Add(TExpressionFuncCycleToGrad);
    Add(TExpressionFuncCycleToDeg);
    Add(TExpressionFuncCoTan);
    Add(TExpressionFuncLnXP1);
    Add(TExpressionFuncRandG);
    Add(TExpressionFuncArcTan2);
    Add(TExpressionFuncHypot);
    Add(TExpressionFuncMean);
    Add(TExpressionFuncSum);
    Add(TExpressionFuncSumOfSquares);
    Add(TExpressionFuncStdDev);
    Add(TExpressionFuncPopNStdDev);
    Add(TExpressionFuncVariance);
    Add(TExpressionFuncPopNVariance);
    Add(TExpressionFuncTotalVariance);
    Add(TExpressionFuncNorm);
  end;
end;

//==============================================================================
// TExpressionFuncGradToDeg = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncGradToDeg.Symbol: string;
begin
  Result := 'GradToDeg';
end;
procedure TExpressionFuncGradToDeg.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToDeg(GradToRad(Pop)));
end;

//==============================================================================
// TExpressionFuncGradToCycle = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncGradToCycle.Symbol: string;
begin
  Result := 'GradToCycle';
end;
procedure TExpressionFuncGradToCycle.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToCycle(GradToRad(Pop)));
end;

//==============================================================================
// TExpressionFuncDegToGrad = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncDegToGrad.Symbol: string;
begin
  Result := 'DegToGrad';
end;
procedure TExpressionFuncDegToGrad.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToGrad(DegToRad(Pop)));
end;

//==============================================================================
// TExpressionFuncDegToCycle = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncDegToCycle.Symbol: string;
begin
  Result := 'DegToCycle';
end;
procedure TExpressionFuncDegToCycle.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToCycle(DegToRad(Pop)));
end;

//==============================================================================
// TExpressionFuncCycleToGrad = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncCycleToGrad.Symbol: string;
begin
  Result := 'CycleToGrad';
end;
procedure TExpressionFuncCycleToGrad.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToGrad(CycleToRad(Pop)));
end;

//==============================================================================
// TExpressionFuncCycleToDeg = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncCycleToDeg.Symbol: string;
begin
  Result := 'CycleToDeg';
end;
procedure TExpressionFuncCycleToDeg.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(RadToDeg(CycleToRad(Pop)));
end;


//==============================================================================
// TExpressionFuncCoTan = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncCoTan.Symbol: string;
begin
  Result := 'CoTan';
end;
procedure TExpressionFuncCoTan.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(CoTan(Pop));
end;

//==============================================================================
// TExpressionFuncLnXP1 = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncLnXP1.Symbol: string;
begin
  Result := 'LnXP1';
end;
procedure TExpressionFuncLnXP1.Execute(Sender: TObject; aStack: TExpressionStack);
begin
  with aStack do
    Push(LnXP1(Pop));
end;

//==============================================================================
// TExpressionFuncRandG = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncRandG.Symbol: string;
begin
  Result := 'Ran';
end;
procedure TExpressionFuncRandG.Execute(Sender: TObject; aStack: TExpressionStack);
var
  vValue: extended;
begin
  with aStack do begin
    vValue := Pop;
    Push(RandG(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionFuncArcTan2 = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncArcTan2.Symbol: string;
begin
  Result := 'ArcTan2';
end;
procedure TExpressionFuncArcTan2.Execute(Sender: TObject; aStack: TExpressionStack);
var
  vValue: extended;
begin
  with aStack do begin
    vValue := Pop;
    Push(ArcTan2(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionFuncHypot = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncHypot.Symbol: string;
begin
  Result := 'Hypot';
end;
procedure TExpressionFuncHypot.Execute(Sender: TObject; aStack: TExpressionStack);
var
  vValue: extended;
begin
  with aStack do begin
    vValue := Pop;
    Push(Hypot(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionFuncMean = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncMean.Symbol: string;
begin
  Result := 'Mean';
end;
procedure TExpressionFuncMean.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(Mean(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncSum = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncSum.Symbol: string;
begin
  Result := 'Sum';
end;
procedure TExpressionFuncSum.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(Sum(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncSumOfSquares = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncSumOfSquares.Symbol: string;
begin
  Result := 'SumOfSquares';
end;
procedure TExpressionFuncSumOfSquares.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(SumOfSquares(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncStdDev = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncStdDev.Symbol: string;
begin
  Result := 'StdDev';
end;
procedure TExpressionFuncStdDev.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(StdDev(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncPopNStdDev = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncPopNStdDev.Symbol: string;
begin
  Result := 'PopNStdDev';
end;
procedure TExpressionFuncPopNStdDev.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(PopNStdDev(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncVariance = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncVariance.Symbol: string;
begin
  Result := 'Variance';
end;
procedure TExpressionFuncVariance.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(Variance(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncPopNVariance = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncPopNVariance.Symbol: string;
begin
  Result := 'PopNVariance';
end;
procedure TExpressionFuncPopNVariance.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(PopNVariance(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncTotalVariance = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncTotalVariance.Symbol: string;
begin
  Result := 'TotalVariance';
end;
procedure TExpressionFuncTotalVariance.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(TotalVariance(Slice(vValues, vCount)));
    end;
  end;
end;

//==============================================================================
// TExpressionFuncNorm = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncNorm.Symbol: string;
begin
  Result := 'Norm';
end;
procedure TExpressionFuncNorm.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValues: array [0..cMaxCount-1] of double;
  vIndex, vCount: integer;
begin
  with aStack do begin
    vCount := Pop;
    if vCount > cMaxCount then
      SetResult(DISP_E_BADPARAMCOUNT)
    else begin
      for vIndex := 0 to vCount-1 do
        vValues[vIndex] := Pop;
      Push(Norm(Slice(vValues, vCount)));
    end;
  end;
end;

end.
