unit ExpressionFuncsStandard;

// values      = True, False, MinInteger, MaxInteger
// conversion  = AsString, AsInteger, AsBoolean, AsReal, IsEmpty, IsNull
// ranges      = Max, Min, Average, Ranged, InRange
// math        = Round, Trunc, Absolute, Sign, Square, SquareRoot
// random      = RandomRange, RandomFrom, Random
// conditional = IfThen

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser,Variants;

//==============================================================================
type
  //============================================================================
  // value
  TExpressionFuncTrue = class(TExpressionFunction)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncFalse = class(TExpressionFunction)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncMinInteger = class(TExpressionFunction)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncMaxInteger = class(TExpressionFunction)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // conversion
  TExpressionFuncAsString = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAsInteger = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAsBoolean = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAsReal = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncIsEmpty = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncIsNull = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // ranges
  TExpressionFuncMax = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncMin = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAverage = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncRanged = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncInRange = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncRound = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncTrunc = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncAbsolute = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSign = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSquare = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncSquareRoot = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // random
  TExpressionFuncRandomRange = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncRandomFrom = class(TExpressionFunctionWithNArg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncRandom = class(TExpressionFunction)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // conditional
  TExpressionFuncIfThen = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscSupport, Expression;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncTrue);
    Add(TExpressionFuncFalse);
    Add(TExpressionFuncMinInteger);
    Add(TExpressionFuncMaxInteger);
    Add(TExpressionFuncAsString);
    Add(TExpressionFuncAsInteger);
    Add(TExpressionFuncAsBoolean);
    Add(TExpressionFuncAsReal);
    Add(TExpressionFuncIsEmpty);
    Add(TExpressionFuncIsNull);
    Add(TExpressionFuncMax);
    Add(TExpressionFuncMin);
    Add(TExpressionFuncAverage);
    Add(TExpressionFuncRanged);
    Add(TExpressionFuncInRange);
    Add(TExpressionFuncRound);
    Add(TExpressionFuncTrunc);
    Add(TExpressionFuncAbsolute);
    Add(TExpressionFuncSign);
    Add(TExpressionFuncSquare);
    Add(TExpressionFuncSquareRoot);
    Add(TExpressionFuncRandomRange);
    Add(TExpressionFuncRandomFrom);
    Add(TExpressionFuncRandom);
    Add(TExpressionFuncIfThen);
  end;
end;

//==============================================================================
// TExpressionFuncTrue = class(TExpressionFunction)
//==============================================================================
class function TExpressionFuncTrue.Symbol: string;
begin
  Result := 'True';
end;
procedure TExpressionFuncTrue.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  AStack.Push(True);
end;

//==============================================================================
// TExpressionFuncFalse = class(TExpressionFunction)
//==============================================================================
class function TExpressionFuncFalse.Symbol: string;
begin
  Result := 'False';
end;
procedure TExpressionFuncFalse.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  AStack.Push(False);
end;

//==============================================================================
// TExpressionFuncMinInteger = class(TExpressionFunction)
//==============================================================================
class function TExpressionFuncMinInteger.Symbol: string;
begin
  Result := 'MinInteger';
end;
procedure TExpressionFuncMinInteger.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  AStack.Push(Low(Integer));
end;

//==============================================================================
// TExpressionFuncMaxInteger = class(TExpressionFunction)
//==============================================================================
class function TExpressionFuncMaxInteger.Symbol: string;
begin
  Result := 'MaxInteger';
end;
procedure TExpressionFuncMaxInteger.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  AStack.Push(High(Integer));
end;

//==============================================================================
// TExpressionFuncAsString = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAsString.Symbol: string;
begin
  Result := 'AsString';
end;
procedure TExpressionFuncAsString.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarAsType(Pop, varString));
end;

//==============================================================================
// TExpressionFuncAsInteger = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAsInteger.Symbol: string;
begin
  Result := 'AsInteger';
end;
procedure TExpressionFuncAsInteger.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarAsType(Pop, varInteger));
end;

//==============================================================================
// TExpressionFuncAsBoolean = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAsBoolean.Symbol: string;
begin
  Result := 'AsBoolean';
end;
procedure TExpressionFuncAsBoolean.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarAsType(Pop, varBoolean));
end;

//==============================================================================
// TExpressionFuncAsReal = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAsReal.Symbol: string;
begin
  Result := 'AsReal';
end;
procedure TExpressionFuncAsReal.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarAsType(Pop, varDouble));
end;

//==============================================================================
// TExpressionFuncIsEmpty = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncIsEmpty.Symbol: string;
begin
  Result := 'IsEmpty';
end;
procedure TExpressionFuncIsEmpty.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarIsEmpty(Pop));
end;

//==============================================================================
// TExpressionFuncIsNull = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncIsNull.Symbol: string;
begin
  Result := 'IsNull';
end;
procedure TExpressionFuncIsNull.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarIsNull(Pop));
end;

//==============================================================================
// TExpressionFuncMax = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncMax.Symbol: string;
begin
  Result := 'Max';
end;
procedure TExpressionFuncMax.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue1, vValue2: variant;
  vCount: integer;
begin
  with AStack do begin
    vCount := Pop;
    vValue1 := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vValue1, [varSmallInt, varInteger, varSingle, varDouble,
                             varCurrency, varDate, varOleStr, varString]);
    {$endif}
    Dec(vCount);
    while vCount > 0 do begin
      vValue2 := Pop;
      {$ifdef StrongChecking}
      VarAssureTypes(vValue2, [varSmallInt, varInteger, varSingle, varDouble,
                               varCurrency, varDate, varOleStr, varString]);
      {$endif}
      if vValue1 < vValue2 then
        vValue1 := vValue2;
      Dec(vCount);
    end;
    Push(vValue1);
  end;
end;

//==============================================================================
// TExpressionFuncMin = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncMin.Symbol: string;
begin
  Result := 'Min';
end;
procedure TExpressionFuncMin.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue1, vValue2: variant;
  vCount: integer;
begin
  with AStack do begin
    vCount := Pop;
    vValue1 := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vValue1, [varSmallInt, varInteger, varSingle, varDouble,
                             varCurrency, varDate, varOleStr, varString]);
    {$endif}
    Dec(vCount);
    while vCount > 0 do begin
      vValue2 := Pop;
      {$ifdef StrongChecking}
      VarAssureTypes(vValue2, [varSmallInt, varInteger, varSingle, varDouble,
                               varCurrency, varDate, varOleStr, varString]);
      {$endif}
      if vValue1 > vValue2 then
        vValue1 := vValue2;
      Dec(vCount);
    end;
    Push(vValue1);
  end;
end;

//==============================================================================
// TExpressionFuncAverage = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncAverage.Symbol: string;
begin
  Result := 'Average';
end;
procedure TExpressionFuncAverage.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: extended;
  vCount: integer;
begin
  with AStack do begin
    vCount := Pop;
    vValue := Pop;
    Dec(vCount);
    while vCount > 0 do begin
      vValue := (vValue + VarAsType(Pop, varDouble)) / 2;
      Dec(vCount);
    end;
    Push(vValue);
  end;
end;

//==============================================================================
// TExpressionFuncRanged = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncRanged.Symbol: string;
begin
  Result := 'Ranged';
end;
procedure TExpressionFuncRanged.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue, vMin, vMax: variant;
begin
  with AStack do begin
    vMax := Pop;
    vMin := Pop;
    if vMin > vMax then begin
      vValue := vMin;
      vMin := vMax;
      vMax := vValue;
    end;
    vValue := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vMax, [varSmallInt, varInteger, varSingle, varDouble,
                          varCurrency, varDate, varOleStr, varString]);
    VarAssureTypes(vMin, [varSmallInt, varInteger, varSingle, varDouble,
                          varCurrency, varDate, varOleStr, varString]);
    VarAssureTypes(vValue, [varSmallInt, varInteger, varSingle, varDouble,
                            varCurrency, varDate, varOleStr, varString]);
    {$endif}
    if vValue < vMin then
      vValue := vMin;
    if vValue > vMax then
      vValue := vMax;
    Push(vValue);
  end;
end;

//==============================================================================
// TExpressionFuncInRange = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncInRange.Symbol: string;
begin
  Result := 'InRange';
end;
procedure TExpressionFuncInRange.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue, vMin, vMax: variant;
begin
  with AStack do begin
    vMax := Pop;
    vMin := Pop;
    if vMin > vMax then begin
      vValue := vMin;
      vMin := vMax;
      vMax := vValue;
    end;
    vValue := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vMax, [varSmallInt, varInteger, varSingle, varDouble,
                          varCurrency, varDate, varOleStr, varString]);
    VarAssureTypes(vMin, [varSmallInt, varInteger, varSingle, varDouble,
                          varCurrency, varDate, varOleStr, varString]);
    VarAssureTypes(vValue, [varSmallInt, varInteger, varSingle, varDouble,
                            varCurrency, varDate, varOleStr, varString]);
    {$endif}
    Push((vValue >= vMin) and (vValue <= vMax));
  end;
end;

//==============================================================================
// TExpressionFuncRound = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncRound.Symbol: string;
begin
  Result := 'Round';
end;
procedure TExpressionFuncRound.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: comp;
begin
  with AStack do begin
    vValue := Round(Pop);
    Push(vValue);
  end;
end;

//==============================================================================
// TExpressionFuncTrunc = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncTrunc.Symbol: string;
begin
  Result := 'Trunc';
end;
procedure TExpressionFuncTrunc.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: comp;
begin
  with AStack do begin
    vValue := Trunc(Pop);
    Push(vValue);
  end;
end;

//==============================================================================
// TExpressionFuncAbsolute = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncAbsolute.Symbol: string;
begin
  Result := 'Absolute';
end;
procedure TExpressionFuncAbsolute.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: double;
begin
  with AStack do begin
    vValue := Pop;
    Push(Abs(vValue));
  end;
end;

//==============================================================================
// TExpressionFuncSign = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncSign.Symbol: string;
begin
  Result := 'Sign';
end;
procedure TExpressionFuncSign.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: double;
begin
  with AStack do begin
    vValue := Pop;
    if vValue > 0 then
      vValue := 1
    else if vValue < 0 then
      vValue := -1
    else
      vValue := 0;
    Push(vValue);
  end;
end;

//==============================================================================
// TExpressionFuncSquare = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncSquare.Symbol: string;
begin
  Result := 'Square';
end;
procedure TExpressionFuncSquare.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(Sqr(Pop));
end;

//==============================================================================
// TExpressionFuncSquareRoot = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncSquareRoot.Symbol: string;
begin
  Result := 'SquareRoot';
end;
procedure TExpressionFuncSquareRoot.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(Sqrt(Pop));
end;

//==============================================================================
// TExpressionFuncRandomRange = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncRandomRange.Symbol: string;
begin
  Result := 'RandomRange';
end;
procedure TExpressionFuncRandomRange.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vFirst, vSecond: integer;
begin
  with AStack do begin
    vSecond := Pop;
    vFirst := Pop;
    if vFirst > vSecond then
      Push(Random(vFirst-vSecond)+vSecond)
    else
      Push(Random(vSecond-vFirst)+vFirst);
  end;
end;

//==============================================================================
// TExpressionFuncRandomFrom = class(TExpressionFunctionWithNArg)
//==============================================================================
class function TExpressionFuncRandomFrom.Symbol: string;
begin
  Result := 'RandomFrom';
end;
procedure TExpressionFuncRandomFrom.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue, vResult: variant;
  vIndex, vCount, vHit: integer;
begin
  with AStack do begin
    vCount := Pop;
    vHit := Random(vCount)+1;
    vResult := varUnknown;
    for vIndex := 1 to vCount do begin
      vValue := Pop;
      if vIndex = vHit then
        vResult := vValue;
    end;
    Assert(vResult <> varUnknown, ResultTextFor(E_UNEXPECTED));
    Push(vResult);
  end;
end;

//==============================================================================
// TExpressionFuncRandom = class(TExpressionFunction)
//==============================================================================
class function TExpressionFuncRandom.Symbol: string;
begin
  Result := 'Random';
end;
procedure TExpressionFuncRandom.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(Random);
end;

//==============================================================================
// TExpressionFuncIfThen = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncIfThen.Symbol: string;
begin
  Result := 'IfThen';
end;
procedure TExpressionFuncIfThen.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vThen, vElse: variant;
begin
  with AStack do begin
    vElse := Pop;
    vThen := Pop;
    if VarAsType(Pop, varBoolean) then
      Push(vThen)
    else
      Push(vElse);
  end;
end;

end.
