unit ExpressionFuncsStrings;

//==============================================================================
// ExpressionFuncsStrings
// string      = PositionOf, PartOf, Insert, Remove, Length, Format, Uppercase,
//               Lowercase

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser;

//==============================================================================
type
  //============================================================================
  // string
  TExpressionFuncPositionOf = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncPartOf = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncInsert = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncRemove = class(TExpressionFunctionWith3Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncLength = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncFormat = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncUppercase = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncLowercase = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscSupport, Expression;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncPositionOf);
    Add(TExpressionFuncPartOf);
    Add(TExpressionFuncInsert);
    Add(TExpressionFuncRemove);
    Add(TExpressionFuncLength);
    Add(TExpressionFuncFormat);
    Add(TExpressionFuncUppercase);
    Add(TExpressionFuncLowercase);
  end;
end;

//==============================================================================
// TExpressionFuncPositionOf = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncPositionOf.Symbol: string;
begin
  Result := 'PositionOf';
end;
procedure TExpressionFuncPositionOf.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vSubstring: string;
begin
  with AStack do begin
    vSubstring := Pop;
    Push(AnsiPos(Pop, vSubstring));
  end;
end;

//==============================================================================
// TExpressionFuncPartOf = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncPartOf.Symbol: string;
begin
  Result := 'PartOf';
end;
procedure TExpressionFuncPartOf.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vAt, vLength: integer;
begin
  with AStack do begin
    vLength := Pop;
    vAt := Pop;
    Push(Copy(Pop, vAt, vLength));
  end;
end;

//==============================================================================
// TExpressionFuncInsert = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncInsert.Symbol: string;
begin
  Result := 'Insert';
end;
procedure TExpressionFuncInsert.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText: string;
  vAt: integer;
begin
  with AStack do begin
    vAt := Pop;
    vText := Pop;
    Insert(Pop, vText, vAt);
    Push(vText);
  end;
end;

//==============================================================================
// TExpressionFuncRemove = class(TExpressionFunctionWith3Arg)
//==============================================================================
class function TExpressionFuncRemove.Symbol: string;
begin
  Result := 'Remove';
end;
procedure TExpressionFuncRemove.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText: string;
  vAt, vLength: integer;
begin
  with AStack do begin
    vLength := Pop;
    vAt := Pop;
    vText := Pop;
    Delete(vText, vAt, vLength);
    Push(vText);
  end;
end;

//==============================================================================
// TExpressionFuncLength = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncLength.Symbol: string;
begin
  Result := 'Length';
end;
procedure TExpressionFuncLength.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(Length(Pop));
end;

//==============================================================================
// TExpressionFuncFormat = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncFormat.Symbol: string;
begin
  Result := 'Format';
end;
procedure TExpressionFuncFormat.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPattern: string;
begin
  with AStack do begin
    vPattern := Pop;
    Push(VarFormat(vPattern, Pop));
  end;
end;

//==============================================================================
// TExpressionFuncUppercase = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncUppercase.Symbol: string;
begin
  Result := 'Uppercase';
end;
procedure TExpressionFuncUppercase.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(AnsiUppercase(Pop));
end;

//==============================================================================
// TExpressionFuncLowercase = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncLowercase.Symbol: string;
begin
  Result := 'Lowercase';
end;
procedure TExpressionFuncLowercase.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(AnsiLowercase(Pop));
end;

end.
