unit ExpressionFuncsStringsExtended;

//==============================================================================
// ExpressionFuncsStringsExtended
// string      = Evaluate, Quote, QuoteWith, Dequote, DequoteWith,
//               FieldCount, FieldOf, WordCount, WordOf, LineCount, LineOf

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser;

//==============================================================================
type
  //============================================================================
  // string
  TExpressionFuncEvaluate = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncFieldCount = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncFieldOf = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncQuote = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncQuoteWith = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncDequote = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncDequoteWith = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncWordCount = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncWordOf = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncLineCount = class(TExpressionFunctionWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionFuncLineOf = class(TExpressionFunctionWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscSupport, StringSupport, Expression;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.FunctionLexicon do begin
    Add(TExpressionFuncEvaluate);
    Add(TExpressionFuncFieldCount);
    Add(TExpressionFuncFieldOf);
    Add(TExpressionFuncQuote);
    Add(TExpressionFuncQuoteWith);
    Add(TExpressionFuncDequote);
    Add(TExpressionFuncDequoteWith);
    Add(TExpressionFuncWordCount);
    Add(TExpressionFuncWordOf);
    Add(TExpressionFuncLineCount);
    Add(TExpressionFuncLineOf);
  end;
end;

//==============================================================================
// TExpressionFuncEvaluate = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncEvaluate.Symbol: string;
begin
  Result := 'Evaluate';
end;
procedure TExpressionFuncEvaluate.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do begin
    Assert(Sender is TExpression, ResultTextFor(STG_E_INVALIDHANDLE));
    Assert(TExpression(Sender).Parser is TCustomExpressionParser, ResultTextFor(STG_E_INVALIDHANDLE));
    Push(TCustomExpressionParser(TExpression(Sender).Parser).Evaluate(Pop));
  end;
end;

//==============================================================================
// TExpressionFuncFieldCount = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncFieldCount.Symbol: string;
begin
  Result := 'FieldCount';
end;
procedure TExpressionFuncFieldCount.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(TextFieldCount(Pop));
end;

//==============================================================================
// TExpressionFuncFieldOf = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncFieldOf.Symbol: string;
begin
  Result := 'FieldOf';
end;
procedure TExpressionFuncFieldOf.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vIndex: integer;
begin
  with AStack do begin
    vIndex := Pop;
    Push(TextFieldOf(Pop, vIndex));
  end;
end;

//==============================================================================
// TExpressionFuncQuote = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncQuote.Symbol: string;
begin
  Result := 'Quote';
end;
procedure TExpressionFuncQuote.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(AnsiQuotedStr(Pop, vQuoteChar));
end;

//==============================================================================
// TExpressionFuncQuoteWith = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncQuoteWith.Symbol: string;
begin
  Result := 'QuoteWith';
end;
procedure TExpressionFuncQuoteWith.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vQuote: char;
begin
  with AStack do begin
    vQuote := String(Pop)[1];
    Push(AnsiQuotedStr(Pop, vQuote));
  end;
end;

//==============================================================================
// TExpressionFuncDequote = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncDequote.Symbol: string;
begin
  Result := 'Dequote';
end;
procedure TExpressionFuncDequote.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(AnsiDequoteStr(Pop, vQuoteChar));
end;

//==============================================================================
// TExpressionFuncDequoteWith = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncDequoteWith.Symbol: string;
begin
  Result := 'DequoteWith';
end;
procedure TExpressionFuncDequoteWith.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vQuote: char;
begin
  with AStack do begin
    vQuote := String(Pop)[1];
    Push(AnsiDequoteStr(Pop, vQuote));
  end;
end;

//==============================================================================
// TExpressionFuncWordCount = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncWordCount.Symbol: string;
begin
  Result := 'WordCount';
end;
procedure TExpressionFuncWordCount.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(TextWordCount(Pop));
end;

//==============================================================================
// TExpressionFuncWordOf = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncWordOf.Symbol: string;
begin
  Result := 'WordOf';
end;
procedure TExpressionFuncWordOf.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vIndex: integer;
begin
  with AStack do begin
    vIndex := Pop;
    Push(TextWordOf(Pop, vIndex));
  end;
end;

//==============================================================================
// TExpressionFuncLineCount = class(TExpressionFunctionWith1Arg)
//==============================================================================
class function TExpressionFuncLineCount.Symbol: string;
begin
  Result := 'LineCount';
end;
procedure TExpressionFuncLineCount.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(TextLineCount(Pop));
end;

//==============================================================================
// TExpressionFuncLineOf = class(TExpressionFunctionWith2Arg)
//==============================================================================
class function TExpressionFuncLineOf.Symbol: string;
begin
  Result := 'LineOf';
end;
procedure TExpressionFuncLineOf.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vIndex: integer;
begin
  with AStack do begin
    vIndex := Pop;
    Push(TextLineOf(Pop, vIndex));
  end;
end;

end.
