unit ExpressionLexicon;

//==============================================================================
interface

//==============================================================================
uses                              
  Windows, Messages, SysUtils, Classes, TokenSupport, ExpressionTokens;

//==============================================================================
type
  //============================================================================
  // basic types
  TExpressionOperation = class(TExpressionCode)
  public
    class function ArgCount: integer; virtual;
    class function Syntax: string; virtual; abstract;
  end;
  TExpressionOperationClass = class of TExpressionOperation;

  //============================================================================
  TExpressionOperationWith1Arg = class(TExpressionOperation)
  public
    class function ArgCount: integer; override;
    class function Syntax: string; override;
  end;
  TExpressionOperationWith2Arg = class(TExpressionOperation)
  public
    class function ArgCount: integer; override;
    class function Syntax: string; override;
  end;

  //============================================================================
  TExpressionFunction = class(TExpressionOperation)
  public
    class function Syntax: string; override;
  end;
  TExpressionFunctionWith1Arg = class(TExpressionFunction)
  public
    class function ArgCount: integer; override;
  end;
  TExpressionFunctionWith2Arg = class(TExpressionFunction)
  public
    class function ArgCount: integer; override;
  end;
  TExpressionFunctionWith3Arg = class(TExpressionFunction)
  public
    class function ArgCount: integer; override;
  end;
  TExpressionFunctionWithNArg = class(TExpressionFunction)
  public
    class function ArgCount: integer; override;
  end;

  //============================================================================
  TExpressionLexicon = class(TTokenLexicon)
  protected
    function GetOperation(index: integer): TExpressionOperationClass; virtual;
  public
    function Add(aClass: TExpressionOperationClass): integer; virtual;
    property Operations[index: integer]: TExpressionOperationClass
      read GetOperation; default;
    function Lookup(const ASymbol: string): TExpressionOperationClass;
    procedure ListSymbols(aList: TStrings); virtual;
    procedure ListSyntaxes(aList: TStrings); virtual;
  end;

  //============================================================================
  TExpressionOpers1Lexicon = class(TExpressionLexicon)
  public
    function Add(aClass: TExpressionOperationClass): integer; override;
  end;
  TExpressionOpers2Lexicon = class(TExpressionLexicon)
  public
    function Add(aClass: TExpressionOperationClass): integer; override;
  end;

//==============================================================================
implementation

//==============================================================================
uses
  MiscConstants, ExpressionConstants, ResultSupport;

//==============================================================================
// TExpressionOperation = class(TTokenObject)
//==============================================================================
class function TExpressionOperation.ArgCount: integer;
begin
  Result := 0;
end;
class function TExpressionOperationWith1Arg.ArgCount: integer;
begin
  Result := 1;
end;
class function TExpressionOperationWith1Arg.Syntax: string;
begin
  Result := Symbol + ' value';
end;
class function TExpressionOperationWith2Arg.ArgCount: integer;
begin
  Result := 2;
end;
class function TExpressionOperationWith2Arg.Syntax: string;
begin
  Result := 'value ' + Symbol + ' value';
end;
class function TExpressionFunction.Syntax: string;
var
  vIndex: integer;
begin
  // simple part
  Result := 'value = ' + Symbol;

  // are there arguments?
  if ArgCount <> 0 then begin

    // start things out
    Result := Result + '(';

    // a unlimited one?
    if ArgCount = cUnlimitedArgs then
      Result := Result + 'value [,...]'

    // if not then add the right number
    else begin
      Result := Result + 'value';
      for vIndex := 2 to ArgCount do
        Result := Result + ', value';
    end;

    // finish things off
    Result := Result + ')';
  end;
end;
class function TExpressionFunctionWith1Arg.ArgCount: integer;
begin
  Result := 1;
end;
class function TExpressionFunctionWith2Arg.ArgCount: integer;
begin
  Result := 2;
end;
class function TExpressionFunctionWith3Arg.ArgCount: integer;
begin
  Result := 3;
end;
class function TExpressionFunctionWithNArg.ArgCount: integer;
begin
  Result := cUnlimitedArgs;
end;

//==============================================================================
// TExpressionLexicon = class(TClassList)
//==============================================================================
function TExpressionLexicon.Add(aClass: TExpressionOperationClass): integer;
begin
  Result := inherited Add(aClass);
end;
function TExpressionLexicon.GetOperation(Index: integer): TExpressionOperationClass;
begin
  Result := TExpressionOperationClass(Classes[Index]);
end;
function TExpressionLexicon.Lookup(const aSymbol: string): TExpressionOperationClass;
begin
  Result := TExpressionOperationClass(inherited Lookup(aSymbol));
end;
procedure TExpressionLexicon.ListSymbols(aList: TStrings);
var
  vIndex: integer;
begin
  for vIndex := 0 to Count-1 do
    aList.AddObject(Operations[vIndex].Symbol, TObject(Operations[vIndex]));
end;
procedure TExpressionLexicon.ListSyntaxes(aList: TStrings);
var
  vIndex: integer;
begin
  for vIndex := 0 to Count-1 do
    aList.AddObject(Operations[vIndex].Syntax, TObject(Operations[vIndex]));
end;

//============================================================================
function TExpressionOpers1Lexicon.Add(aClass: TExpressionOperationClass): integer;
begin
  if aClass.InheritsFrom(TExpressionOperationWith1Arg) then
    Result := inherited Add(aClass)
  else begin
    Result := cIndexNotFound;
    SetResult(MK_E_NOTBINDABLE);
  end;
end;
function TExpressionOpers2Lexicon.Add(aClass: TExpressionOperationClass): integer;
begin
  if aClass.InheritsFrom(TExpressionOperationWith2Arg) then
    Result := inherited Add(aClass)
  else begin
    Result := cIndexNotFound;
    SetResult(MK_E_NOTBINDABLE);
  end;
end;

end.
