unit ExpressionObjectsDataset;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, MiscSupport, SymbolSupport,
  ExpressionObjects, DB, DBTables,Variants;

//==============================================================================
type
  //============================================================================
  TExpressionDatasetFields = class(TExpressionIndexedObject)
  protected
    function IndexedField: TField;

    function GetValue: variant;
    function GetDisplayName: string;
    function GetDisplayWidth: integer;
    function GetDisplayLabel: string;
    function GetKind: TFieldKind;
    function GetName: string;
    function GetNumber: integer;
    function GetSize: integer;
    function GetFieldType: TFieldType;
    function GetReadOnly: boolean;
    function GetRequired: boolean;
    function GetVisible: boolean;
    function GetOriginalName: string;
    function GetExists: boolean;

    function GetIsCalculated: boolean;
    function GetIsBlob: boolean;
    function GetIsNull: boolean;
    function GetIsIndex: boolean;
  public
    constructor Create(const aSymbol: string; aReference: TDataset);
    function SelfReference: boolean; override;
  published
    property Value: variant read GetValue;
    property DisplayName: string read GetDisplayName;
    property DisplayWidth: integer read GetDisplayWidth;
    property DisplayLabel: string read GetDisplayLabel;
    property Kind: TFieldKind read GetKind;
    property Name: string read GetName;
    property Number: integer read GetNumber;
    property Size: integer read GetSize;
    property FieldType: TFieldType read GetFieldType;
    property ReadOnly: boolean read GetReadOnly;
    property Required: boolean read GetRequired;
    property Visible: boolean read GetVisible;
    property OriginalName: string read GetOriginalName;
    property Exists: boolean read GetExists;

    property IsCalculated: boolean read GetIsCalculated;
    property IsBlob: boolean read GetIsBlob;
    property IsNull: boolean read GetIsNull;
    property IsIndex: boolean read GetIsIndex;
  end;

  //============================================================================
  TExpressionDataset = class(TExpressionDatasetFields)
  end;

  //============================================================================
  TExpressionDatasetRecord = class(TExpressionObject)
  protected
    function GetEOF: boolean;
    function GetBOF: boolean;
    function GetFieldCount: integer;
    function GetModified: boolean;
    function GetNumber: integer;
    function GetCount: integer;
    function GetSize: integer;
    function GetState: TDataSetState;
    function GetFound: boolean;
    function GetCanModify: boolean;
    function GetActive: boolean;
    function GetBookmark: string;
    function GetDatabaseName: string;
  public
    constructor Create(const aSymbol: string; aReference: TDataset);
    function SelfReference: boolean; override;
  published
    property Value: integer read GetNumber;
    property EOF: boolean read GetEOF;
    property BOF: boolean read GetBOF;
    property FieldCount: integer read GetFieldCount;
    property Modified: boolean read GetModified;
    property Number: integer read GetNumber;
    property Count: integer read GetCount;
    property Size: integer read GetSize;
    property State: TDataSetState read GetState;
    property Found: boolean read GetFound;
    property CanModify: boolean read GetCanModify;
    property Active: boolean read GetActive;
    property Bookmark: string read GetBookmark;
    property DatabaseName: string read GetDatabaseName;
  end;

  //============================================================================
  TExpressionDatasetDirect = class(TExpressionObject)
  protected
    function CheckName(const aProperty: string): string; override;
  public
    constructor Create(const aSymbol: string; aReference: TDataset);
    function Valid(const aProperty: string): boolean; override;
    function ValueOf(const aProperty: string): variant; override;
  end;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport;

//==============================================================================
// TExpressionDatasetFields = class(TExpressionIndexedObject)
//==============================================================================
constructor TExpressionDatasetFields.Create(const aSymbol: string; aReference: TDataset);
begin
  inherited Create(aSymbol, aReference);
end;
function TExpressionDatasetFields.SelfReference: boolean;
begin
  Result := True;
end;
function TExpressionDatasetFields.IndexedField: TField;
begin
  Result := TDataset(Reference).FieldByName(ItemIndex);
  Assert(Assigned(Result), ResultTextFor(TYPE_E_AMBIGUOUSNAME));
end;
function TExpressionDatasetFields.GetValue: variant;
begin
  Result := Unassigned;
  try
    Result := IndexedField.AsVariant;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetDisplayName: string;
begin
  Result := '';
  try
    Result := IndexedField.DisplayName;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetDisplayWidth: integer;
begin
  Result := -1;
  try
    Result := IndexedField.DisplayWidth;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetDisplayLabel: string;
begin
  Result := '';
  try
    Result := IndexedField.DisplayLabel;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetKind: TFieldKind;
begin
  Result := Low(TFieldKind);
  try
    Result := IndexedField.FieldKind;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetName: string;
begin
  Result := '';
  try
    Result := IndexedField.FieldName;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetNumber: integer;
begin
  Result := -1;
  try
    Result := IndexedField.FieldNo;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetSize: integer;
begin
  Result := -1;
  try
    Result := IndexedField.DataSize;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetFieldType: TFieldType;
begin
  Result := Low(TFieldType);
  try
    Result := IndexedField.DataType;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetReadOnly: boolean;
begin
  Result := False;
  try
    Result := IndexedField.CanModify;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetRequired: boolean;
begin
  Result := False;
  try
    Result := IndexedField.Required;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetVisible: boolean;
begin
  Result := False;
  try
    Result := IndexedField.Visible;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetOriginalName: string;
begin
  Result := '';
  try
    Result := IndexedField.Origin;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetExists: boolean;
begin
  Result := TDataset(Reference).FieldByName(ItemIndex) <> nil;
end;
function TExpressionDatasetFields.GetIsCalculated: boolean;
begin
  Result := False;
  try
    Result := IndexedField.Calculated;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetIsBlob: boolean;
begin
  Result := False;
  try
    Result := IndexedField.IsBlob;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetIsNull: boolean;
begin
  Result := False;
  try
    Result := IndexedField.IsNull;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetFields.GetIsIndex: boolean;
begin
  Result := False;
  try
    Result := IndexedField.IsIndexField;
  except
    SetErrorException;
  end;
end;

//==============================================================================
// TExpressionDatasetRecord = class(TExpressionObject)
//==============================================================================
constructor TExpressionDatasetRecord.Create(const aSymbol: string; aReference: TDataset);
begin
  inherited Create(aSymbol, aReference);
end;
function TExpressionDatasetRecord.SelfReference: boolean;
begin
  Result := True;
end;
function TExpressionDatasetRecord.GetEOF: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).EOF;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetBOF: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).BOF;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetFieldCount: integer;
begin
  Result := -1;
  try
    Result := TDataset(Reference).FieldCount;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetModified: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).Modified;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetNumber: integer;
begin
  Result := -1;
  try
    Result := TDataset(Reference).RecNo;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetCount: integer;
begin
  Result := -1;
  try
    Result := TDataset(Reference).RecordCount;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetSize: integer;
begin
  Result := -1;
  try
    Result := TDataset(Reference).RecordSize;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetState: TDataSetState;
begin
  Result := Low(TDataSetState);
  try
    Result := TDataset(Reference).State;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetFound: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).Found;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetCanModify: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).CanModify;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetActive: boolean;
begin
  Result := False;
  try
    Result := TDataset(Reference).Active;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetBookmark: string;
begin
  Result := '';
  try
    Result := TDataset(Reference).Bookmark;
  except
    SetErrorException;
  end;
end;
function TExpressionDatasetRecord.GetDatabaseName: string;
begin
  Result := '';
  try
    if Reference is TDBDataset then
      Result := TDBDataset(Reference).DatabaseName
    else
      Result := '';
  except
    SetErrorException;
  end;
end;

//==============================================================================
// TExpressionDatasetDirect = class(TExpressionObject)
//==============================================================================
constructor TExpressionDatasetDirect.Create(const aSymbol: string; aReference: TDataset);
begin
  // pass the buck
  inherited Create(aSymbol, aReference);
end;
function TExpressionDatasetDirect.CheckName(const aProperty: string): string;
var
  vIndex: integer;
begin
  // we had better have an active data set by now!
  Assert(TDataset(Reference).Active, ResultTextFor(STG_E_INVALIDHANDLE));

  // do we still need to figure out the default property?
  if FCheckDefault then
    for vIndex := 0 to TDataset(Reference).FieldCount-1 do
      if IsValidIdent(TDataset(Reference).Fields[vIndex].FieldName) then begin
        FDefaultProperty := TDataset(Reference).Fields[vIndex].FieldName;
        break;
      end;

  // ok let actually do some work
  Result := inherited CheckName(aProperty);
  if not IsValidIdent(Result) then
    Result := '';
end;
function TExpressionDatasetDirect.Valid(const aProperty: string): boolean;
begin
  Result := TDataset(Reference).FieldByName(CheckName(aProperty)) <> nil;
end;
function TExpressionDatasetDirect.ValueOf(const aProperty: string): variant;
var
  vField: TField;
begin
  Result := Unassigned;
  vField := TDataset(Reference).FieldByName(CheckName(aProperty));
  if Assigned(vField) then
    Result := vField.AsVariant;
end;

end.
