unit ExpressionOpersComplex;

// Low    = =, <>, >=, <=, >, <
// Medium = +, -, OR
// High   = *, /, AND
// Unary  = NOT, -

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser;

//==============================================================================
type
  //============================================================================
  // low priority operations
  TExpressionLogicalEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalNotEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalGreaterEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalLessEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalGreater = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalLess = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // medium priority operations
  TExpressionValueAdd = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueSubtract = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalOr = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // high priority operations
  TExpressionValueMultiply = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueDivide = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalAnd = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // unary operations
  TExpressionLogicalNot = class(TExpressionOperationWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueNegation = class(TExpressionOperationWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscSupport, ComplexSupport;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.ExpressionLexicon do begin
    Add(TExpressionLogicalEqual);
    Add(TExpressionLogicalNotEqual);
    Add(TExpressionLogicalGreaterEqual);
    Add(TExpressionLogicalLessEqual);
    Add(TExpressionLogicalGreater);
    Add(TExpressionLogicalLess);
  end;
  with aParser.FormulaLexicon do begin
    Add(TExpressionValueAdd);
    Add(TExpressionValueSubtract);
    Add(TExpressionLogicalOr);
  end;
  with aParser.TermLexicon do begin
    Add(TExpressionValueMultiply);
    Add(TExpressionValueDivide);
    Add(TExpressionLogicalAnd);
  end;
  with aParser.UnaryLexicon do begin
    Add(TExpressionLogicalNot);
    Add(TExpressionValueNegation);
  end;
end;

//==============================================================================
// TExpressionLogicalEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalEqual.Symbol: string;
begin
  Result := '=';
end;
procedure TExpressionLogicalEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexEqual(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalNotEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalNotEqual.Symbol: string;
begin
  Result := '<>';
end;
procedure TExpressionLogicalNotEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexNotEqual(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalGreaterEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalGreaterEqual.Symbol: string;
begin
  Result := '>=';
end;
procedure TExpressionLogicalGreaterEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexGreaterOrEqual(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalLessEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalLessEqual.Symbol: string;
begin
  Result := '<=';
end;
procedure TExpressionLogicalLessEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexLessOrEqual(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalGreater = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalGreater.Symbol: string;
begin
  Result := '>';
end;
procedure TExpressionLogicalGreater.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexGreater(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalLess = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalLess.Symbol: string;
begin
  Result := '<';
end;
procedure TExpressionLogicalLess.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexLess(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionValueAdd = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueAdd.Symbol: string;
begin
  Result := '+';
end;
procedure TExpressionValueAdd.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexAdd(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionValueSubtract = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueSubtract.Symbol: string;
begin
  Result := '-';
end;
procedure TExpressionValueSubtract.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexSubtract(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalOr = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalOr.Symbol: string;
begin
  Result := 'Or';
end;
procedure TExpressionLogicalOr.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexAsType(Pop, varBoolean) or VarComplexAsType(vValue, varBoolean));
  end;
end;

//==============================================================================
// TExpressionValueMultiply = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueMultiply.Symbol: string;
begin
  Result := '*';
end;
procedure TExpressionValueMultiply.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexMultiply(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionValueDivide = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueDivide.Symbol: string;
begin
  Result := '/';
end;
procedure TExpressionValueDivide.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexDivide(Pop, vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalAnd = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalAnd.Symbol: string;
begin
  Result := 'And';
end;
procedure TExpressionLogicalAnd.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(VarComplexAsType(Pop, varBoolean) and VarComplexAsType(vValue, varBoolean));
  end;
end;

//==============================================================================
// TExpressionLogicalNot = class(TExpressionOperationWith1Arg)
//==============================================================================
class function TExpressionLogicalNot.Symbol: string;
begin
  Result := 'Not';
end;
procedure TExpressionLogicalNot.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(not VarComplexAsType(Pop, varBoolean));
end;

//==============================================================================
// TExpressionValueNegation = class(TExpressionOperationWith1Arg)
//==============================================================================
class function TExpressionValueNegation.Symbol: string;
begin
  Result := '-';
end;
procedure TExpressionValueNegation.Execute(Sender: TObject; AStack: TExpressionStack);
begin
  with AStack do
    Push(VarComplexNegation(Pop));
end;

end.
