unit ExpressionOpersStandard;

// Low    = =, <>, >=, <=, >, <
// Medium = +, -, &, OR, XOR
// High   = *, /, ^, DIV, MOD, AND
// Unary  = NOT, -

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser,Variants;

//==============================================================================
type
  //============================================================================
  // low priority operations
  TExpressionLogicalEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalNotEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalGreaterEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalLessEqual = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalGreater = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalLess = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // medium priority operations
  TExpressionValueAdd = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueSubtract = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionStringAppend = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalOr = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalXor = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // high priority operations
  TExpressionValueMultiply = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueDivide = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValuePower = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionIntegerDivide = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionModulusDivide = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionLogicalAnd = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // unary operations
  TExpressionLogicalNot = class(TExpressionOperationWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionValueNegation = class(TExpressionOperationWith1Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscSupport, Math;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.ExpressionLexicon do begin
    Add(TExpressionLogicalEqual);
    Add(TExpressionLogicalNotEqual);
    Add(TExpressionLogicalGreaterEqual);
    Add(TExpressionLogicalLessEqual);
    Add(TExpressionLogicalGreater);
    Add(TExpressionLogicalLess);
  end;
  with aParser.FormulaLexicon do begin
    Add(TExpressionValueAdd);
    Add(TExpressionValueSubtract);
    Add(TExpressionStringAppend);
    Add(TExpressionLogicalOr);
    Add(TExpressionLogicalXor);
  end;
  with aParser.TermLexicon do begin
    Add(TExpressionValueMultiply);
    Add(TExpressionValueDivide);
    Add(TExpressionValuePower);
    Add(TExpressionIntegerDivide);
    Add(TExpressionModulusDivide);
    Add(TExpressionLogicalAnd);
  end;
  with aParser.UnaryLexicon do begin
    Add(TExpressionLogicalNot);
    Add(TExpressionValueNegation);
  end;
end;

//==============================================================================
// TExpressionLogicalEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalEqual.Symbol: string;
begin
  Result := '=';
end;
procedure TExpressionLogicalEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop = vValue);
  end;
end;

//==============================================================================
// TExpressionLogicalNotEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalNotEqual.Symbol: string;
begin
  Result := '<>';
end;
procedure TExpressionLogicalNotEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop <> vValue);
  end;
end;

//==============================================================================
// TExpressionLogicalGreaterEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalGreaterEqual.Symbol: string;
begin
  Result := '>=';
end;
procedure TExpressionLogicalGreaterEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop >= vValue);
  end;
end;

//==============================================================================
// TExpressionLogicalLessEqual = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalLessEqual.Symbol: string;
begin
  Result := '<=';
end;
procedure TExpressionLogicalLessEqual.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop <= vValue);
  end;
end;

//==============================================================================
// TExpressionLogicalGreater = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalGreater.Symbol: string;
begin
  Result := '>';
end;
procedure TExpressionLogicalGreater.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop > vValue);
  end;
end;

//==============================================================================
// TExpressionLogicalLess = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalLess.Symbol: string;
begin
  Result := '<';
end;
procedure TExpressionLogicalLess.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    Push(Pop < vValue);
  end;
end;

//==============================================================================
// TExpressionValueAdd = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueAdd.Symbol: string;
begin
  Result := '+';
end;
procedure TExpressionValueAdd.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vLeft, vRight: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;

    // check for the implied string concatination
    if (VarType(vLeft) = varString) or (VarType(vRight) = varString) then
      Push(Concat(VarAsType(vLeft, varString), VarAsType(vRight, varString)))

    // mathmatic addition
    else begin
      {$ifdef StrongChecking}
      VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                              varDouble, varCurrency, varDate]);
      VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                              varDouble, varCurrency, varDate]);
      {$endif}
      Push(vLeft + vRight);
    end;
  end;
end;

//==============================================================================
// TExpressionValueSubtract = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueSubtract.Symbol: string;
begin
  Result := '-';
end;
procedure TExpressionValueSubtract.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency, varDate]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency, varDate]);
    {$endif}
    Push(vLeft - vRight);
  end;
end;

//==============================================================================
// TExpressionStringAppend = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionStringAppend.Symbol: string;
begin
  Result := '&';
end;
procedure TExpressionStringAppend.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: string;
begin
  with AStack do begin
    vValue := Pop;
    Push(Concat(VarAsType(Pop, varString), vValue));
  end;
end;

//==============================================================================
// TExpressionLogicalOr = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalOr.Symbol: string;
begin
  Result := 'Or';
end;
procedure TExpressionLogicalOr.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varBoolean, varSmallInt, varInteger]);
    VarAssureTypes(vLeft, [varBoolean, varSmallInt, varInteger]);
    {$endif}
    Push(vLeft or vRight);
  end;
end;

//==============================================================================
// TExpressionLogicalXor = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalXor.Symbol: string;
begin
  Result := 'Xor';
end;
procedure TExpressionLogicalXor.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varBoolean, varSmallInt, varInteger]);
    VarAssureTypes(vLeft, [varBoolean, varSmallInt, varInteger]);
    {$endif}
    Push(vLeft xor vRight);
  end;
end;

//==============================================================================
// TExpressionValueMultiply = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueMultiply.Symbol: string;
begin
  Result := '*';
end;
procedure TExpressionValueMultiply.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(vLeft * vRight);
  end;
end;

//==============================================================================
// TExpressionValueDivide = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValueDivide.Symbol: string;
begin
  Result := '/';
end;
procedure TExpressionValueDivide.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(vLeft / vRight);
  end;
end;

//==============================================================================
// TExpressionValuePower = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionValuePower.Symbol: string;
begin
  Result := '^';
end;
procedure TExpressionValuePower.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(Power(vLeft, vRight));
  end;
end;

//==============================================================================
// TExpressionIntegerDivide = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionIntegerDivide.Symbol: string;
begin
  Result := 'Div';
end;
procedure TExpressionIntegerDivide.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(vLeft div vRight);
  end;
end;

//==============================================================================
// TExpressionModulusDivide = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionModulusDivide.Symbol: string;
begin
  Result := 'Mod';
end;
procedure TExpressionModulusDivide.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    VarAssureTypes(vLeft, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(vLeft mod vRight);
  end;
end;

//==============================================================================
// TExpressionLogicalAnd = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalAnd.Symbol: string;
begin
  Result := 'And';
end;
procedure TExpressionLogicalAnd.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vRight, vLeft: variant;
begin
  with AStack do begin
    vRight := Pop;
    vLeft := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vRight, [varBoolean, varSmallInt, varInteger]);
    VarAssureTypes(vLeft, [varBoolean, varSmallInt, varInteger]);
    {$endif}
    Push(vLeft and vRight);
  end;
end;

//==============================================================================
// TExpressionLogicalNot = class(TExpressionOperationWith1Arg)
//==============================================================================
class function TExpressionLogicalNot.Symbol: string;
begin
  Result := 'Not';
end;
procedure TExpressionLogicalNot.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vValue, [varBoolean, varSmallInt, varInteger]);
    {$endif}
    Push(not vValue);
  end;
end;

//==============================================================================
// TExpressionValueNegation = class(TExpressionOperationWith1Arg)
//==============================================================================
class function TExpressionValueNegation.Symbol: string;
begin
  Result := '-';
end;
procedure TExpressionValueNegation.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vValue: variant;
begin
  with AStack do begin
    vValue := Pop;
    {$ifdef StrongChecking}
    VarAssureTypes(vValue, [varSmallInt, varInteger, varSingle,
                            varDouble, varCurrency]);
    {$endif}
    Push(-vValue);
  end;
end;

end.
