unit ExpressionOpersStrings;

// Low    = ==, RESEMBLES, MATCHES, CONTAINS, IN, STARTS, ENDS
// Medium = &&

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ExpressionStack, ExpressionLexicon,
  ExpressionParser,Variants;

//==============================================================================
type
  //============================================================================
  // low priority operations
  TExpressionLogicalEqualNames = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareResembles = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareMatches = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareContains = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareIn = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareStarts = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;
  TExpressionCompareEnds = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

  //============================================================================
  // medium priority operations
  TExpressionStringAppendWSpace = class(TExpressionOperationWith2Arg)
  public
    class function Symbol: string; override;
    procedure Execute(Sender: TObject; AStack: TExpressionStack); override;
  end;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MatchingSupport;

//==============================================================================
procedure RegisterLexicon(aParser: TCustomExpressionParser);
begin
  with aParser.ExpressionLexicon do begin
    Add(TExpressionLogicalEqualNames);
    Add(TExpressionCompareResembles);
    Add(TExpressionCompareMatches);
    Add(TExpressionCompareContains);
    Add(TExpressionCompareIn);
    Add(TExpressionCompareStarts);
    Add(TExpressionCompareEnds);
  end;
  with aParser.FormulaLexicon do begin
    Add(TExpressionStringAppendWSpace);
  end;
end;

//==============================================================================
// TExpressionLogicalEqualNames = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionLogicalEqualNames.Symbol: string;
begin
  Result := '==';
end;
procedure TExpressionLogicalEqualNames.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPattern: string;
begin
  with AStack do begin
    vPattern := Pop;
    Push(AnsiCompareFilename(Pop, vPattern) = 0);
  end;
end;

//==============================================================================
// TExpressionCompareResembles = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareResembles.Symbol: string;
begin
  Result := 'Resembles';
end;
procedure TExpressionCompareResembles.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPattern: string;
begin
  with AStack do begin
    vPattern := Pop;
    Push(Resembles(Pop, vPattern));
  end;
end;

//==============================================================================
// TExpressionCompareMatches = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareMatches.Symbol: string;
begin
  Result := 'Matches';
end;
procedure TExpressionCompareMatches.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vPattern: string;
begin
  with AStack do begin
    vPattern := Pop;
    Push(Matches(Pop, vPattern));
  end;
end;

//==============================================================================
// TExpressionCompareContains = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareContains.Symbol: string;
begin
  Result := 'Contains';
end;
procedure TExpressionCompareContains.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText: string;
begin
  with AStack do begin
    vText := Pop;
    Push(AnsiPos(AnsiUppercase(vText), AnsiUppercase(Pop)) > 0);
  end;
end;

//==============================================================================
// TExpressionCompareIn = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareIn.Symbol: string;
begin
  Result := 'In';
end;
procedure TExpressionCompareIn.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText: string;
begin
  with AStack do begin
    vText := Pop;
    Push(AnsiPos(AnsiUppercase(Pop), AnsiUppercase(vText)) > 0);
  end;
end;

//==============================================================================
// TExpressionCompareStarts = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareStarts.Symbol: string;
begin
  Result := 'Starts';
end;
procedure TExpressionCompareStarts.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText, vSubText: string;
begin
  with AStack do begin
    vSubText := Pop;
    vText := Pop;
    Push(AnsiCompareText(vText, Copy(vSubText, 1, Length(vText))) = 0);
  end;
end;

//==============================================================================
// TExpressionCompareEnds = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionCompareEnds.Symbol: string;
begin
  Result := 'Ends';
end;
procedure TExpressionCompareEnds.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText, vSubText: string;
begin
  with AStack do begin
    vSubText := Pop;
    vText := Pop;
    Push(AnsiCompareText(vText, Copy(vSubText, Length(vSubText)-Length(vText)+1, Length(vText))) = 0);
  end;
end;

//==============================================================================
// TExpressionStringAppendWSpace = class(TExpressionOperationWith2Arg)
//==============================================================================
class function TExpressionStringAppendWSpace.Symbol: string;
begin
  Result := '&&';
end;
procedure TExpressionStringAppendWSpace.Execute(Sender: TObject; AStack: TExpressionStack);
var
  vText: string;
begin
  with AStack do begin
    vText := Pop;
    Push(Concat(VarAsType(Pop, varString), ' ', vText));
  end;
end;

end.
