unit FileSupport;

// still to do
// figure out GetPersonalPath

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes;

//==============================================================================
type
  TFileAttribute = (ftReadOnly, ftHidden, ftSystem, ftVolumeID,
                    ftDirectory, ftArchive, ftCompressed, ftNormal,
                    ftTemporary, ftAtomicWrite, ftXActionWrite, ft11,
                    ft12, ft13, ft14, ft15,
                    ft16, ft17, ft18, ft19,
                    ft20, ft21, ft22, ft23,
                    ft24, ft25, ft26, ft27,
                    ft28, ft29, ft30, ft31);
  TFileAttributes = set of TFileAttribute;
  TFileLongName = array [0..MAX_PATH-1] of Char;
  TFileShortName = array [0..13] of Char;

//==============================================================================
function PathAndFile(const aPathname, aFilename: string): string;
function FindPathForFile(const aFilename: string): string;
function ShortpathForPath(const aPathname: string): string;
function ParentOfPath(const aPathname: string): string;
function StringToPath(const aValue: string): string;

//==============================================================================
function TheTemporaryPath: string;
function TheSystemPath: string;
function ThePersonalPath: string;

//==============================================================================
function FileTimeToLocalDateTime(const aTime: TFileTime): TDateTime; overload;
function FileTimeToLocalDateTime(const aTime, aDefault: TFileTime): TDateTime; overload;
function FileTimeToDateTime(const aTime: TFileTime): TDateTime; overload;
function FileTimeToDateTime(const aTime, aDefault: TFileTime): TDateTime; overload;
function LocalDateTimeToFileTime(const aTime: TDateTime): TFileTime; overload;
function LocalDateTimeToFileTime(const aTime, aDefault: TDateTime): TFileTime; overload;
function DateTimeToFileTime(const aTime: TDateTime): TFileTime; overload;
function DateTimeToFileTime(const aTime, aDefault: TDateTime): TFileTime; overload;

//==============================================================================
function IsNullFileTime(const aTime: TFileTime): boolean;
function NullFileTime: TFileTime;

//==============================================================================
implementation

//==============================================================================
uses
  DateTimeSupport;

//==============================================================================
function PathAndFile(const aPathname, aFilename: string): string;
begin
  Result := ExtractFilePath(aPathname) + ExtractFileName(aFilename);
end;
function FindPathForFile(const aFilename: string): string;
var
  vPath, vFile: PChar;
  vSize: integer;
begin
  // assume we don't do nothing ;-)
  Result := aFilename;

  // does the root fail the test
  if ExtractFilePath(aFilename) = '' then begin

    // how big do we need?
    vSize := MAX_PATH;
    vPath := StrAlloc(vSize);
    vSize := SearchPath(nil, PChar(aFilename), nil, vSize, vPath, vFile);
    if vSize > MAX_PATH then begin
      StrDispose(vPath);
      vPath := StrAlloc(vSize);
      SearchPath(nil, PChar(aFilename), nil, vSize, vPath, vFile);
    end;

    // return it!
    Result := vPath;

    // clean up
    StrDispose(vPath);
  end;
end;
function ShortpathForPath(const aPathname: string): string;
var
  vPath: PChar;
  vSize: integer;
begin
  // assume we don't do nothing ;-)
  Result := aPathname;

  // does the root fail the test
  if ExtractFilePath(aPathname) <> '' then begin

    // how big do we need?
    vSize := MAX_PATH;
    vPath := StrAlloc(vSize + 1);
    vSize := GetShortPathname(PChar(aPathname), vPath, vSize);
    if vSize > MAX_PATH then begin
      StrDispose(vPath);
      vPath := StrAlloc(vSize + 1);
      GetShortPathname(PChar(aPathname), vPath, vSize);
    end;

    // return it!
    Result := vPath;

    // clean up
    StrDispose(vPath);
  end;
end;
function ParentOfPath(const aPathname: string): string;
var
  vLast: string;
begin
  Result := aPathname;
  if Copy(Result, Length(Result)-1, 2) <> ':\' then begin
    vLast := Copy(Result, Length(Result), 1);
    if vLast = ':' then
      Result := Result + '\'
    else begin
      if vLast = '\' then
        Result := Copy(Result, 1, Length(Result) - 1);
      Result := ExtractFilePath(Result);
    end;
  end;
end;
function StringToPath(const aValue: string): string;
var
  vLast: string;
begin
  Result := aValue;
  vLast := Copy(Result, Length(Result), 1);
  if (vLast <> ':') and
     (vLast <> '\') then
    Result := Result + '\';
end;

//==============================================================================
function TheTemporaryPath: string;
var
  vSize: integer;
  vPath: PChar;
begin
  // try to get the info
  vSize := MAX_PATH;
  vPath := StrAlloc(vSize + 1);
  vSize := GetTempPath(vSize, vPath);
  if vSize > MAX_PATH then begin
    StrDispose(vPath);
    vPath := StrAlloc(vSize + 1);
    GetTempPath(vSize, vPath);
  end;

  // return it!
  Result := vPath;
end;
function TheSystemPath: string;
var
  vSize: integer;
  vPath: PChar;
begin
  // try to get the info
  vSize := MAX_PATH;
  vPath := StrAlloc(vSize + 1);
  vSize := GetSystemDirectory(vPath, vSize);
  if vSize > MAX_PATH then begin
    StrDispose(vPath);
    vPath := StrAlloc(vSize + 1);
    GetSystemDirectory(vPath, vSize);
  end;

  // return it!
  Result := vPath;
end;
function ThePersonalPath: string;
begin
  // we need to find out where the personal path is?
  Result := TheSystemPath;
end;

//==============================================================================
function FileTimeToLocalDateTime(const aTime: TFileTime): TDateTime;
var
  vFileTime: TFileTime;
begin
  if FileTimeToLocalFileTime(aTime, vFileTime) then
    Result := FileTimeToDateTime(vFileTime)
  else
    Result := 0;
end;
function FileTimeToLocalDateTime(const aTime, aDefault: TFileTime): TDateTime;
begin
  if IsNullFileTime(aTime) then
    Result := FileTimeToLocalDateTime(aDefault)
  else
    Result := FileTimeToLocalDateTime(aTime);
end;
function FileTimeToDateTime(const aTime: TFileTime): TDateTime;
var
  vSystemTime: TSystemTime;
begin
  if FileTimeToSystemTime(aTime, vSystemTime) then
    Result := SystemTimeToDateTime(vSystemTime)
  else
    Result := 0;
end;
function FileTimeToDateTime(const aTime, aDefault: TFileTime): TDateTime;
begin
  if IsNullFileTime(aTime) then
    Result := FileTimeToDateTime(aDefault)
  else
    Result := FileTimeToDateTime(aTime);
end;
function LocalDateTimeToFileTime(const aTime: TDateTime): TFileTime;
var
  vFileTime: TFileTime;
begin
  Result := NullFileTime;
  vFileTime := DateTimeToFileTime(aTime);
  if (not IsNullFileTime(vFileTime)) and
     (not LocalFileTimeToFileTime(vFileTime, Result)) then
    Result := NullFileTime;
end;
function LocalDateTimeToFileTime(const aTime, aDefault: TDateTime): TFileTime;
begin
  if IsNullDateTime(aTime) then
    Result := LocalDateTimeToFileTime(aDefault)
  else
    Result := LocalDateTimeToFileTime(aTime);
end;
function DateTimeToFileTime(const aTime: TDateTime): TFileTime;
var
  vSystemTime: TSystemTime;
begin
  DateTimeToSystemTime(aTime, vSystemTime);
  if not SystemTimeToFileTime(vSystemTime, Result) then
    Result := NullFileTime;
end;
function DateTimeToFileTime(const aTime, aDefault: TDateTime): TFileTime;
begin
  if IsNullDateTime(aTime) then
    Result := DateTimeToFileTime(aDefault)
  else
    Result := DateTimeToFileTime(aTime);
end;

//==============================================================================
function IsNullFileTime(const aTime: TFileTime): boolean;
begin
  with aTime do
    Result := (dwLowDateTime = 0) and
              (dwHighDateTime = 0);
end;
function NullFileTime: TFileTime;
begin
  with Result do begin
    dwLowDateTime := 0;
    dwHighDateTime := 0;
  end;
end;

end.
