unit ListSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes;

//==============================================================================
type
  //============================================================================
  TIntegerList = class(TObject)
  private
    FList: TList;
  protected
    function GetInteger(aIndex: integer): integer;
    procedure SetInteger(aIndex, aValue: integer);
  public
    constructor Create; virtual;
    destructor Destroy; override;

    function Count: integer;
    function Add(aValue: integer): integer; virtual;
    procedure Delete(aIndex: integer); virtual;
    procedure Clear; virtual;
    function Inc(aIndex: integer): integer; virtual;
    function Dec(aIndex: integer): integer; virtual;
    property Integers[aIndex: integer]: integer read GetInteger write SetInteger; default;
  end;

  //============================================================================
  TFreeList = class(TList)
  private
    FOwnChildren: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; {$ifdef ver120}override;{$endif}
    procedure Delete(aIndex: Integer); virtual;
    function Remove(Item: Pointer): Integer; virtual;
    property OwnChildren: Boolean read FOwnChildren write FOwnChildren;
  end;

  //============================================================================
  TThreadFreeList = class(TThreadList)
  private
    FOwnChildren: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; virtual;
    procedure Delete(aIndex: Integer); virtual;
    procedure Remove(Item: Pointer); virtual;
    function Count: integer;
    property OwnChildren: Boolean read FOwnChildren write FOwnChildren;
  end;

  //============================================================================
  TFreeStringList = class(TStringList)
  private
    FOwnChildren: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    procedure Delete(aIndex: Integer); override;
    property OwnChildren: Boolean read FOwnChildren write FOwnChildren;
  end;

  //============================================================================
  TSortedStringList = class(TStringList)
  public
    constructor Create;
    function Defined(const aText: string): boolean; virtual;
    function Add(const aText: string): integer; override;
  end;

  //============================================================================
  TSortedFreeStringList = class(TFreeStringList)
  public
    constructor Create;
    function Defined(const aText: string): boolean; virtual;
    function Add(const aText: string): integer; override;
  end;

  //============================================================================
  TDuelSortedStringList = class(TSortedStringList)
  protected
    function GetValue(aText: string): string;
    procedure SetValue(aText, aValue: string);
    function GetValueByIndex(index: integer): string;
    procedure SetValueByIndex(index: integer; aValue: string);
  public
    destructor Destroy; override;
    procedure Clear; override;
    procedure Delete(aIndex: Integer); override;
    function AddDuel(const aText, aValue: string): integer; virtual;
    property Values[aText: string]: string read GetValue write SetValue;
    property ValuesByIndex[index: integer]: string read GetValueByIndex write SetValueByIndex;
  end;

  //============================================================================
  TTaggedValueItem = class(TObject)
  private
    FTag: integer;
    FValue: variant;
  public
    constructor Create(aTag: integer; const aValue: variant);
    property Tag: integer read FTag write FTag;
    property Value: variant read FValue write FValue;
  end;
  TTaggedValueList = class(TObject)
  private
    FList: TFreeList;
  protected
    property List: TFreeList read FList;
    function GetTag(aIndex: integer): integer; virtual;
    function GetValue(aTag: integer): variant; virtual;
    procedure SetValue(aTag: integer; const aValue: variant); virtual;
    function GetValueByIndex(aIndex: integer): variant; virtual;
    procedure SetValueByIndex(aIndex: integer; const aValue: variant); virtual;
    function Insert(aIndex: integer; aTag: integer; const aValue: variant): integer; virtual;
    function FindTag(aTag: integer; var aIndex: integer): boolean; virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; virtual;
    function Count: integer; virtual;
    property Tags[aIndex: integer]: integer read GetTag;
    property Values[aTag: integer]: variant read GetValue write SetValue;
    property ValuesByIndex[aIndex: integer]: variant read GetValueByIndex write SetValueByIndex;
    function Add(aTag: integer; const aValue: variant): integer; virtual;
    function Let(aTag: integer; const aValue: variant): integer; virtual;
    function IndexOf(aTag: integer): integer; virtual;
    function IndexOfValue(const aValue: variant): integer; virtual;
    function TagExists(aTag: integer): boolean; virtual;
    function Remove(aTag: integer): integer; virtual;
    procedure Delete(aIndex: integer); virtual;
  end;

  //============================================================================
  TObjectList = class(TObject)
  private
    FList: TFreeList;
  protected
    property List: TFreeList read FList;
    function GetItem(aIndex: integer): TObject; virtual;
    function Insert(aIndex: integer; aObject: TObject): integer; virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; virtual;
    function Count: integer; virtual;
    property Objects[aIndex: integer]: TObject read GetItem;
    function Add(aObject: TObject): integer;
    function IndexOf(aObject: TObject): integer; virtual;
    function Remove(aObject: TObject): integer;
    procedure Delete(aIndex: integer); virtual;
  end;

  //============================================================================
  TClassList = class(TObject)
  private
    FList: TList;
  protected
    property List: TList read FList;
    function GetClass(aIndex: integer): TClass; virtual;
    function Insert(aIndex: integer; aClass: TClass): integer; virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Clear; virtual;
    function Count: integer; virtual;
    property Classes[aIndex: integer]: TClass read GetClass;
    function Add(aClass: TClass): integer;
    function IndexOf(aClass: TClass): integer; virtual;
    function Remove(aClass: TClass): integer; virtual;
    procedure Delete(aIndex: integer); virtual;
  end;

//==============================================================================
procedure TrimStrings(aStrings: TStrings);

//==============================================================================
implementation

//==============================================================================
uses
  MiscConstants, MiscSupport, ResultSupport;

//==============================================================================
procedure TrimStrings(aStrings: TStrings);
var
  vIndex: integer;
begin
  with aStrings do
    for vIndex := Count-1 downto 0 do
      if Trim(Strings[vIndex]) = '' then
        Delete(vIndex);
end;

//==============================================================================
// TIntegerList = class(TObject)
//==============================================================================
constructor TIntegerList.Create;
begin
  inherited Create;
  FList := TList.Create;
end;
destructor TIntegerList.Destroy;
begin
  Dump(FList);
  inherited Destroy;
end;
function TIntegerList.GetInteger(aIndex: integer): integer;
begin
  Result := Integer(FList[aIndex]);
end;
procedure TIntegerList.SetInteger(aIndex, aValue: integer);
begin
  FList[aIndex] := TObject(aValue);
end;
function TIntegerList.Count: integer;
begin
  Result := FList.Count;
end;
function TIntegerList.Add(aValue: integer): integer;
begin
  Result := FList.Add(TObject(aValue));
end;
procedure TIntegerList.Delete(aIndex: integer);
begin
  FList.Delete(aIndex);
end;
procedure TIntegerList.Clear;
begin
  FList.Clear;
end;
function TIntegerList.Inc(aIndex: integer): integer;
begin
  Integers[aIndex] := Integers[aIndex] + 1;
  Result := Integers[aIndex];
end;
function TIntegerList.Dec(aIndex: integer): integer;
begin
  Integers[aIndex] := Integers[aIndex] - 1;
  Result := Integers[aIndex];
end;

//==============================================================================
// TFreeList = class(TList)
//==============================================================================
constructor TFreeList.Create;
begin
  inherited Create;
  FOwnChildren := True;
end;
destructor TFreeList.Destroy;
begin
  Clear;
  inherited Destroy;
end;
procedure TFreeList.Clear;
var
  vIndex: integer;
begin
  if OwnChildren then
    for vIndex := 0 to Count-1 do
      TObject(Items[vIndex]).Free;
  inherited Clear;
end;
procedure TFreeList.Delete(aIndex: Integer);
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  if OwnChildren then
    TObject(Items[aIndex]).Free;
  inherited Delete(aIndex);
end;
function TFreeList.Remove(Item: Pointer): Integer;
begin
  Result := IndexOf(Item);
  Assert(Result <> cIndexNotFound, ResultTextFor(TYPE_E_ELEMENTNOTFOUND));
  if Result <> cIndexNotFound then
    Delete(Result);
end;

//==============================================================================
// TThreadFreeList = class(TThreadList)
//==============================================================================
constructor TThreadFreeList.Create;
begin
  inherited Create;
  FOwnChildren := True;
end;
destructor TThreadFreeList.Destroy;
begin
  Clear;
  inherited Destroy;
end;
procedure TThreadFreeList.Clear;
var
  vIndex: integer;
  vList: TList;
begin
  vList := LockList;
  try
    if OwnChildren then
      for vIndex := 0 to vList.Count-1 do
        TObject(vList[vIndex]).Free;
    vList.Clear;
  finally
    UnlockList;
  end;
end;
procedure TThreadFreeList.Delete(aIndex: Integer);
var
  vList: TList;
begin
  vList := LockList;
  try
    Assert((aIndex >= 0) and (aIndex < vList.Count), ResultTextFor(DISP_E_BADINDEX));
    if OwnChildren then
      TObject(vList[aIndex]).Free;
    vList.Delete(aIndex);
  finally
    UnlockList;
  end;
end;
procedure TThreadFreeList.Remove(Item: Pointer);
var
  vList: TList;
  vIndex: integer;
begin
  vList := LockList;
  try
    vIndex := vList.IndexOf(Item);
    Assert((vIndex >= 0) and (vIndex < vList.Count), ResultTextFor(DISP_E_BADINDEX));
    if OwnChildren then
      TObject(vList[vIndex]).Free;
    vList.Delete(vIndex);
  finally
    UnlockList;
  end;
end;
function TThreadFreeList.Count: integer;
var
  vList: TList;
begin
  vList := LockList;
  try
    Result := vList.Count;
  finally
    UnlockList;
  end;
end;


//==============================================================================
// TFreeStringList = class(TStringList)
//==============================================================================
constructor TFreeStringList.Create;
begin
  inherited Create;
  FOwnChildren := True;
end;
destructor TFreeStringList.Destroy;
begin
  Clear;
  inherited Destroy;
end;
procedure TFreeStringList.Clear;
var
  vIndex: integer;
begin
  if OwnChildren then
    for vIndex := 0 to Count-1 do
      TObject(Objects[vIndex]).Free;
  inherited Clear;
end;
procedure TFreeStringList.Delete(aIndex: Integer);
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  if OwnChildren then
    TObject(Objects[aIndex]).Free;
  inherited Delete(aIndex);
end;

//==============================================================================
// TSortedStringList = class(TStringList)
//==============================================================================
constructor TSortedStringList.Create;
begin
  inherited Create;
  Sorted := True;
  Duplicates := dupIgnore;
end;
function TSortedStringList.Defined(const aText: string): boolean;
begin
  Result := IndexOf(aText) <> cIndexNotFound;
end;
function TSortedStringList.Add(const aText: string): integer;
begin
  Result := cIndexNotFound;
  if not Defined(aText) then
    Result := inherited Add(aText);
end;

//==============================================================================
// TSortedFreeStringList = class(TFreeStringList)
//==============================================================================
constructor TSortedFreeStringList.Create;
begin
  inherited Create;
  Sorted := True;
  Duplicates := dupIgnore;
end;
function TSortedFreeStringList.Defined(const aText: string): boolean;
begin
  Result := IndexOf(aText) <> cIndexNotFound;
end;
function TSortedFreeStringList.Add(const aText: string): integer;
begin
  Result := cIndexNotFound;
  if not Defined(aText) then
    Result := inherited Add(aText);
end;

//==============================================================================
// TDuelSortedStringList = class(TSortedStringList)
//==============================================================================
destructor TDuelSortedStringList.Destroy;
begin
  Clear;
  inherited Destroy;
end;
procedure TDuelSortedStringList.Clear;
var
  vIndex: integer;
begin
  for vIndex := Count-1 downto 0 do
    Delete(vIndex);
  inherited Clear;
end;
procedure TDuelSortedStringList.Delete(aIndex: Integer);
begin
  ValuesByIndex[aIndex] := '';
  inherited Delete(aIndex);
end;
function TDuelSortedStringList.AddDuel(const aText, aValue: string): integer;
begin
  Result := Add(aText);
  if Result <> cIndexNotFound then
    ValuesByIndex[Result] := aValue;
end;
function TDuelSortedStringList.GetValue(aText: string): string;
begin
  Result := ValuesByIndex[IndexOf(aText)];
end;
procedure TDuelSortedStringList.SetValue(aText, aValue: string);
begin
  ValuesByIndex[IndexOf(aText)] := aValue;
end;
function TDuelSortedStringList.GetValueByIndex(index: integer): string;
begin
  Result := '';
  if (index <> cIndexNotFound) and
     (Objects[index] <> nil) then
    Result := PChar(Objects[index]);
end;
procedure TDuelSortedStringList.SetValueByIndex(index: integer; aValue: string);
begin
  if (index <> cIndexNotFound) and
     (Objects[index] <> nil) then begin
    StrDispose(PChar(Objects[index]));
    Objects[index] := nil;
  end;
  if (index <> cIndexNotFound) and
     (aValue <> '') then begin
    Objects[index] := Pointer(StrAlloc(Length(aValue) + 1));
    StrCopy(PChar(Objects[index]), PChar(aValue));
  end;
end;

//==============================================================================
// TTaggedValueItem = class(TObject)
//==============================================================================
constructor TTaggedValueItem.Create(aTag: integer; const aValue: variant);
begin
  inherited Create;
  Tag := aTag;
  Value := aValue;
end;

//==============================================================================
// TTaggedValueList = class(TObject)
//==============================================================================
constructor TTaggedValueList.Create;
begin
  inherited Create;
  FList := TFreeList.Create;
end;
destructor TTaggedValueList.Destroy;
begin
  Dump(FList);
  inherited Destroy;
end;
function TTaggedValueList.GetTag(aIndex: integer): integer;
begin
  Result := TTaggedValueItem(List[aIndex]).Tag;
end;
function TTaggedValueList.GetValue(aTag: integer): variant;
var
  vIndex: integer;
begin
  Result := '';
  vIndex := IndexOf(aTag);
  if vIndex <> cIndexNotFound then
    Result := ValuesByIndex[vIndex];
end;
procedure TTaggedValueList.SetValue(aTag: integer; const aValue: variant);
var
  vIndex: integer;
begin
  vIndex := IndexOf(aTag);
  if vIndex <> cIndexNotFound then
    ValuesByIndex[vIndex] := aValue;
end;
function TTaggedValueList.GetValueByIndex(aIndex: integer): variant;
begin
  Result := TTaggedValueItem(List[aIndex]).Value;
end;
procedure TTaggedValueList.SetValueByIndex(aIndex: integer; const aValue: variant);
begin
  TTaggedValueItem(List[aIndex]).Value := aValue;
end;
function TTaggedValueList.Insert(aIndex: integer; aTag: integer; const aValue: variant): integer;
begin
  List.Insert(aIndex, TTaggedValueItem.Create(aTag, aValue));
  Result := aIndex;
end;
function TTaggedValueList.FindTag(aTag: integer; var aIndex: integer): boolean;
var
  vLowIndex, vHighIndex: integer;
begin
  Result := False;
  vLowIndex := 0;
  vHighIndex := Count-1;
  while vLowIndex <= vHighIndex do begin
    aIndex := (vLowIndex + vHighIndex) shr 1;
    if aTag < Tags[aIndex] then
      vLowIndex := aIndex + 1
    else begin
      vHighIndex := aIndex - 1;
      if aTag = Tags[aIndex] then begin
        Result := True;
        vLowIndex := aIndex;
      end;
    end;
  end;
  aIndex := vLowIndex;
end;
procedure TTaggedValueList.Clear;
begin
  DumpItems(List);
end;
function TTaggedValueList.Count: integer;
begin
  Result := List.Count;
end;
function TTaggedValueList.Add(aTag: integer; const aValue: variant): integer;
begin
  if not FindTag(aTag, Result) then
    Result := Insert(Result, aTag, aValue)
  else
    Result := cIndexNotFound;
  Assert(Result <> cIndexNotFound, ResultTextFor(TYPE_E_NAMECONFLICT));
end;
function TTaggedValueList.Let(aTag: integer; const aValue: variant): integer;
begin
  if not FindTag(aTag, Result) then
    Result := Insert(Result, aTag, aValue)
  else
    ValuesByIndex[Result] := aValue;
end;
function TTaggedValueList.IndexOf(aTag: integer): integer;
begin
  if not FindTag(aTag, Result) then
    Result := cIndexNotFound;
end;
function TTaggedValueList.IndexOfValue(const aValue: variant): integer;
var
  vIndex: integer;
begin
  Result := cIndexNotFound;
  for vIndex := 0 to Count-1 do
    if Values[vIndex] = aValue then begin
      Result := vIndex;
      break;
    end;
end;
function TTaggedValueList.TagExists(aTag: integer): boolean;
begin
  Result := IndexOf(aTag) <> cIndexNotFound;
end;
function TTaggedValueList.Remove(aTag: integer): integer;
begin
  Result := IndexOf(aTag);
  if Result <> cIndexNotFound then
    Delete(Result);
end;
procedure TTaggedValueList.Delete(aIndex: integer);
begin
  List.Delete(aIndex);
end;

//==============================================================================
// TObjectList = class(TObject)
//==============================================================================
constructor TObjectList.Create;
begin
  inherited Create;
  FList := TFreeList.Create;
end;
destructor TObjectList.Destroy;
begin
  Dump(FList);
  inherited Destroy;
end;
procedure TObjectList.Clear;
begin
  List.Clear;
end;
function TObjectList.Count: integer;
begin
  Result := List.Count;
end;
function TObjectList.GetItem(aIndex: integer): TObject;
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  Result := List[aIndex];
end;
function TObjectList.Add(aObject: TObject): integer;
begin
  Result := List.Add(aObject);
end;
function TObjectList.Insert(aIndex: integer; aObject: TObject): integer;
begin
  List.Insert(aIndex, aObject);
  Result := aIndex;
end;
function TObjectList.IndexOf(aObject: TObject): integer;
begin
  Result := List.IndexOf(aObject);
end;
function TObjectList.Remove(aObject: TObject): integer;
begin
  Result := IndexOf(aObject);
  Assert(Result <> cIndexNotFound, ResultTextFor(TYPE_E_ELEMENTNOTFOUND));
  if Result <> cIndexNotFound then
    Delete(Result);
end;
procedure TObjectList.Delete(aIndex: integer);
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  List.Delete(aIndex);
end;

//==============================================================================
// TClassList = class(TObject)
//==============================================================================
constructor TClassList.Create;
begin
  inherited Create;
  FList := TList.Create;
end;
destructor TClassList.Destroy;
begin
  Dump(FList);
  inherited Destroy;
end;
procedure TClassList.Clear;
begin
  List.Clear;
end;
function TClassList.Count: integer;
begin
  Result := List.Count;
end;
function TClassList.Add(aClass: TClass): integer;
begin
  Result := List.Add(TObject(aClass));
end;
function TClassList.Insert(aIndex: integer; aClass: TClass): integer;
begin
  List.Insert(aIndex, TObject(aClass));
  Result := aIndex;
end;
function TClassList.IndexOf(aClass: TClass): integer;
begin
  Result := List.IndexOf(TObject(aClass));
end;
function TClassList.GetClass(aIndex: integer): TClass;
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  Result := TClass(List[aIndex]);
end;
function TClassList.Remove(aClass: TClass): integer;
begin
  Result := IndexOf(aClass);
  Assert(Result <> cIndexNotFound, ResultTextFor(TYPE_E_ELEMENTNOTFOUND));
  if Result <> cIndexNotFound then
    Delete(Result);
end;
procedure TClassList.Delete(aIndex: integer);
begin
  Assert((aIndex >= 0) and (aIndex < Count), ResultTextFor(DISP_E_BADINDEX));
  List.Delete(aIndex);
end;

end.
