unit MiscSupport;

{x$define WatchForNil} // remove the x if you want nil's looked out for

// I still need to punch up VarFormat quite a bit

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes,Variants;

//==============================================================================
procedure Dump(var aObject);
procedure DumpList(var aList);
procedure DumpItems(aList: TList);
procedure DumpStringList(var aList);
procedure DumpStringListItems(aList: TStringList);

//==============================================================================
function IntsToComp(const aHigh, aLow: integer): comp;
procedure CompToInts(const aValue: comp; var aHigh, aLow: integer);

//==============================================================================
function BooleanToText(aValue: boolean): string;
function TextToBoolean(const aValue: string): boolean;
function IfThenInt(aValue: boolean; aTrue, aFalse: integer): integer;
function IfThenText(aValue: boolean; const aTrue, aFalse: string): string;
function RandomBoolean: boolean;
function RandomThenInt(aTrue, aFalse: integer): integer;
function RandomThenText(const aTrue, aFalse: string): string;

//==============================================================================
procedure AssertRange(const aIndex, aRange: integer);
function AssureRange(const aValue, aMin, aMax: integer): integer;
function VarAssureRange(const aValue, aMin, aMax: variant): variant;
function SingleAssureRange(const aValue, aMin, aMax: single): single;
function DoubleAssureRange(const aValue, aMin, aMax: double): double;
function WrapRange(const aValue, aMin, aMax: integer): integer;
function VarWrapRange(const aValue, aMin, aMax: variant): variant;
function SingleWrapRange(const aValue, aMin, aMax: single): single;
function DoubleWrapRange(const aValue, aMin, aMax: double): double;

//==============================================================================
procedure VarAssureTypes(const aValue: variant; aTypes: array of integer);
function VarTypeAsText(aValue: variant): string;
function VarAsDisplayText(aValue: variant): string;
function VarFormat(const aPattern: string; aValue: variant): string;

//==============================================================================
function AnsiMatchText(const aText: string; const aValues: array of string): boolean;
function AnsiIndexText(const aText: string; const aValues: array of string): integer;

//==============================================================================
implementation

//==============================================================================
uses
  MiscConstants, ResultSupport;

//==============================================================================
procedure Dump(var aObject);
begin
  {$ifdef WatchForNil}
  Assert(Pointer(aObject) <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  {$endif}

  // validate
  if Pointer(aObject) <> nil then begin

    // wipe out!
    TObject(aObject).Free;
    Pointer(aObject) := nil;
  end;
end;
procedure DumpList(var aList);
begin
  {$ifdef WatchForNil}
  Assert(Pointer(aList) <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  {$endif}

  // validate
  if Pointer(aList) <> nil then begin
    Assert(TObject(aList) is TList, ResultTextFor(TYPE_E_TYPEMISMATCH));

    // wipe out!
    DumpItems(TList(aList));
    Dump(aList);
  end;
end;
procedure DumpItems(aList: TList);
var
  vCount: integer;
begin
  {$ifdef WatchForNil}
  Assert(aList <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  {$endif}

  // do it!
  if aList <> nil then while aList.Count > 0 do begin

    // get rid of the last one
    vCount := aList.Count;
    TObject(aList[vCount-1]).Free;

    // if it did not self-reduce then reduce it ourself
    if vCount = aList.Count then
      aList.Delete(vCount-1);
  end;
end;
procedure DumpStringList(var aList);
begin
  {$ifdef WatchForNil}
  Assert(Pointer(aList) <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  {$endif}

  // validate
  if Pointer(aList) <> nil then begin
    Assert(TObject(aList) is TStringList, ResultTextFor(TYPE_E_TYPEMISMATCH));

    // wipe out!
    DumpStringListItems(TStringList(aList));
    Dump(aList);
  end;
end;
procedure DumpStringListItems(aList: TStringList);
var
  vCount: integer;
begin
  {$ifdef WatchForNil}
  Assert(aList <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  {$endif}

  // do it!
  if aList <> nil then while aList.Count > 0 do begin

    // get rid of the last one
    vCount := aList.Count;
    TObject(aList.Objects[vCount-1]).Free;

    // if it did not self-reduce then reduce it ourself
    if vCount = aList.Count then
      aList.Delete(vCount-1);
  end;
end;
//==============================================================================
type
  THackComp = record
    Low, High: integer;
  end;
function IntsToComp(const aHigh, aLow: integer): comp;
begin
  THackComp(Result).High := aHigh;
  THackComp(Result).Low := aLow;
end;
procedure CompToInts(const aValue: comp; var aHigh, aLow: integer);
begin
  aHigh := THackComp(aValue).High;
  aLow := THackComp(aValue).Low;
end;

//==============================================================================
function BooleanToText(aValue: boolean): string;
begin
  if aValue then
    Result := 'True'
  else
    Result := 'False';
end;
function TextToBoolean(const aValue: string): boolean;
begin
  Result := aValue[1] in ['T', 't', 'Y', 'y'];
end;
function IfThenInt(aValue: boolean; aTrue, aFalse: integer): integer;
begin
  if aValue then
    Result := aTrue
  else
    Result := aFalse;
end;
function IfThenText(aValue: boolean; const aTrue, aFalse: string): string;
begin
  if aValue then
    Result := aTrue
  else
    Result := aFalse;
end;
function RandomBoolean: boolean;
begin
  Result := Random(2) = 0;
end;
function RandomThenInt(aTrue, aFalse: integer): integer;
begin
  Result := IfThenInt(RandomBoolean, aTrue, aFalse);
end;
function RandomThenText(const aTrue, aFalse: string): string;
begin
  Result := IfThenText(RandomBoolean, aTrue, aFalse);
end;

//==============================================================================
procedure AssertRange(const aIndex, aRange: integer);
begin
  Assert((aIndex >= 0) and (aIndex < aRange), ResultTextFor(DISP_E_BADINDEX));
end;
function AssureRange(const aValue, aMin, aMax: integer): integer;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMin;
  if Result > aMax then
    Result := aMax;
end;
function VarAssureRange(const aValue, aMin, aMax: variant): variant;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMin;
  if Result > aMax then
    Result := aMax;
end;
function SingleAssureRange(const aValue, aMin, aMax: single): single;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMin;
  if Result > aMax then
    Result := aMax;
end;
function DoubleAssureRange(const aValue, aMin, aMax: double): double;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMin;
  if Result > aMax then
    Result := aMax;
end;
function WrapRange(const aValue, aMin, aMax: integer): integer;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMax;
  if Result > aMax then
    Result := aMin;
end;
function VarWrapRange(const aValue, aMin, aMax: variant): variant;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMax;
  if Result > aMax then
    Result := aMin;
end;
function SingleWrapRange(const aValue, aMin, aMax: single): single;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMax;
  if Result > aMax then
    Result := aMin;
end;
function DoubleWrapRange(const aValue, aMin, aMax: double): double;
begin
  Result := aValue;
  if Result < aMin then
    Result := aMax;
  if Result > aMax then
    Result := aMin;
end;

//==============================================================================
procedure VarAssureTypes(const aValue: variant; aTypes: array of integer);
var
  vIndex, vType: integer;
  vError: boolean;
begin
  vError := True;
  vType := VarType(aValue);
  for vIndex := Low(aTypes) to High(aTypes) do
    if vType = aTypes[vIndex] then begin
      vError := False;
      break;
    end;
  if vError then
    SetResult(DISP_E_BADVARTYPE);
end;
function VarTypeAsText(aValue: variant): string;
begin
  case VarType(aValue) of
  varEmpty:    Result := 'Empty';
  varNull:     Result := 'Null';
  varSmallint: Result := 'Smallint';
  varInteger:  Result := 'Integer';
  varSingle:   Result := 'Single';
  varDouble:   Result := 'Double';
  varCurrency: Result := 'Currency';
  varDate:     Result := 'Date';
  varError:    Result := 'Error';
  varBoolean:  Result := 'Boolean';
  varVariant:  Result := 'Variant';
  varByte:     Result := 'Byte';
  varString:   Result := 'String';
  varArray:    Result := 'Array';
  else         Result := 'Unknown';
  end;
end;
function VarAsDisplayText(aValue: variant): string;
begin
  Result := VarTypeAsText(aValue) + ': ';
  case VarType(aValue) of
    varSmallint, varInteger, varSingle, varDouble,
    varCurrency, varDate, varBoolean, varString:
      Result := Result + VarAsType(aValue, varString);
  else
    Result := Result + '(' + VarTypeAsText(aValue) + ')';
  end;
end;
function VarFormat(const aPattern: string; aValue: variant): string;
begin
  if VarType(aValue) = varDate then
    Result := FormatDateTime(aPattern, VarAsType(aValue, varDate))
  else
    Result := Format(aPattern, [aValue]);
end;

//==============================================================================
function AnsiMatchText(const aText: string; const aValues: array of string): boolean;
begin
  Result := AnsiIndexText(aText, aValues) <> cIndexNotFound;
end;
function AnsiIndexText(const aText: string; const aValues: array of string): integer;
var
  vIndex: integer;
begin
  Result := cIndexNotFound;
  for vIndex := Low(aValues) to High(aValues) do
    if AnsiCompareText(aText, aValues[vIndex]) = 0 then begin
      Result := vIndex;
      break;
    end;
end;

end.
