unit ParserSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, MiscSupport, MiscConstants, TokenSupport;

//==============================================================================
type
  //============================================================================
  TParserCursor = class(TObject)
  private
    FText: string;
    FAtEnd: boolean;
    FBackAt, FAt, FTextLength: integer;
  public
    // construction
    constructor Create(const aText: string; aAt: integer); virtual;

    // parsing primitives
    property Text: string read FText;
    property At: integer read FAt;
    property TextLength: integer read FTextLength;
    procedure SetAt(aValue: integer);
    procedure MoveAt(aDelta: integer);
    property AtEnd: boolean read FAtEnd;
    function SkipWhiteSpaces: boolean;
    procedure Backup; {only useful with NextSymbol and NextDigits}

    // snatch and grab
    function NextIs(const aSymbol: string): boolean; virtual;
    function NextNCharsIs(aLength: integer; aChars: TCharSet): string; virtual;
    function NextTokenOf(aLexicon: TTokenLexicon): TTokenObject; virtual;
    function NextSymbol: string; virtual;
    function NextSymbolIs(const aSymbols: array of string): string; virtual;
    function NextSymbolIsN(const aSymbols: array of string): integer; virtual;
    function NextChar: char; virtual;
    function NextDigits: string; virtual;

    // where we are
    function TheChar: char; virtual;
    function TheSymbol: string; virtual;
    function TheDigits: string; virtual;
  end;

{var
  //vAtEnd: integer = 0;
  vSetAt: integer = 0;
  vMoveAt: integer = 0;}

//==============================================================================
implementation

//============================================================================
// TParserCursor = class(TObject)
//============================================================================
// construction
constructor TParserCursor.Create(const aText: string; aAt: integer);
begin
  inherited Create;
  FText := aText;
  FTextLength := Length(Text);
  SetAt(aAt)
end;

//==============================================================================
// parsing primitives
function TParserCursor.SkipWhiteSpaces: boolean;
var
  vAt: integer;
begin
  // where did we start?
  vAt := At;

  // for now
  while TheChar in cWhiteSpace do
    NextChar;

  // comments
  { skip white spaces
    if // then skip to eol
    else if /* skip to matching */
    skip white spaces}

  // did we move?
  Result := vAt <> At;
end;
{function TParserCursor.AtEnd: boolean;
begin
  Result := At = -1;
  Inc(vAtEnd);
end;}
procedure TParserCursor.SetAt(AValue: integer);
begin
  FAt := AValue;
  FAtEnd := At > TextLength;
  //if At > TextLength then
    //FAt := -1;
  {Inc(vSetAt);}
end;
procedure TParserCursor.MoveAt(ADelta: integer);
begin
  Inc(FAt, ADelta);
  FAtEnd := At > TextLength;
  //if At > TextLength then
    //FAt := -1;
  {Inc(vMoveAt);}
end;
procedure TParserCursor.Backup;
begin

end;


//==============================================================================
// snatch and grab
function TParserCursor.NextIs(const aSymbol: string): boolean;
var
  vText: string;
  vLabelWatch: boolean;
  vSymbolLength: integer;
begin
  // assume failure
  Result := False;

  // is this symbol a label?
  vLabelWatch := ASymbol[1] in cSymbolStart;
  vSymbolLength := Length(aSymbol);

  // past the end yet?
  if not AtEnd then begin

    // skip white spaces
    SkipWhiteSpaces;

    // still more?
    if not AtEnd then begin

      // grab the next symbol
      vText := Copy(Text, At, vSymbolLength);

      // is it the right one?
      if (vText <> '') and
         (AnsiCompareText(vText, aSymbol) = 0) and
         (not vLabelWatch or
          not (Text[At + vSymbolLength] in cSymbolStart)) then begin
        MoveAt(vSymbolLength);
        Result := True;
      end;
    end;
  end;
end;
function TParserCursor.NextNCharsIs(aLength: integer; aChars: TCharSet): string;
var
  Index: integer;
begin
  Result := '';
  if (At <> -1) and
     (At + aLength < Length(Text)) then begin
    Result := Copy(Text, At, aLength);
    for Index := 1 to aLength do
      if not (Result[Index] in aChars) then begin
        Result := '';
        Break;
      end;
    if Result <> '' then
      MoveAt(aLength);
  end;
end;
function TParserCursor.NextTokenOf(aLexicon: TTokenLexicon): TTokenObject;
var
  Index: integer;
begin
  Result := nil;
  for Index := aLexicon.Count-1 downto 0 do
    if NextIs(aLexicon.Tokens[Index].Symbol) then begin
      Result := aLexicon.Tokens[Index].Create;
      break;
    end;
end;
function TParserCursor.NextSymbol: string;
begin
  // remember where we where
  FBackAt := At;

  // assume failure
  Result := '';

  // skip white spaces
  SkipWhiteSpaces;

  // from the right stuff?
  if TheChar in cSymbolStart then begin
    Result := NextChar;

    // and continues to be?
    while (TheChar in cSymbolFinish) do
      Result := Result + NextChar;
  end;
end;
function TParserCursor.NextSymbolIs(const aSymbols: array of string): string;
var
  Index: integer;
begin
  Result := '';
  Index := NextSymbolIsN(aSymbols);
  if Index <> cIndexNotFound then
    Result := aSymbols[Index];
end;
function TParserCursor.NextSymbolIsN(const aSymbols: array of string): integer;
var
  vSymbol: string;
  Index: integer;
begin
  Result := cIndexNotFound;
  vSymbol := TheSymbol;
  for Index := Low(aSymbols) to High(aSymbols) do
    if AnsiCompareText(vSymbol, aSymbols[Index]) = 0 then begin
      Result := Index;
      NextSymbol;
      break;
    end;
end;
function TParserCursor.NextChar: char;
begin
  // moving on
  Result := TheChar;
  if At <> -1 then
    MoveAt(1);
end;
function TParserCursor.NextDigits: string;
begin
  // remember where we where
  FBackAt := At;

  // assume failure
  Result := '';

  // skip white spaces
  SkipWhiteSpaces;

  // good characters?
  while (TheChar in cDigits) do begin
    Result := Result + TheChar;
    NextChar;
  end;
end;


//==============================================================================
// where we are
function TParserCursor.TheChar: char;
begin
  // only if we are ok
  if At <> -1 then
    Result := Text[At]
  else
    Result := #0;
end;
function TParserCursor.TheSymbol: string;
begin
  // grab the next symbol
  Result := NextSymbol;

  // back up a bit
  Backup;
end;
function TParserCursor.TheDigits: string;
begin
  // grab the next digit set
  Result := NextDigits;

  // back up a bit
  Backup;
end;

end.
