unit PropertySupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, TypInfo,Variants;

//==============================================================================
function PropertyExists(aObject: TObject; const aName: string): boolean;
function GetPropertyValue(aObject: TObject; const aName: string): variant;
procedure SetPropertyValue(aObject: TObject; const aName: string; aValue: variant);

//==============================================================================
function ParentPropertyOf(var aName: string): string;
function CompoundProperty(aName: string): boolean;
function PropertyInfoOf(var aObject: TObject; aName: string): PPropInfo;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport;

//==============================================================================
function ParentPropertyOf(var aName: string): string;
var
  Index: integer;
begin
  Index := AnsiPos('.', aName);
  if Index > 0 then begin
    Result := Copy(aName, 1, Index-1);
    aName := Copy(aName, Index+1, Length(aName));
  end else begin
    Result := aName;
    aName := '';
  end;
end;
function CompoundProperty(aName: string): boolean;
begin
  Result := AnsiPos('.', aName) <> 0;
end;
function PropertyInfoOf(var aObject: TObject; aName: string): PPropInfo;
var
  vName: string;
begin
  Assert(aObject <> nil, ResultTextFor(STG_E_INVALIDHANDLE));
  Assert(aName <> '', ResultTextFor(DISP_E_PARAMNOTOPTIONAL));

  // assume failure
  Result := nil;

  // do we have a compound property here?
  if CompoundProperty(aName) then begin
    while aName <> '' do begin
      vName := ParentPropertyOf(aName);
      Result := GetPropInfo(aObject.ClassInfo, vName);
      if (aName = '') or
         (Result = nil) then
        break;
      if Result.PropType^^.Kind <> tkClass then begin
        Result := nil;
        break;
      end;
      aObject := TObject(GetOrdProp(aObject, Result));
    end;
  end else
    Result := GetPropInfo(aObject.ClassInfo, aName);
end;
function PropertyExists(aObject: TObject; const aName: string): boolean;
var
  vPropInfo: PPropInfo;
begin
  // assume failure
  Result := False;

  // get the prop info
  vPropInfo := PropertyInfoOf(aObject, aName);
  if vPropInfo <> nil then

    // we only care about the following...
    case vPropInfo.PropType^^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet, tkWChar, tkFloat,
      tkString, tkLString, tkWString, tkVariant:
        Result := True;
    end;
end;
function GetPropertyValue(aObject: TObject; const aName: string): variant;
var
  vPropInfo: PPropInfo;
begin
  // assume failure
  Result := Unassigned;

  // get the prop info
  vPropInfo := PropertyInfoOf(aObject, aName);
  if vPropInfo <> nil then

    // return the right type
    case vPropInfo.PropType^^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet, tkWChar:
        Result := GetOrdProp(aObject, vPropInfo);
      tkFloat: begin
        Result := GetFloatProp(aObject, vPropInfo);

        // TDateTime hack
        if vPropInfo.PropType^^.Name = 'TDateTime' then
          Result := VarAsType(Result, varDate);
      end;
      tkString, tkLString, tkWString:
        Result := GetStrProp(aObject, vPropInfo);
      tkVariant:
        Result := GetVariantProp(aObject, vPropInfo);
    else
      Assert(False, ResultTextFor(DISP_E_BADVARTYPE));
    end;
end;
procedure SetPropertyValue(aObject: TObject; const aName: string; aValue: variant);
var
  vPropInfo: PPropInfo;
begin
  // get the prop info
  vPropInfo := PropertyInfoOf(aObject, aName);
  if vPropInfo <> nil then

    // set the right type
    case vPropInfo.PropType^^.Kind of
      tkInteger, tkChar, tkEnumeration, tkSet, tkWChar:
        SetOrdProp(aObject, vPropInfo, aValue);
      tkFloat:
        SetFloatProp(aObject, vPropInfo, aValue);
      tkString, tkLString, tkWString:
        SetStrProp(aObject, vPropInfo, aValue);
      tkVariant:
        SetVariantProp(aObject, vPropInfo, aValue);
    else
      Assert(False, ResultTextFor(DISP_E_BADVARTYPE));
    end;
end;

end.
