unit ResultSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, SysUtils, Classes;

//==============================================================================
function AnyErrors: boolean;

//==============================================================================
function LastResult: HResult;
function LastResultCode: integer;
function LastResultText: string;
function LastResultAt: integer;

//==============================================================================
function SetResult(aCode: HResult): HResult;
function SetResultAt(aCode: HResult; aAt: integer): HResult;
function ClearResult: HResult;

//==============================================================================
function SetFacilityError(aFacility, aCode: Integer): HResult;
function SetFacilityErrorAt(aFacility, aCode, aAt: Integer): HResult;

//==============================================================================
function SetToWindowsError: HResult;
function SetError(aCode: Integer): HResult;
function SetErrorText(aCode: Integer; const aText: string): HResult;
function SetErrorAt(aCode, aAt: Integer): HResult;
function SetErrorException: HResult;

//==============================================================================
procedure MoveResultAtTo(aAt: Integer);
function ResultTextFor(aResult: HResult): string;

//==============================================================================
implementation

//==============================================================================
uses
  MiscConstants;

//==============================================================================
var
  vResult: HResult;
  vResultText: string;
  vResultAt: integer;

//============================================================================
function AnyErrors: boolean;
begin
  Result := Failed(vResult);
end;

//==============================================================================
function LastResult: HResult;
begin
  Result := vResult;
end;
function LastResultCode: integer;
begin
  Result := HResultCode(vResult);
end;
function LastResultText: string;
begin
  Result := vResultText;
  if Result = '' then
    Result := ResultTextFor(vResult);
end;
function LastResultAt: integer;
begin
  Result := vResultAt;
end;

//==============================================================================
function SetResult(aCode: HResult): HResult;
begin
  Result := SetResultAt(aCode, cIndexNotFound);
end;
function SetResultAt(aCode: HResult; aAt: integer): HResult;
begin
  vResult := aCode;
  vResultAt := aAt;
  vResultText := '';
  Result := vResult;
end;
function ClearResult: HResult;
begin
  Result := SetResult(NOERROR);
end;

//==============================================================================
function SetFacilityError(aFacility, aCode: Integer): HResult;
begin
  Result := SetFacilityErrorAt(aFacility, aCode, cIndexNotFound);
end;
function SetFacilityErrorAt(aFacility, aCode, aAt: Integer): HResult;
begin
  Assert((aFacility and $fe00ffff) = 0, ResultTextFor(DISP_E_BADINDEX));
  Assert((aCode and $ffff0000) = 0, ResultTextFor(DISP_E_BADINDEX));
  Result := SetResultAt(MakeResult(SEVERITY_ERROR, aFacility, aCode), aAt);
end;

//==============================================================================
function SetToWindowsError: HResult;
begin
  Result := SetResult(GetLastError);
end;
function SetError(aCode: Integer): HResult;
begin
  Result := SetErrorAt(aCode, cIndexNotFound);
end;
function SetErrorText(aCode: Integer; const aText: string): HResult;
begin
  Result := SetErrorAt(aCode, cIndexNotFound);
  vResultText := aText;
end;
function SetErrorAt(aCode, aAt: Integer): HResult;
begin
  if (aCode and $ffff0000) <> 0 then
    Result := SetResultAt(aCode, aAt)
  else
    Result := SetFacilityErrorAt(FACILITY_NULL, aCode, aAt);
end;
function SetErrorException: HResult;
begin
  Result := SetResult(E_UNEXPECTED);
  vResultText := Exception(ExceptObject).Message;
end;

//==============================================================================
procedure MoveResultAtTo(aAt: Integer);
begin
  vResultAt := aAt;
end;
function ResultTextFor(aResult: HResult): string;
begin
  Result := SysErrorMessage(aResult);
  if Result = '' then begin
    if Failed(aResult) then
      Result := 'ERROR'
    else
      Result := 'STATUS';
    Result := Result + Format(': (%d) %d', [HResultFacility(aResult),
                                            HResultCode(aResult)]);
  end;
end;

end.
