unit StackSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, MiscSupport;

//==============================================================================
type
  //============================================================================
  TStack = class(TList)
  public
    function Atleast(aCount: integer): boolean;
    procedure Push(aItem: pointer); virtual;
    function Pop: pointer; virtual;
    function Peek: pointer; virtual;
  end;

  //============================================================================
  TObjectStack = class(TObject)
  private
    FStack: TStack;
  protected
    property Stack: TStack read FStack;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Clear; virtual;

    function Count: integer; virtual;
    function Atleast(aCount: integer): boolean; virtual;
    procedure PushObject(aObject: TObject); virtual;
    function PopObject: TObject; virtual;
    function PeekObject: TObject; virtual;
  end;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport;

//==============================================================================
// TStack = class(TList)
//==============================================================================
function TStack.Atleast(aCount: integer): boolean;
begin
  Result := Count >= aCount;
end;
procedure TStack.Push(AItem: pointer);
begin
  Add(AItem);
end;
function TStack.Pop: pointer;
begin
  Result := Peek;
  Delete(Count-1);
end;
function TStack.Peek: pointer;
begin
  Assert(Count > 0, ResultTextFor(DISP_E_BADPARAMCOUNT));
  Result := Items[Count-1];
end;

//==============================================================================
// TObjectStack = class(TObject)
//==============================================================================
constructor TObjectStack.Create;
begin
  inherited Create;
  FStack := TStack.Create;
end;
destructor TObjectStack.Destroy;
begin
  Clear;
  Dump(FStack);
  inherited Destroy;
end;
procedure TObjectStack.Clear;
begin
  DumpItems(FStack);
end;
function TObjectStack.Count: integer;
begin
  Result := Stack.Count;
end;
function TObjectStack.Atleast(aCount: integer): boolean;
begin
  Result := Stack.Atleast(aCount);
end;
procedure TObjectStack.PushObject(aObject: TObject);
begin
  Stack.Push(aObject);
end;
function TObjectStack.PopObject: TObject;
begin
  Result := Stack.Pop;
end;
function TObjectStack.PeekObject: TObject;
begin
  Result := Stack.Peek;
end;

end.
