unit StringSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes;

//==============================================================================
function TextFieldCount(const aText: string): integer;
function TextFieldOf(const aText: string; aIndex: integer): string;
function TextWordCount(const aText: string): integer;
function TextWordOf(const aText: string; aIndex: integer): string;
function TextLineCount(const aText: string): integer;
function TextLineOf(const aText: string; aIndex: integer): string;

//==============================================================================
function DelimToDelim(const aText: string; aOldDelim, aNewDelim: char): string;

//==============================================================================
function CorrectFilenameCase(const aName: string): string;

//==============================================================================
function AnsiDequoteStr(const aText: string; aQuote: Char): string;

//==============================================================================
function BytesToText(aBytes: integer): string;
function BytesToEnglish(aBytes: integer): string;

//==============================================================================
function AnsiCompare(const aText1, aText2: string): boolean;
function AnsiCompares(const aText: string; const aTexts: array of string): boolean;
function AnsiCompareN(const aText: string; const aTexts: array of string): integer;

//==============================================================================
var
  vTextFieldDefault: string = '';
  vTextWordDefault: string = '';
  vTextLineDefault: string = '';
  vQuoteChar: char = '"';
  vEscapeChar: char = '/';

//==============================================================================
implementation

//==============================================================================
uses
{$ifdef UseXProcs}
  xProcs;
{$else}
  MiscConstants;
{$endif}

//==============================================================================
function TextFieldCount(const aText: string): integer;
var
  vStrings: TStringList;
begin
  Result := -1;
  vStrings := TStringList.Create;
  if vStrings <> nil then with vStrings do try
    CommaText := aText;
    Result := Count;
  finally
    Free;
  end;
end;
function TextFieldOf(const aText: string; aIndex: integer): string;
var
  vStrings: TStringList;
begin
  Result := vTextFieldDefault;
  vStrings := TStringList.Create;
  if vStrings <> nil then with vStrings do try
    CommaText := aText;
    if (aIndex >= 0) and (aIndex < Count) then
      Result := Strings[aIndex];
  finally
    Free;
  end;
end;

//==============================================================================
function TextWordCount(const aText: string): integer;
begin
  {$ifdef UseXProcs}
  Result := strTokenCount(aText, ' ');
  {$else}
  Result := cIndexNotFound;
  {$endif}
end;
function TextWordOf(const aText: string; aIndex: integer): string;
begin
  {$ifdef UseXProcs}
  Result := strTokenAt(aText, ' ', aIndex);
  {$else}
  Result := vTextWordDefault;
  {$endif}
end;

//==============================================================================
function TextLineCount(const aText: string): integer;
var
  vStrings: TStringList;
begin
  Result := -1;
  vStrings := TStringList.Create;
  if vStrings <> nil then with vStrings do try
    Text := aText;
    Result := Count;
  finally
    Free;
  end;
end;
function TextLineOf(const aText: string; aIndex: integer): string;
var
  vStrings: TStringList;
begin
  Result := vTextLineDefault;
  vStrings := TStringList.Create;
  if vStrings <> nil then with vStrings do try
    Text := aText;
    if (aIndex >= 0) and (aIndex < Count) then
      Result := Strings[aIndex];
  finally
    Free;
  end;
end;

//==============================================================================
function DelimToDelim(const aText: string; aOldDelim, aNewDelim: char): string;
var
  vIndex, vLength: integer;
  vQuoted: boolean;
  vChar: Char;
begin
  Result := '';
  vIndex := 1;
  vLength := Length(aText);
  vQuoted := False;
  while vIndex <= vLength do begin
    vChar := aText[vIndex];

    if vChar = vQuoteChar then
      vQuoted := not vQuoted

    else if vChar = vEscapeChar then begin
      Inc(vIndex);
      vChar := aText[vIndex];

    end else if (not vQuoted) and
                (vChar = aOldDelim) then
      vChar := aNewDelim;

    Result := Result + vChar;
    Inc(vIndex);
  end;
end;

//==============================================================================
function CorrectFilenameCase(const aName: string): string;
begin
  if AnsiCompareStr(AnsiUppercaseFilename(aName),
                    aName) = 0 then
    Result := AnsiUppercaseFilename(Copy(aName, 1, 1)) +
              AnsiLowercaseFilename(Copy(aName, 2, MAX_PATH))
  else
    Result := aName;
end;

//==============================================================================
function AnsiDequoteStr(const aText: string; aQuote: Char): string;
var
  vText: PChar;
begin
  vText := PChar(aText);
  Result := AnsiExtractQuotedStr(vText, aQuote);
  if Result = '' then
    Result := aText;
end;

//==============================================================================
function BytesToText(aBytes: integer): string;
begin
  // convert gigs
  if aBytes >= cGigabyte then begin
    if (aBytes mod cGigabyte) > 0 then
      aBytes := aBytes div cGigabyte + 1
    else
      aBytes := aBytes div cGigabyte;
    Result := IntToStr(aBytes) + 'GB'

  // convert megs
  end else if aBytes >= cMegabyte then begin
    if (aBytes mod cMegabyte) > 0 then
      aBytes := aBytes div cMegabyte + 1
    else
      aBytes := aBytes div cMegabyte;
    Result := IntToStr(aBytes) + 'MB'

  // convert kilos
  end else if aBytes >= cKilobyte then begin
    if (aBytes mod cKilobyte) > 0 then
      aBytes := aBytes div cKilobyte + 1
    else
      aBytes := aBytes div cKilobyte;
    Result := IntToStr(aBytes) + 'KB'

  // convert the smaller items
  end else
    Result := '1KB'
end;
function BytesToEnglish(aBytes: integer): string;
var
  vText: string;
  procedure EnglishFor(aLevel: integer; const aText: string);
  var
    vValue: integer;
  begin
    Result := '';
    vValue := aBytes div aLevel;
    if vValue > 0 then begin
      if vText <> '' then
        vText := vText + ' ';
      vText := vText + IntToStr(vValue) + ' ' + vText;
      aBytes := aBytes - vValue * aLevel;
    end;
  end;
begin
  if aBytes = 0 then
    vText := 'empty'
  else begin
    EnglishFor(cGigabyte, 'Billion');
    EnglishFor(cMegabyte, 'Million');
    EnglishFor(cKilobyte, 'Thousand');
    if aBytes > 0 then begin
      if vText <> '' then
        vText := vText + ' and ';
      vText := vText + IntToStr(aBytes);
    end;
    vText := vText + ' Bytes';
  end;
  Result := vText;
end;

//==============================================================================
function AnsiCompare(const aText1, aText2: string): boolean;
begin
  Result := AnsiCompareText(aText1, aText2) = 0;
end;
function AnsiCompares(const aText: string; const aTexts: array of string): boolean;
begin
  Result := AnsiCompareN(aText, aTexts) <> cIndexNotFound;
end;
function AnsiCompareN(const aText: string; const aTexts: array of string): integer;
var
  vIndex: integer;
begin
  Result := cIndexNotFound;
  for vIndex := Low(aTexts) to High(aTexts) do
    if AnsiCompare(aText, aTexts[vIndex]) then begin
      Result := vIndex;
      break;
    end;
end;

end.
