unit SymbolSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ListSupport;

//==============================================================================
type
  //============================================================================
  TSymbolObject = class(TPersistent)
  private
    FSymbol: string;
    FReference: TObject;
  public
    constructor Create(const aSymbol: string; aReference: TObject);
    property Symbol: string read FSymbol;
    property Reference: TObject read FReference;
  end;

  //============================================================================
  TSymbolList = class(TObjectList)
  protected
    function GetSymbol(index: integer): TSymbolObject;
  public
    function Add(aObject: TSymbolObject): integer;
    property Symbols[index: integer]: TSymbolObject read GetSymbol; default;
    function Lookup(const aSymbol: string): TSymbolObject;
    function LookupIndex(const aSymbol: string): integer;
    function LookupReference(aReference: TObject): TSymbolObject;
    function Exists(const aSymbol: string): boolean;
    function Dump(const aSymbol: string): integer; 
  end;

//==============================================================================
function SymbolDictionary: TSortedStringList;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscConstants, MiscSupport;

//==============================================================================
// TSymbolObject = class(TObject)
//==============================================================================
constructor TSymbolObject.Create(const aSymbol: string; aReference: TObject);
begin
  inherited Create;
  Assert(IsValidIdent(aSymbol), ResultTextFor(TYPE_E_AMBIGUOUSNAME));
  FSymbol := aSymbol;
  FReference := aReference;
end;

//==============================================================================
// TSymbolList = class(TObjectList)
//==============================================================================
function TSymbolList.Add(aObject: TSymbolObject): integer;
begin
  Assert(Lookup(aObject.Symbol) = nil, ResultTextFor(TYPE_E_NAMECONFLICT));
  Result := inherited Add(aObject);
end;
function TSymbolList.GetSymbol(index: integer): TSymbolObject;
begin
  Result := TSymbolObject(Objects[index]);
end;
function TSymbolList.Lookup(const aSymbol: string): TSymbolObject;
var
  Index: integer;
begin
  Result := nil;
  Index := LookupIndex(aSymbol);
  if Index <> cIndexNotFound then
    Result := Symbols[Index];
end;
function TSymbolList.LookupIndex(const aSymbol: string): integer;
var
  Index: integer;
begin
  Result := cIndexNotFound;
  for Index := 0 to Count-1 do
    if AnsiCompareText(Symbols[Index].Symbol, aSymbol) = 0 then begin
      Result := Index;
      break;
    end;
end;
function TSymbolList.Exists(const aSymbol: string): boolean;
begin
  Result := LookupIndex(aSymbol) <> cIndexNotFound;
end;
function TSymbolList.LookupReference(aReference: TObject): TSymbolObject;
var
  Index: integer;
begin
  Result := nil;
  for Index := 0 to Count-1 do
    if Symbols[Index].Reference = aReference then begin
      Result := Symbols[Index];
      break;
    end;
end;
function TSymbolList.Dump(const aSymbol: string): integer;
begin
  Result := LookupIndex(aSymbol);
  if Result <> cIndexNotFound then
    Delete(Result);
end;

//==============================================================================
var
  vSymbolDictionary: TSortedStringList = nil;
  vUnitOk: boolean = True;
function SymbolDictionary: TSortedStringList;
begin
  if vUnitOk and
     (vSymbolDictionary = nil) then
    vSymbolDictionary := TSortedStringList.Create;
  Result := vSymbolDictionary;
end;

//==============================================================================
initialization
  SymbolDictionary;
finalization
  vUnitOk := False;
  Dump(vSymbolDictionary);
end.
