unit TokenSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, ListSupport;

//==============================================================================
type
  //============================================================================
  TTokenObject = class(TObject)
  private
    FAt: integer;
  public
    class function Symbol: string; virtual;

    property At: integer read FAt write FAt;
    function Caption: string; virtual;
  end;
  TTokenClass = class of TTokenObject;

  //============================================================================
  TTokenList = class(TObjectList)
  protected
    function GetToken(Index: integer): TTokenObject;
  public
    property Tokens[Index: integer]: TTokenObject read GetToken;
    procedure AsStrings(aStrings: TStrings);
  end;

  //============================================================================
  TTokenLexicon = class(TClassList)
  protected
    function GetToken(Index: integer): TTokenClass;
    function FindSymbol(const aSymbol: string; var Index: integer): boolean;
  public
    function Add(aClass: TTokenClass): integer;
    property Tokens[Index: integer]: TTokenClass read GetToken;
    function Lookup(const aSymbol: string): TTokenClass;
  end;

//==============================================================================
implementation

//==============================================================================
uses
  ResultSupport, MiscConstants, SymbolSupport;

//==============================================================================
// TTokenObject = class(TObject)
//==============================================================================
class function TTokenObject.Symbol: string;
begin
  Result := '{undefined}';
end;
function TTokenObject.Caption: string;
begin
  Result := Symbol;
end;

//==============================================================================
// TTokenList = class(TObjectList)
//==============================================================================
function TTokenList.GetToken(Index: integer): TTokenObject;
begin
  Result := TTokenObject(Objects[Index]);
end;
procedure TTokenList.AsStrings(aStrings: TStrings);
var
  Index: integer;
begin
  // clear it out
  aStrings.Clear;

  // something to do?
  if Count > 0 then
    for Index := 0 to Count-1 do
      aStrings.Add(Tokens[Index].Caption)

  // report the error, if any
  else if AnyErrors then
    aStrings.Add(LastResultText);
end;

//==============================================================================
// TTokenLexicon = class(TClassList)
//==============================================================================
function TTokenLexicon.Add(aClass: TTokenClass): integer;
begin
  if not FindSymbol(aClass.Symbol, Result) then begin
    Result := Insert(Result, aClass);
    SymbolDictionary.Add(aClass.Symbol);
  end else
    Result := cIndexNotFound;
  Assert(Result <> cIndexNotFound, ResultTextFor(TYPE_E_NAMECONFLICT));
end;
function TTokenLexicon.GetToken(Index: integer): TTokenClass;
begin
  Result := TTokenClass(Classes[Index]);
end;
function TTokenLexicon.FindSymbol(const aSymbol: string; var Index: integer): boolean;
var
  LowIndex, HighIndex, Compare: integer;
begin
  Result := False;
  LowIndex := 0;
  HighIndex := Count-1;
  while LowIndex <= HighIndex do begin
    Index := (LowIndex + HighIndex) shr 1;
    Compare := AnsiCompareText(Tokens[Index].Symbol, aSymbol);
    if Compare < 0 then
      LowIndex := Index + 1
    else begin
      HighIndex := Index - 1;
      if Compare = 0 then begin
        Result := True;
        LowIndex := Index;
      end;
    end;
  end;
  Index := LowIndex;
end;
function TTokenLexicon.Lookup(const aSymbol: string): TTokenClass;
var
  Index: integer;
begin
  Result := nil;
  if FindSymbol(aSymbol, Index) then
    Result := Tokens[Index];
end;

end.
