unit VersionInfoSupport;

//==============================================================================
interface

//==============================================================================
uses
  Windows, Messages, SysUtils, Classes, Forms, ListSupport, MiscSupport,Variants;

//==============================================================================
type
  TVersionIds = (viCompanyName, viFileDescription,
                 viFileVersion, viInternalName,
                 viLegalCopyright, viLegalTradeMarks,
                 viOriginalFilename, viProductName,
                 viProductVersion, viComments);

//==============================================================================
const
  cVersionIds: array [TVersionIds] of string = ('CompanyName', 'FileDescription',
                                                'FileVersion', 'InternalName',
                                                'LegalCopyright', 'LegalTradeMarks',
                                                'OriginalFilename', 'ProductName',
                                                'ProductVersion', 'Comments');

//==============================================================================
type
  TVersionInfo = class(TObject)
  private
    FFilename: string;
    FData: Pointer;
  protected
    function GetValue(aIdent: string): string; virtual;
  public
    constructor Create(aFilename: string); virtual;
    destructor Destroy; override;

    property Filename: string read FFilename;
    property Data: Pointer read FData;
    property Values[aIdent: string]: string read GetValue; default;
    function Defined(const aIdent: string): boolean;
    function Info: PVSFixedFileInfo;
  end;

//==============================================================================
function VersionInfo(const aName: string): string;
function VersionInfoFormatted(const aName, aFormat: string): string;
function VersionInfoFormattedArray(const aNames, aFormats: array of string): string;
function VersionInfoOf(const aFilename, aName: string): string;
function VersionInfoOfFormatted(const aFilename, aName, aFormat: string): string;
function VersionInfoOfFormattedArray(const aFilename: string; const aNames, aFormats: array of string): string;

//==============================================================================
const
  cVersionInfoPrefix = 'StringFileInfo\040904E4\';

//==============================================================================
implementation

//==============================================================================
// TVersionInfo = class(TObject)
//==============================================================================
constructor TVersionInfo.Create(aFilename: string);
var

  vSize: {$ifndef ver120}integer{$else}cardinal{$endif};
begin
  // call dad
  inherited Create;

  // make sure we have a filename
  if aFilename = '' then
    FFilename := Application.ExeName
  else
    FFilename := aFilename;

  // how big is it?
  //vSize := GetFileVersionInfoSize(PChar(Filename), vSize);
  if vSize > 0 then begin
    FData := AllocMem(vSize);
    try

      // get the info
      GetFileVersionInfo(PChar(Filename), 0, vSize, FData);
    except
      FreeMem(FData);
      FData := nil;
    end;
  end;
end;
destructor TVersionInfo.Destroy;
begin
  // put away our toys
  if FData <> nil then begin
    FreeMem(FData);
    FData := nil;
  end;

  // call dad
  inherited Destroy;
end;
function TVersionInfo.GetValue(aIdent: string): string;
var
  vValue: PChar;
  vSize: {$ifndef ver120}integer{$else}cardinal{$endif};
begin
  Result := '';
//  if (FData <> nil) and
//     VerQueryValue(Data, PChar(cVersionInfoPrefix + aIdent),
//                   Pointer(vValue), vSize) then
//    Result := vValue;
end;
function TVersionInfo.Defined(const aIdent: string): boolean;
begin
  Result := (FData <> nil) and
            (Values[aIdent] <> '');
end;
function TVersionInfo.Info: PVSFixedFileInfo;
var
  vValue: pointer;
  vSize: {$ifndef ver120}integer{$else}cardinal{$endif};
begin
  Result := nil;
//  if (FData <> nil) and
//     VerQueryValue(Data, PChar('\'), vValue, vSize) then
//    Result := vValue;
end;

//==============================================================================
function VersionInfo(const aName: string): string;
begin
  Result := VersionInfoFormatted(aName, '%s');
end;
function VersionInfoFormatted(const aName, aFormat: string): string;
begin
  Result := VersionInfoFormattedArray([aName], [aFormat]);
end;
function VersionInfoFormattedArray(const aNames, aFormats: array of string): string;
begin
  Result := VersionInfoOfFormattedArray('', aNames, aFormats);
end;

//==============================================================================
function VersionInfoOf(const aFilename, aName: string): string;
begin
  Result := VersionInfoOfFormatted(aFilename, aName, '%s');
end;
function VersionInfoOfFormatted(const aFilename, aName, aFormat: string): string;
begin
  Result := VersionInfoOfFormattedArray(aFilename, [aName], [aFormat]);
end;
function VersionInfoOfFormattedArray(const aFilename: string; const aNames, aFormats: array of string): string;
var
  vInfo: TVersionInfo;
  vIndex: integer;
begin
  // assure everything is on the up and up
  Assert((Low(aNames) = Low(aFormats)) and
         (High(aNames) = High(aFormats)), 'Mismatch array sizes');

  // get the version info
  vInfo := TVersionInfo.Create(aFilename);
  try

    // now format and return the information
    Result := '';
    for vIndex := Low(aNames) to High(aNames) do
      if vInfo.Defined(aNames[vIndex]) then begin
        if Result <> '' then
          Result := Result + #10;
        Result := Result + Format(aFormats[vIndex], [vInfo[aNames[vIndex]]]);
      end;

  // clean up!
  finally
    Dump(vInfo);
  end;
end;

end.
