unit MainUnit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Symbolic, Parser10, StdCtrls, ParseExpr, RbwParser,
  uPSCompiler, uPSRuntime, 
//  ifpscomp, ifps3, ifps3lib_std, ifps3lib_stdr, ifpidelphi, ifpidelphiruntime,
  JclExprEval, ExpressionParser, Expressioneer, ExpressIt,
  CalcExpress, PegtopFormulas, SsExpEval, MathParser, StBase, StExpr;

type
  TForm1 = class(TForm)
    PaintBox1: TPaintBox;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Button1: TButton;
    Parser1: TParser;
    RadioGroup1: TRadioGroup;
    Edit4: TEdit;
    Label4: TLabel;
    RbwParser1: TRbwParser;
    Label5: TLabel;
    Label6: TLabel;
    ExpressIt1: TExpressIt;
    MathParser1: TMathParser;
    PegtopFormula1: TPegtopFormula;
    cbDraw: TCheckBox;
    ExpressionEvaluator1: TExpressionEvaluator;
    StExpression1: TStExpression;
    procedure PaintBox1Paint(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ExpressIt1LateValue(Sender: TObject; const aSymbol: String;
      var aValue: Variant);
    procedure MathParser1GetVar(Sender: TObject; VarName: String;
      var Value: Extended; var Found: Boolean);
    procedure MathParser1ParseError(Sender: TObject; ParseError: Integer);
    procedure cbDrawClick(Sender: TObject);
    procedure ExpressionEvaluator1GetValue(Sender: TObject;
      Identifier: String; var Value: Extended; var Undefined: Boolean);
  private
    FExpressionCompiler: TExpressionCompiler;
    FCompiledExpression: TCompiledExpression;
    ecX, ecY: double;
    stX, stY: TStFloat;
//    functionList : TFunctionStringList;
    { Private declarations }
  public
    ExpressionParser : TExpressionParser;
    epX, epY : double;
    CompiledTime : TDateTime;
    Expression : TExpression;
    XVar, YVar : TRealVariable;
    PascalScriptResult : double;
    Exec: TIFPSExec;
    SymbolExpr: Symbolic.TExpression;
    SymVars : TStringList;
    SymbolEval: Symbolic.TEvaluator;
    Vars    : Array[0..1] OF Symbolic.ArbFloat;


    procedure CompileScript(const Script: string);
    procedure ExecuteScript;
//    Variables : TStringList;
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses ParseClass, Math, StrUtils;


{$R *.dfm}

function LocalAbs(X: double): double;
begin
  result := Abs(X);
end;

procedure SetVariable(const Value : double);
begin
  Form1.PascalScriptResult := Value;
end;

Procedure GetValues(var X, Y : double);
begin
  X := Form1.epX;
  Y := Form1.epY
end;

function MySin(Value : double) : double;
begin
  result := System.Sin(Value);
end;

function MyCos(Value : double) : double;
begin
  result := System.Cos(Value);
end;

function MyTan(Value : double) : double;
begin
  result := Math.Tan(Value);
end;

function MyAbs(Value : double) : double;
begin
  result := System.Abs(Value);
end;

function ScriptOnUses(Sender: TIFPSPascalCompiler; const Name: string): Boolean;
{ the OnUses callback function is called for each "uses" in the script.
  It's always called with the parameter 'SYSTEM' at the top of the script. 
  For example: uses ii1, ii2;   
  This will call this function 3 times. First with 'SYSTEM' then 'II1' and then 'II2'.
}
begin
  if Name = 'SYSTEM' then
  begin
//    RegisterStandardLibrary_C(Sender);
    { Register the standard library. The standard library is in the 
      ifps3lib_std.pas unit. This will register all standard functions like 
      Copy, Pos, Length.
    }

    TPSPascalCompiler(Sender).AddFunction('procedure SetVariable(Value : double)');
    TPSPascalCompiler(Sender).AddFunction('Procedure GetValues(var X, Y : double)');
    TPSPascalCompiler(Sender).AddFunction('function Sin(Value : double) : double');
    TPSPascalCompiler(Sender).AddFunction('function Cos(Value : double) : double');
    TPSPascalCompiler(Sender).AddFunction('function Tan(Value : double) : double');
    TPSPascalCompiler(Sender).AddFunction('function Abs(Value : double) : double');

{    RegisterDelphiFunctionC(Sender, 'procedure SetVariable(Value : double)');
    RegisterDelphiFunctionC(Sender, 'Procedure GetValues(var X, Y : double)');
    RegisterDelphiFunctionC(Sender, 'function Sin(Value : double) : double');
    RegisterDelphiFunctionC(Sender, 'function Cos(Value : double) : double');
    RegisterDelphiFunctionC(Sender, 'function Tan(Value : double) : double');
    RegisterDelphiFunctionC(Sender, 'function Abs(Value : double) : double'); }
    { This will register the function to the script engine. Now it can be used from within the script. When adding
      functions always leave the const out, this is not yet supported but doesn't make a change. This function can be
      found in the ifpidelphi.pas file. }

    Result := True;
  end else
    Result := False;
end;

procedure TForm1.CompileScript(const Script: string);
var
  Compiler: TIFPSPascalCompiler;
  { TIFPSPascalCompiler is the compiler part of the scriptengine. This will 
    translate a Pascal script into a compiled for the executer understands. }
  Data : string;
begin
  Compiler := TIFPSPascalCompiler.Create; // create an instance of the compiler.
  try
    Compiler.OnUses := ScriptOnUses; // assign the OnUses event.
    if not Compiler.Compile(Script) then  // Compile the Pascal script into bytecode.
    begin
      raise Exception.create('Invalid script');
      Exit;
    end;

    Compiler.GetOutput(Data); // Save the output of the compiler in the string Data.
  finally
    Compiler.Free; // After compiling the script, there is no need for the compiler anymore.
  end;

  Exec.Free;
  Exec := nil;
  Exec := TIFPSExec.Create;  // Create an instance of the executer.

//  Exec.RegisterStandardLibrary_R(Exec);
  { The functions registered at compile time also need to be registered at runtime. These
    functions can be found in the ifps3lib_stdr.pas unit. }

  Exec.RegisterDelphiFunction(@SetVariable, 'SETVARIABLE', cdRegister);
  Exec.RegisterDelphiFunction(@GetValues, 'GETVALUES', cdRegister);
  Exec.RegisterDelphiFunction(@MySin, 'SIN', cdRegister);
  Exec.RegisterDelphiFunction(@MyCos, 'COS', cdRegister);
  Exec.RegisterDelphiFunction(@MyTan, 'TAN', cdRegister);
  Exec.RegisterDelphiFunction(@MyAbs, 'ABS', cdRegister);

{  RegisterDelphiFunctionR(Exec, @SetVariable, 'SETVARIABLE', cdRegister);
  RegisterDelphiFunctionR(Exec, @GetValues, 'GETVALUES', cdRegister);
  RegisterDelphiFunctionR(Exec, @MySin, 'SIN', cdRegister);
  RegisterDelphiFunctionR(Exec, @MyCos, 'COS', cdRegister);
  RegisterDelphiFunctionR(Exec, @MyTan, 'TAN', cdRegister);
  RegisterDelphiFunctionR(Exec, @MyAbs, 'ABS', cdRegister);  }
  { This will register the function to the executer. The first parameter is the executer. The second parameter is a
    pointer to the function. The third parameter is the name of the function (in uppercase). And the last parameter is the
    calling convention (usually Register). This function can be found in the ifpidelphiruntime.pas file. }

  if not Exec.LoadData(Data) {<> ENoError} then // Load the data from the Data string.
  begin
    { For some reason the script could not be loaded. This is usually the case when a
      library that has been used at compile time isn't registered at runtime. }
    raise Exception.Create('Error executing script');
     // You could raise an exception here.
    Exit;
  end;
end;

procedure TForm1.ExecuteScript;
begin
  Exec.RunScript; // Run the script.
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  X, Y : integer;
//  AnX, AY : double;
  ThisTime : TDateTime;
  sTime : string;
  AnInteger: integer;
  ThisValue: integer;
  //First, Sum: integer;
  StartCount, StopCount, Speed: TLargeInteger;
  Sum64: Int64;
begin
  Sum64 := 0;
  QueryPerformanceFrequency(Speed);
  AnInteger := 0;
  ThisTime := Now;
  case RadioGroup1.ItemIndex of
    0:  // compiled
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
            epX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              epY := Y/Height*3;
              QueryPerformanceCounter(StartCount);
              ThisValue := Trunc(9*255*255*abs((sin(5*epX)+cos(epY*3)+Tan(epX*2))/1.5));
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    1: // TPegtopFormula
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            PegtopFormula1.SetArg(0,ecX);
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              PegtopFormula1.SetArg(1,ecY);
              QueryPerformanceCounter(StartCount);

              ThisValue := Trunc(PegtopFormula1.Eval);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    2: // TParser
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
            Parser1.X := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              Parser1.Y := Y/Height*3;

              QueryPerformanceCounter(StartCount);
              ThisValue := Trunc(Parser1.Value);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    3: // TExpressionParser
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
            epX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              epY := Y/Height*3;

              QueryPerformanceCounter(StartCount);
              ThisValue := Trunc(ExpressionParser.EvaluateCurrent);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    4: // TExpressionCompiler
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              QueryPerformanceCounter(StartCount);


              ThisValue := Trunc(FCompiledExpression);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    5: // TRbwParser
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin

            XVar.Value := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              YVar.Value := Y/Height*3;

              QueryPerformanceCounter(StartCount);
              Expression.Evaluate;
              ThisValue := Trunc(Expression.DoubleResult);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    6: // Pascal Script
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            epX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              epY := Y/Height*3;
              QueryPerformanceCounter(StartCount);
              ExecuteScript;

              ThisValue := Trunc(PascalScriptResult);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    7: // TMathParser
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              QueryPerformanceCounter(StartCount);

              MathParser1.Parse;
              ThisValue := Trunc(MathParser1.ParseValue);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    8: // TStExpression
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            stX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              stY := Y/Height*3;
              QueryPerformanceCounter(StartCount);


              ThisValue := Trunc(StExpression1.AnalyzeExpression);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    9: // TExpressIt
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              QueryPerformanceCounter(StartCount);


              if not ExpressIt1.Execute then Beep;
              ThisValue := Trunc(VarAsType(ExpressIt1.Resultant, varDouble));
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    10: // TExpressionEvaluator
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              QueryPerformanceCounter(StartCount);


              ThisValue := Trunc(ExpressionEvaluator1.Value);
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    11: // Symbolic
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            Vars[0] := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              Vars[1]  := Y/Height*3;
              QueryPerformanceCounter(StartCount);


              ThisValue := Trunc(SymbolEval.Evaluate(Vars));
              QueryPerformanceCounter(StopCount);
              Sum64 := Sum64 + StopCount - StartCount;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end;
    (*8: // TCalcExpress
      begin
        With PaintBox1 do
        begin
          for X := 0 to Width -1 do
          begin
//            Application.ProcessMessages;
            ecX := X/Width*3;
            for Y := 0 to Height -1 do
            begin
              ecY := Y/Height*3;
              First := GetTickCount;



              ThisValue := Trunc(CalcExpress1.calc([ecX,ecY]));
              Sum := Sum + GetTickCount - First;
              if cbDraw.Checked then
              begin
                AnInteger := AnInteger + ThisValue;
                Canvas.Pen.Color := ThisValue;
                Canvas.Brush.Color := ThisValue;
                Canvas.Rectangle(X-1, Y-1, X+1, Y+1);
              end;
            end;
          end;
        end;
      end; *)
  end;
  ThisTime := Now - ThisTime;
//  sTime := FloatToStr(Sum/1000);
  sTime :=  FloatToStr(Sum64/Speed);
//  DateTimeToString(sTime, 'ss.zzz', );
  Edit1.Text := sTime;
  case RadioGroup1.itemIndex of
    0:
      begin
        CompiledTime := Sum64/Speed;
        Edit2.Text := '';
      end;
    else
      begin
        if CompiledTime <> 0 then
        begin
          Edit2.Text := Format('%.4g',[(Sum64/Speed-CompiledTime)/CompiledTime*100])+'%';
        end;
      end;
  end;
  Edit4.Text := IntToStr(Width*Height);
  Label5.Caption := intToStr(AnInteger);

end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  Paintbox1.Invalidate
end;

{
function _sin(Values : array of pointer) : double;
begin
  result := Sin(PDouble(Values[0])^);
end;

function _cos(Values : array of pointer) : double;
begin
  result := Cos(PDouble(Values[0])^);
end;

function _tan(Values : array of pointer) : double;
begin
  result := tan(PDouble(Values[0])^);
end;

function _abs(Values : array of pointer) : double;
begin
  result := Abs(PDouble(Values[0])^);
end;   }

procedure TForm1.FormCreate(Sender: TObject);
{var
    SinFunction : TFunctionRecord;
    CosFunction : TFunctionRecord;
    TanFunction : TFunctionRecord;
    AbsFunction : TFunctionRecord; }
var
  AString: string;

begin
  ExpressionParser := TExpressionParser.Create;
  ExpressionParser.DefineVariable('X',@epx);
  ExpressionParser.DefineVariable('Y',@epy);
  ExpressionParser.AddExpression(Parser1.Expression);
  Edit3.Text := Parser1.Expression;
  Constraints.MinWidth := Width;

  XVar := RbwParser1.CreateVariable('X',0.0);
  YVar := RbwParser1.CreateVariable('Y',0.0);

//  XVar := RbwParser1.Variables[RbwParser1.CreateVariable('X',0.0)] as TRealVariable;
//  YVar := RbwParser1.Variables[RbwParser1.CreateVariable('Y',0.0)] as TRealVariable;

  AString := Parser1.Expression;
  RbwParser1.Compile(AString);
  Expression := RbwParser1.CurrentExpression;

  FExpressionCompiler:= TExpressionCompiler.Create;
  FExpressionCompiler.AddFunc('Abs', MyAbs);
  FExpressionCompiler.AddFunc('Sin', MySin);
  FExpressionCompiler.AddFunc('Cos', MyCos);
  FExpressionCompiler.AddFunc('Tan', MyTan);
  FExpressionCompiler.AddVar('X', ecX);
  FExpressionCompiler.AddVar('Y', ecY);
  FCompiledExpression := FExpressionCompiler.Compile(AString);

  AString := Edit3.Text;
  AString := AnsiReplaceStr(LowerCase(AString), 'abs', 'absolute');
  ExpressIt1.Expression := AString;

  MathParser1.ParseString := Edit3.Text;
  MathParser1.Parse;

//  CalcExpress1.Formula := Parser1.Expression;

  // UseMathUnit is defined in Project Options.
  // That allows the "tan" function to be recognized.
  StExpression1.AddVariable('X', @stX);
  StExpression1.AddVariable('Y', @stY);

  StExpression1.Expression := Parser1.Expression;

  SymbolExpr := Symbolic.TExpression.Create(Parser1.Expression);
  SymVars := TStringList.Create;
  SymVars.Add('X');
  SymVars.Add('Y');
  SymbolEval := Symbolic.TEvaluator.Create(SymVars, SymbolExpr); 

  Button1Click(Sender);

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ExpressionParser.Free;
//  functionList.Free;
//  XVar.Free;
//  YVar.Free;
{  Variables.Free;
  Expression.Free;  }
    Exec.Free;
  FExpressionCompiler.Free;
  SymbolExpr.Free;
  SymbolEval.Free;

end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  PaintBox1.Invalidate
end;

procedure TForm1.Button1Click(Sender: TObject);
{var
  Tokens : TTokenStringList;  }
var
  Script : string;
  AString: string;
begin
  Parser1.Expression := Edit3.Text;
  AString := Edit3.Text;
  ExpressionParser.AddExpression(AString);
  RbwParser1.Compile(AString);
  Expression := RbwParser1.CurrentExpression;
  Label6.Caption := Expression.Decompile;

  Script :=
    'Program ParserTest; '
    + 'var '
      + 'X, Y : double; '
      + 'Color : double; '
    + 'begin '
      + 'GetValues(X, Y); '
      + 'Color := ' + Edit3.Text + '; '
      + 'SetVariable(Color); '
    + 'end.';
  CompileScript(Script);

  FCompiledExpression := FExpressionCompiler.Compile(Edit3.Text);

  AString := Edit3.Text;
  AString := AnsiReplaceStr(LowerCase(AString), 'abs', 'absolute');
  ExpressIt1.Expression := AString;
  MathParser1.ParseString := Edit3.Text;
  MathParser1.Parse;  

  PegtopFormula1.Parameters.Clear;
  PegtopFormula1.Parameters.Add('X');
  PegtopFormula1.Parameters.Add('Y');
  PegtopFormula1.Expression := Edit3.Text;
  PegtopFormula1.Compile;

  ExpressionEvaluator1.Expression := Edit3.Text;

//  CalcExpress1.Formula := Edit3.Text;
//  CalcExpress1.Variables.Add('X');
//  CalcExpress1.Variables.Add('Y');
  StExpression1.Expression := Edit3.Text;

  SymbolExpr.Free;
  SymbolEval.Free;
  SymVars := TStringList.Create;
  SymVars.Add('X');
  SymVars.Add('Y');
  SymbolExpr := Symbolic.TExpression.Create(Parser1.Expression);
  // SymbolEval takes ownership of SymVars.
  SymbolEval := Symbolic.TEvaluator.Create(SymVars, SymbolExpr); 

  PaintBox1.Invalidate;
end;

procedure TForm1.ExpressIt1LateValue(Sender: TObject;
  const aSymbol: String; var aValue: Variant);
begin
  if (aSymbol = 'X') or (aSymbol = 'x') then
  begin
    aValue := ecX;
  end
  else if (aSymbol = 'Y') or (aSymbol = 'y') then
  begin
    aValue := ecY;
  end;
end;

procedure TForm1.MathParser1GetVar(Sender: TObject; VarName: String;
  var Value: Extended; var Found: Boolean);
begin
  if (VarName = 'X') or (VarName = 'x') then
  begin
    Value := ecX;
    Found := True;
  end
  else if (VarName = 'Y') or (VarName = 'y') then
  begin
    Value := ecY;
    Found := True;
  end;
end;

procedure TForm1.MathParser1ParseError(Sender: TObject;
  ParseError: Integer);
begin
  Beep;
end;

procedure TForm1.cbDrawClick(Sender: TObject);
begin
  PaintBox1.Invalidate;
end;

procedure TForm1.ExpressionEvaluator1GetValue(Sender: TObject;
  Identifier: String; var Value: Extended; var Undefined: Boolean);
begin
  if UpperCase(Identifier) = 'X' then
  begin
    Value := ecX;
    Undefined := False;
  end
  else if UpperCase(Identifier) = 'Y' then
  begin
    Value := ecY;
    Undefined := False;
  end;
end;

end.
